//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/arith
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		arith.h
//
// purpose	Define resources used by and with the arith library.
//-----------------------------------------------------------------------------
#ifndef __ARITH_H__
#define __ARITH_H__

//-----------------------------------------------------------------------------
// configuration
//-----------------------------------------------------------------------------
// #define PIX_DEFAULT_GAMMA (45058994827146944512.0/18446744073709551616.0)
// #define PIX_DEFAULT_GAMMA (2775621817487087.0/1125899906842624.0)
// #define PIX_DEFAULT_GAMMA (0x9.c546d4400107f57p-2L)
#define PIX_DEFAULT_GAMMA (2.44265300408029158662180646555128760155639611184597015380859375L)

//-----------------------------------------------------------------------------
// macro	gcd
//
// purpose	Determine the greatest common denominator by Euclid's algorithm
//		for any integer type.
//
// arguments	1 (integer) 1st number
//		2 (integer) 2nd number
//
// returns	(integer) greatest common denominator
//-----------------------------------------------------------------------------
#define gcd(a,b) ({								\
	__typeof__((a)) libh__a = (a);						\
	__typeof__((b)) libh__b = (b);						\
	if ( libh__a < libh__b ) {						\
		__typeof__((a)) libh__c = libh__a;				\
		libh__a = libh__b;						\
		libh__b = libh__c;						\
	}									\
	while ( libh__b ) {							\
		__typeof__((a)) libh__c = libh__a % libh__b;			\
		libh__a = libh__b;						\
		libh__b = libh__c;						\
	}									\
	libh__a;								\
})

//-----------------------------------------------------------------------------
// macro	ipow
//
// purpose	Raise a number to an integer power and return the result.
//
// arguments	1 (number) number to raise
//		2 (int) power to raise
//
// returns	(number) power result
//-----------------------------------------------------------------------------
#define ipow(n,p) ({								\
	__typeof__(n)	libh__n;						\
	__typeof__(n)	libh__a;						\
	int		libh__p;						\
	int		libh__m;						\
	libh__n = (n);								\
	libh__p = (p);								\
	libh__a = ( __typeof__(n) ) 1;						\
	libh__m = 1;								\
	while ( libh__m <= libh__p ) {						\
		if ( libh__m & libh__p ) libh__a *= libh__n;			\
		libh__n *= libh__n;						\
		libh__m += libh__m;						\
	}									\
	libh__a;								\
})

//-----------------------------------------------------------------------------
// macro	pix_ftoi_base
//
// purpose	Convert a floating point (double) pixel intensity value in
//		the range 0.0 .. 1.0 to a gamma corrected integer (int) pixel
//		intensity value in the range 0 .. N-1.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) pixel intensity from 0.0 to 1.0
//		3 (int) integer scale N
//
// returns	(int) pixel intensity in the range 0 to N-1
//-----------------------------------------------------------------------------
#define pix_ftoi_base(g,v,s) ({							\
	double libh__v;								\
	double libh__g;								\
	int libh__s;								\
	libh__g = (g);								\
	libh__s = (s);								\
	( libh__s <= 1 ) ? 0 : ({						\
		libh__v = (v);							\
		( libh__v == 0.0 ) ? 0 : ({					\
			libh__v=(int)((exp(log(libh__v)/libh__g)*((double)(libh__s-1)))+0.5);	\
			( libh__v < 0 ) ? 0 : ( libh__v >= libh__s ) ? libh__s - 1 : libh__v;	\
		})								\
	})									\
})

//-----------------------------------------------------------------------------
// macro	pix_ftoi
//
// purpose	Convert a floating point (double) pixel intensity value in
//		the range 0.0 .. 1.0 to a gamma corrected integer (int) pixel
//		intensity value in the range 0 .. N-1.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) pixel intensity from 0.0 to 1.0
//		3 (int) integer scale N
//
// returns	(int) pixel intensity in the range 0 to N-1
//-----------------------------------------------------------------------------
#define pix_ftoi(g,v,s) ({							\
	double libh__g;								\
	libh__g = (g);								\
	if ( libh__g <= 0.0 ) libh__g = ( PIX_DEFAULT_GAMMA );			\
	pix_ftoi_base(libh__g,(v),(s));						\
})

//-----------------------------------------------------------------------------
// macro	pix_itof
//
// purpose	Convert an integer (int) pixel intensity value in the range
//		0 .. N-1 to a gamma adjusted floating point (double pixel
//		intensity value in the range 0.0 .. 1.0.
//
// arguments	1 (double) gamma correction factor to use
//		2 (int) pixel intensity from 0 to N-1
//		3 (int) integer scale N
//
// returns	(double) pixel intensity in the range 0.0 .. 1.0
//-----------------------------------------------------------------------------
#define pix_itof(g,v,s) ({							\
	int	libh__value;							\
	int	libh__scale;							\
	libh__value = (v);							\
	libh__scale = (s);							\
	( libh__scale <= 1 ) ? 0.0 :						\
	( libh__value >= libh__scale ) ? 1.0 :					\
	( libh__value <= 0 ) ? 0.0 :						\
	({	double	libh__gvtmp;						\
		libh__gvtmp = (g);						\
		if ( libh__gvtmp <= 0.0 ) libh__gvtmp = PIX_DEFAULT_GAMMA;	\
		libh__gvtmp = exp( log( (double) libh__value / (double) ( libh__scale - 1 ) ) * libh__gvtmp );	\
		( libh__gvtmp <= 0.0 ) ? 0.0 : ( libh__gvtmp >= 1.0 ) ? 1.0 : libh__gvtmp;			\
	})									\
})

//-----------------------------------------------------------------------------
// macro	pix_rgb10_ftoi
//
// purpose	Convert three floating point (double) pixel intensity values
//		in the range 0.0 .. 1.0 to a single gamma corrected color code
//		(int) value with the low order 30 bits representing the color
//		code in triple 10-bit format.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) pixel intensity for 1st color (usually red)
//		3 (double) pixel intensity for 2nd color (usually green)
//		4 (double) pixel intensity for 3rd color (usually blue)
//
// returns	(int) gamma corrected R10,G10,B10 color code
//-----------------------------------------------------------------------------
#define pix_rgb10_ftoi(m,r,g,b) ({						\
	double libh__m;								\
	libh__m = (m);								\
	if ( libh__m <= 0.0 ) libh__m = PIX_DEFAULT_GAMMA;			\
	( ( pix_ftoi_base( libh__m, (r), 1024 ) << 20 ) & 0x3ff00000 ) +	\
	( ( pix_ftoi_base( libh__m, (g), 1024 ) << 10 ) & 0x000ffc00 ) +	\
	( ( pix_ftoi_base( libh__m, (b), 1024 )       ) & 0x000003ff );		\
})

//-----------------------------------------------------------------------------
// macro	pix_rgb4_ftoi
//
// purpose	Convert three floating point (double) pixel intensity values
//		in the range 0.0 .. 1.0 to a single gamma corrected color code
//		(int) value with the low order 12 bits representing the color
//		code in triple 4-bit format.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) pixel intensity for 1st color (usually red)
//		3 (double) pixel intensity for 2nd color (usually green)
//		4 (double) pixel intensity for 3rd color (usually blue)
//
// returns	(int) gamma corrected R4,G4,B4 color code
//-----------------------------------------------------------------------------
#define pix_rgb4_ftoi(m,r,g,b) ({						\
	double libh__m;								\
	libh__m = (m);								\
	if ( libh__m <= 0.0 ) libh__m = PIX_DEFAULT_GAMMA;			\
	( ( pix_ftoi_base( libh__m, (r), 16 ) << 8 ) & 0xf00 ) +		\
	( ( pix_ftoi_base( libh__m, (g), 16 ) << 4 ) & 0x0f0 ) +		\
	( ( pix_ftoi_base( libh__m, (b), 16 )      ) & 0x00f );			\
})

//-----------------------------------------------------------------------------
// macro	pix_rgb5_ftoi
//
// purpose	Convert three floating point (double) pixel intensity values
//		in the range 0.0 .. 1.0 to a single gamma corrected color code
//		(int) value with the low order 15 bits representing the color
//		code in triple 5-bit format.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) pixel intensity for 1st color (usually red)
//		3 (double) pixel intensity for 2nd color (usually green)
//		4 (double) pixel intensity for 3rd color (usually blue)
//
// returns	(int) gamma corrected R5,G5,B5 color code
//-----------------------------------------------------------------------------
#define pix_rgb5_ftoi(m,r,g,b) ({						\
	double libh__m;								\
	libh__m = (m);								\
	if ( libh__m <= 0.0 ) libh__m = PIX_DEFAULT_GAMMA;			\
	( ( pix_ftoi_base( libh__m, (r), 32 ) << 10 ) & 0x7c00 ) +		\
	( ( pix_ftoi_base( libh__m, (g), 32 ) <<  5 ) & 0x03e0 ) +		\
	( ( pix_ftoi_base( libh__m, (b), 32 )       ) & 0x001f );		\
})

//-----------------------------------------------------------------------------
// macro	pix_rgb8_ftoi
//
// purpose	Convert three floating point (double) pixel intensity values
//		in the range 0.0 .. 1.0 to a single gamma corrected color code
//		(int) value with the low order 24 bits representing the color
//		code in triple 8-bit format.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) pixel intensity for 1st color (usually red)
//		3 (double) pixel intensity for 2nd color (usually green)
//		4 (double) pixel intensity for 3rd color (usually blue)
//
// returns	(int) gamma corrected R8,G8,B8 color code
//-----------------------------------------------------------------------------
#define pix_rgb8_ftoi(m,r,g,b) ({						\
	double libh__m;								\
	libh__m = (m);								\
	if ( libh__m <= 0.0 ) libh__m = PIX_DEFAULT_GAMMA;			\
	( ( pix_ftoi_base( libh__m, (r), 256 ) << 16 ) & 0xff0000 ) +		\
	( ( pix_ftoi_base( libh__m, (g), 256 ) <<  8 ) & 0x00ff00 ) +		\
	( ( pix_ftoi_base( libh__m, (b), 256 )       ) & 0x0000ff );		\
})

//-----------------------------------------------------------------------------
// macro	rescale
//
// purpose	Given a new linear value range and an old linear value range,
//		rescale a value from the old range to the new range and return
//		the result.
//
// arguments	1 (number) 1st value of new value range
//		2 (number) 2nd value of new value range
//		3 (number) 1st value of old value range
//		4 (number) 2nd value of old value range
//		5 (number) value to be rescaled
//
// yields	(number) value rescaled to new range
//
// note		Ranges can be reversed to invert the scaling.
//
// note		For reasonable results, all arguments should be floating point.
//		However, integers will work to some extent.
//
// note		Values to be rescaled which are outside the old range will be
//		extrapolated linearly outside of the new range.
//-----------------------------------------------------------------------------
#define rescale(na,nz,oa,oz,v) ({						\
	__typeof__(na)	libh__n;						\
	__typeof__(oa)	libh__o;						\
	__typeof__(v)	libh__v;						\
	libh__n = (na);								\
	libh__o = (oa);								\
	libh__v = ( (nz) - libh__n ) * ( (v) - libh__o );			\
	libh__o = (oz) - libh__o;						\
	( libh__v + ( libh__n * libh__o ) ) / libh__o;				\
})

//-----------------------------------------------------------------------------
// macro	rescale0
//
// purpose	Given a new linear value range and an old linear value range
//		which always begins with 0.0, rescale a value from the old
//		range to the new range and return the result.  With the first
//		value of the old range being 0.0, the calculation is slightly
//		less involved.
//
// arguments	1 (number) 1st value of new value range
//		2 (number) 2nd value of new value range
//		3 (number) 2nd value of old value range (1st is zero)
//		4 (number) value to be rescaled
//
// yields	(number) value rescaled to new range
//
// note		Ranges can be reversed to invert the scaling.
//
// note		For reasonable results, all arguments should be floating point.
//		However, integers will work to some extent.
//
// note		Values to be rescaled which are outside the old range will be
//		extrapolated linearly outside of the new range.
//-----------------------------------------------------------------------------
#define rescale0(na,nz,oz,v) ({							\
	__typeof__(na)	libh__n;						\
	__typeof__(oz)	libh__o;						\
	__typeof__(v)	libh__v;						\
	libh__n = (na);								\
	libh__v = ( (nz) - libh__n ) * (v);					\
	libh__o = (oz);								\
	( libh__v + ( libh__n * libh__o ) ) / libh__o;				\
})

//-----------------------------------------------------------------------------
// macro	rescale_dm
//
// purpose	Given a new linear number range and an old linear number range,
//		rescale multiple numbers from the old range to the new range,
//		storing back the results in the pointed to arguments given.
//
// arguments	1 (double) 1st value of new number range
//		2 (double) 2nd value of new number range
//		3 (double) 1st value of old number range
//		4 (double) 2nd value of old number range
//		5..N (double *) pointer to one or more values
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define rescale_dm(na,nz,oa,oz,a...) ((rescale_dm)((na),(nz),(oa),(oz),a,((double*)(NULL))))

//-----------------------------------------------------------------------------
// macro	rescale_ldm
//
// purpose	Given a new linear number range and an old linear number range,
//		rescale multiple numbers from the old range to the new range,
//		storing back the results in the pointed to arguments given.
//
// arguments	1 (long double) 1st value of new number range
//		2 (long double) 2nd value of new number range
//		3 (long double) 1st value of old number range
//		4 (long double) 2nd value of old number range
//		5..N (long double *) pointer to one or more values
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define rescale_ldm(na,nz,oa,oz,a...) ((rescale_ldm)((na),(nz),(oa),(oz),a,((double*)(NULL))))
//-----------------------------------------------------------------------------
// macro	rescale_mpf
//
// purpose	Given a new linear value range and an old linear value range,
//		rescale a value from the old range to the new range and return
//		the result.
//
// arguments	1 (number) result of rescale
//		2 (number) 1st value of new value range
//		3 (number) 2nd value of new value range
//		4 (number) 1st value of old value range
//		5 (number) 2nd value of old value range
//		6 (number) value to be rescaled
//
// yields	(number) value rescaled to new range
//
// note		Ranges can be reversed to invert the scaling.
//
// note		For reasonable results, all arguments should be floating point.
//		However, integers will work to some extent.
//
// note		Values to be rescaled which are outside the old range will be
//		extrapolated linearly outside of the new range.
//-----------------------------------------------------------------------------
#define rescale_mpf(r,na,nz,oa,oz,v) ({						\
	mpf_t libh__n;								\
	mpf_t libh__o;								\
	mpf_t libh__v;								\
	mpf_t libh__1;								\
	mpf_t libh__2;								\
	mpf_init( libh__v );							\
	mpf_init( libh__1 );							\
	mpf_init( libh__2 );							\
	mpf_init_set( libh__n, (na) );						\
	mpf_init_set( libh__o, (oa) );						\
	mpf_sub( libh__1, (v), libh__o );					\
	mpf_sub( libh__2, (nz), libh__n );					\
	mpf_mul( libh__v, libh__2, libh__1 );					\
	mpf_sub( libh__o, (oz), libh__o );					\
	mpf_mul( libh__1, libh__n, libh__o );					\
	mpf_add( libh__2, libh__v, libh__1 );					\
	mpf_div( (r), libh__2, libh__o );					\
	mpf_clear( libh__2 );							\
	mpf_clear( libh__1 );							\
	mpf_clear( libh__v );							\
	mpf_clear( libh__o );							\
	mpf_clear( libh__n );							\
})
#define rescale_mpf_reference(na,nz,oa,oz,v) ({					\
	__typeof__(na)	libh__n;						\
	__typeof__(oa)	libh__o;						\
	__typeof__(v)	libh__v;						\
	libh__n = (na);								\
	libh__o = (oa);								\
	libh__v = ( (nz) - libh__n ) * ( (v) - libh__o );			\
	libh__o = (oz) - libh__o;						\
	( libh__v + ( libh__n * libh__o ) ) / libh__o;				\
})

//-----------------------------------------------------------------------------
// macro	rescale0_mpf
//
// purpose	Given a new linear value range and an old linear value range
//		which always begins with 0.0, rescale a value from the old
//		range to the new range and return the result.  With the first
//		value of the old range being 0.0, the calculation is slightly
//		less involved.
//
// arguments	1 (number) 1st value of new value range
//		2 (number) 2nd value of new value range
//		3 (number) 2nd value of old value range (1st is zero)
//		4 (number) value to be rescaled
//
// yields	(number) value rescaled to new range
//
// note		Ranges can be reversed to invert the scaling.
//
// note		For reasonable results, all arguments should be floating point.
//		However, integers will work to some extent.
//
// note		Values to be rescaled which are outside the old range will be
//		extrapolated linearly outside of the new range.
//-----------------------------------------------------------------------------
#define rescale0_mpf(r,na,nz,oz,v) ({						\
	mpf_t libh__n;								\
	mpf_t libh__o;								\
	mpf_t libh__v;								\
	mpf_t libh__1;								\
	mpf_t libh__2;								\
	mpf_init( libh__v );							\
	mpf_init( libh__1 );							\
	mpf_init( libh__2 );							\
	mpf_init_set( libh__n, (na) );						\
	mpf_sub( libh__1, (nz), libh__n );					\
	mpf_mul( libh__v, libh__1, (v) );					\
	mpf_init_set( libh__o, (oz) );						\
	mpf_mul( libh__1, libh__n, libh__o );					\
	mpf_add( libh__2, libh__v, libh__1 );					\
	mpf_div( (r), libh__2, libh__o );					\
	mpf_clear( libh__2 );							\
	mpf_clear( libh__1 );							\
	mpf_clear( libh__v );							\
	mpf_clear( libh__o );							\
	mpf_clear( libh__n );							\
})
#define rescale0_mpf_reference(na,nz,oz,v) ({					\
	__typeof__(na)	libh__n;						\
	__typeof__(oz)	libh__o;						\
	__typeof__(v)	libh__v;						\
	libh__n = (na);								\
	libh__v = ( (nz) - libh__n ) * (v);					\
	libh__o = (oz);								\
	( libh__v + ( libh__n * libh__o ) ) / libh__o;				\
})

//-----------------------------------------------------------------------------
// macro	zoom_calc
//
// purpose	Given a set of multiply/add values from a zoom scan, and a set
//		of 2D view coordinates, transform the coordinates into the new
//		zoomed in view.
//
// arguments	1 (lvalue) edge position on X axis on left
//		2 (lvalue) edge position on Y axis on top
//		3 (lvalue) edge position on X axis on right
//		4 (lvalue) edge position on Y axis on bottom
//		5 (value) already scanned X axis multiplier
//		6 (value) already scanned Y axis multiplier
//		7 (value) already scanned X axis adder
//		8 (value) already scanned Y axis adder
//
// returns	(int) == -1 : failure
//		(int) >=  0 : success
//-----------------------------------------------------------------------------
#define zoom_calc(xl,yt,xr,yb,xm,ym,xa,ya) ({					\
	{									\
		typeof(xr) libh__d;						\
		typeof(xr) libh__m;						\
		libh__d = ((xr)-(xl))*(xm)/2.0;					\
		libh__m = ((xr)+(xl))/2.0+(xa);					\
		(xl) = libh__m - libh__d;					\
		(xr) = libh__m + libh__d;					\
	}									\
	{									\
		typeof(yt) libh__d;						\
		typeof(yt) libh__m;						\
		libh__d = ((yt)-(yb))*(ym)/2.0;					\
		libh__m = ((yt)+(yb))/2.0+(ya);					\
		(yb) = libh__m - libh__d;					\
		(yt) = libh__m + libh__d;					\
	}									\
	0;									\
})

//-----------------------------------------------------------------------------
// function	gcd_ui
//
// purpose	Determine the greatest common denominator by Euclid's algorithm
//		for type unsigned int and return it.
//
// arguments	1 (unsigned int) 1st number
//		2 (unsigned int) 2nd number
//
// returns	(unsigned int) greatest common denominator
//-----------------------------------------------------------------------------
unsigned int
gcd_ui (
    unsigned int	arg_num1
    ,
    unsigned int	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	gcd_ul
//
// purpose	Determine the greatest common denominator by Euclid's algorithm
//		for type unsigned long and return it.
//
// arguments	1 (unsigned long) 1st number
//		2 (unsigned long) 2nd number
//
// returns	(unsigned long) greatest common denominator
//-----------------------------------------------------------------------------
unsigned long
gcd_ul (
    unsigned long	arg_num1
    ,
    unsigned long	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	gcd_ull
//
// purpose	Determine the greatest common denominator by Euclid's algorithm
//		for type unsigned long long and return it.
//
// arguments	1 (unsigned long long) 1st number
//		2 (unsigned long long) 2nd number
//
// returns	(unsigned long long) greatest common denominator
//-----------------------------------------------------------------------------
unsigned long long
gcd_ull (
    unsigned long long	arg_num1
    ,
    unsigned long long	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	ipow_d
//
// purpose	Raise a number to a power and return the result.  This is for
//		type double.
//
// arguments	1 (double) number to raise
//		2 (unsigned int) power to raise
//
// returns	(double) power result
//-----------------------------------------------------------------------------
double
ipow_d (
    double		arg_num1
    ,
    unsigned int	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	ipow_ld
//
// purpose	Raise a number to a power and return the result.  This is for
//		type long double.
//
// arguments	1 (long double) number to raise
//		2 (unsigned int) power to raise
//
// returns	(long double) power result
//-----------------------------------------------------------------------------
long double
ipow_ld (
    long double		arg_num1
    ,
    unsigned int	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	ipow_sl
//
// purpose	Raise a number to a power and return the result.  This is for
//		type signed long.
//
// arguments	1 (signed long) number to raise
//		2 (unsigned int) power to raise
//
// returns	(unsigned long) power result
//-----------------------------------------------------------------------------
signed long
ipow_sl (
    signed long		arg_num1
    ,
    unsigned int	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	ipow_sll
//
// purpose	Raise a number to a power and return the result.  This is for
//		type signed long long.
//
// arguments	1 (signed long long) number to raise
//		2 (unsigned int) power to raise
//
// returns	(signed long long) power result
//-----------------------------------------------------------------------------
signed long long
ipow_sll (
    signed long long	arg_num1
    ,
    unsigned int	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	ipow_ul
//
// purpose	Raise a number to a power and return the result.  This is for
//		type unsigned long.
//
// arguments	1 (unsigned long) number to raise
//		2 (unsigned int) power to raise
//
// returns	(unsigned long) power result
//-----------------------------------------------------------------------------
unsigned long
ipow_ul (
    unsigned long	arg_num1
    ,
    unsigned int	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	ipow_ull
//
// purpose	Raise a number to a power and return the result.  This is for
//		type unsigned long long.
//
// arguments	1 (unsigned long long) number to raise
//		2 (unsigned int) power to raise
//
// returns	(unsigned long long) power result
//-----------------------------------------------------------------------------
unsigned long long
ipow_ull (
    unsigned long long	arg_num1
    ,
    unsigned int	arg_num2
    )
;

//-----------------------------------------------------------------------------
// function	rescale_d
//
// purpose	Given a new linear number range and an old linear number range,
//		rescale a number from the old range to the new range and return
//		the result.
//
// arguments	1 (double) 1st value of new number range
//		2 (double) 2nd value of new number range
//		3 (double) 1st value of old number range
//		4 (double) 2nd value of old number range
//		5 (double) number in old range to rescale to new range
//
// returns	(double) number rescaled to new range
//-----------------------------------------------------------------------------
double
rescale_d (
    double		arg_new_a
    ,
    double		arg_new_z
    ,
    double		arg_old_a
    ,
    double		arg_old_z
    ,
    double		arg_num
    )
;

//-----------------------------------------------------------------------------
// function	(rescale_dm)
//
// purpose	Given a new linear number range and an old linear number range,
//		rescale multiple numbers from the old range to the new range,
//		storing back the results in the pointed to arguments given.
//
// arguments	1 (double) 1st value of new number range
//		2 (double) 2nd value of new number range
//		3 (double) 1st value of old number range
//		4 (double) 2nd value of old number range
//		5..N (double *) pointer to one or more values
//		N+1 (double *) NULL to indicate end of list
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
(rescale_dm) (
    double		arg_new_a
    ,
    double		arg_new_z
    ,
    double		arg_old_a
    ,
    double		arg_old_z
    ,
    double *		arg_value_p
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	rescale_ld
//
// purpose	Given a new linear number range and an old linear number range,
//		rescale a number from the old range to the new range and return
//		the result.
//
// arguments	1 (long double) 1st value of new number range
//		2 (long double) 2nd value of new number range
//		3 (long double) 1st value of old number range
//		4 (long double) 2nd value of old number range
//		5 (long double) number in old range to rescale to new range
//
// returns	(long double) number rescaled to new range
//-----------------------------------------------------------------------------
long double
rescale_ld (
    long double		arg_new_a
    ,
    long double		arg_new_z
    ,
    long double		arg_old_a
    ,
    long double		arg_old_z
    ,
    long double		arg_num
    )
;

//-----------------------------------------------------------------------------
// function	(rescale_ldm)
//
// purpose	Given a new linear number range and an old linear number range,
//		rescale multiple numbers from the old range to the new range,
//		storing back the results in the pointed to arguments given.
//
// arguments	1 (long double) 1st value of new number range
//		2 (long double) 2nd value of new number range
//		3 (long double) 1st value of old number range
//		4 (long double) 2nd value of old number range
//		5..N (long double *) pointer to one or more values
//		N+1 (long double *) NULL to indicate end of list
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
(rescale_ldm) (
    long double		arg_new_a
    ,
    long double		arg_new_z
    ,
    long double		arg_old_a
    ,
    long double		arg_old_z
    ,
    long double *	arg_value_p
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	zoom_calc_d
//
// purpose	Given a set of multiply/add values from a zoom scan, and a set
//		of 2D view coordinates, transform the coordinates into the new
//		zoomed in view.
//
// arguments	1 (double *) edge position on X axis on left
//		2 (double *) edge position on Y axis on top
//		3 (double *) edge position on X axis on right
//		4 (double *) edge position on Y axis on bottom
//		5 (double) already scanned X axis multiplier
//		6 (double) already scanned Y axis multiplier
//		7 (double) already scanned X axis adder
//		8 (double) already scanned Y axis adder
//
// returns	(int) == -1 : failure
//		(int) >=  0 : success
//-----------------------------------------------------------------------------
int
zoom_calc_d (
    double *		arg_xl_p
    ,
    double *		arg_yt_p
    ,
    double *		arg_xr_p
    ,
    double *		arg_yb_p
    ,
    double		arg_mul_x
    ,
    double		arg_mul_y
    ,
    double		arg_add_x
    ,
    double		arg_add_y
    )
;

//-----------------------------------------------------------------------------
// function	zoom_calc_ld
//
// purpose	Given a set of multiply/add values from a zoom scan, and a set
//		of 2D view coordinates, transform the coordinates into the new
//		zoomed in view.
//
// arguments	1 (long double *) edge position on X axis on left
//		2 (long double *) edge position on Y axis on top
//		3 (long double *) edge position on X axis on right
//		4 (long double *) edge position on Y axis on bottom
//		5 (long double) already scanned X axis multiplier
//		6 (long double) already scanned Y axis multiplier
//		7 (long double) already scanned X axis adder
//		8 (long double) already scanned Y axis adder
//
// returns	(int) == -1 : failure
//		(int) >=  0 : success
//-----------------------------------------------------------------------------
int
zoom_calc_ld (
    long double *	arg_xl_p
    ,
    long double *	arg_yt_p
    ,
    long double *	arg_xr_p
    ,
    long double *	arg_yb_p
    ,
    long double		arg_mul_x
    ,
    long double		arg_mul_y
    ,
    long double		arg_add_x
    ,
    long double		arg_add_y
    )
;

//-----------------------------------------------------------------------------
// function	zoom_scan_d
//
// purpose	Scan a string of zooming command characters and produce a set
//		of values to multiply and add to transform 2D coordinates to
//		given a zoomed and positioned view.
//
// arguments	1 (const char *) zooming/positioning command string
//		2 (double *) where to store X multiplier
//		3 (double *) where to store Y multiplier
//		4 (double *) where to store X adder
//		5 (double *) where to store Y adder
//
// returns	(int) == -1 : general failure
//		(int) >=  0 : number of successful commands
//
// note		If an unrecognized command is found, command interpretation
//		stops and the returned number represents an index into the
//		command sequence string to the unrecognized character.  The
//		stored results only apply to the last successful command.
//
// commands	0 o O	- zoom out (unzoom) by 2x
//		1	- zoom in by 2x to the upper left quadrant
//		2	- zoom in by 2x to the upper center quadrant
//		3	- zoom in by 2x to the upper fright quadrant
//		4	- zoom in by 2x to the middle left quadrant
//		5 i I	- zoom in by 2x to the middle center quadrant
//		6	- zoom in by 2x to the middle right quadrant
//		7	- zoom in by 2x to the lower left quadrant
//		8	- zoom in by 2x to the lower center quadrant
//		9	- zoom in by 2x to the lower right quadrant
//		x X	- zoom only X axis by 2x
//		y Y	- zoom only Y axis by 2x
//		l L	- shift by 1/2 width leftward
//		u U	- shift by 1/2 height upward
//		r R	- shift by 1/2 width rightward
//		d D	- shift by 1/2 height downward
//-----------------------------------------------------------------------------
int
zoom_scan_d (
    const char *	arg_cmd_str
    ,
    double *		arg_mul_x_p
    ,
    double *		arg_mul_y_p
    ,
    double *		arg_add_x_p
    ,
    double *		arg_add_y_p
    )
;

//-----------------------------------------------------------------------------
// function	zoom_scan_ld
//
// purpose	Scan a string of zooming command characters and produce a set
//		of values to multiply and add to transform 2D coordinates to
//		given a zoomed and positioned view.
//
// arguments	1 (const char *) zooming/positioning command string
//		2 (long double *) where to store X multiplier
//		3 (long double *) where to store Y multiplier
//		4 (long double *) where to store X adder
//		5 (long double *) where to store Y adder
//
// returns	(int) == -1 : general failure
//		(int) >=  0 : number of successful commands
//
// note		If an unrecognized command is found, command interpretation
//		stops and the returned number represents an index into the
//		command sequence string to the unrecognized character.  The
//		stored results only apply to the last successful command.
//
// commands	0 o O	- zoom out (unzoom) by 2x
//		1	- zoom in by 2x to the upper left quadrant
//		2	- zoom in by 2x to the upper center quadrant
//		3	- zoom in by 2x to the upper fright quadrant
//		4	- zoom in by 2x to the middle left quadrant
//		5 i I	- zoom in by 2x to the middle center quadrant
//		6	- zoom in by 2x to the middle right quadrant
//		7	- zoom in by 2x to the lower left quadrant
//		8	- zoom in by 2x to the lower center quadrant
//		9	- zoom in by 2x to the lower right quadrant
//		x X	- zoom only X axis by 2x
//		y Y	- zoom only Y axis by 2x
//		l L	- shift by 1/2 width leftward
//		u U	- shift by 1/2 height upward
//		r R	- shift by 1/2 width rightward
//		d D	- shift by 1/2 height downward
//-----------------------------------------------------------------------------
int
zoom_scan_ld (
    const char *	arg_cmd_str
    ,
    long double *	arg_mul_x_p
    ,
    long double *	arg_mul_y_p
    ,
    long double *	arg_add_x_p
    ,
    long double *	arg_add_y_p
    )
;

#endif /* __ARITH_H__ */
