//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/callsign
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		callsign.h
//-----------------------------------------------------------------------------
#ifndef __CALLSIGN_H__
#define __CALLSIGN_H__

#include <stdio.h>
//-----------------------------------------------------------------------------
// struct	country_call
//-----------------------------------------------------------------------------
struct country_call {
    const char *	code			;
    const char *	call_first		;
    const char *	call_last		;
};

//-----------------------------------------------------------------------------
// struct	country_name
//-----------------------------------------------------------------------------
struct country_name {
    const char *	code			;
    const char *	name_sorting		;
    const char *	name_short		;
    const char *	name_long		;
};

//-----------------------------------------------------------------------------
// macro	callsign_to_country_code
//
// purpose	Given a radio callsign, return the country code identifying
//		the country the callsign is in the range for.
//
// argument	1 (const char *) callsign string
//
// returns	(const char *) NULL : ambiguous (e.g. a callsign like "a")
//		(const char *) "-" : unassigned or reserved callsign range
//		(const char *) country code assigned the callsign range
//
// note		Do not modify the returned string.
//-----------------------------------------------------------------------------
#define callsign_to_country_code(c) ({						\
	const struct country_call * __country_p;				\
	__country_p = country_by_call((c));					\
	__country_p->code;							\
})

//-----------------------------------------------------------------------------
// macro	callsign_to_country_name_long
//
// purpose	Given a radio callsign, return the country long name for
//		the country the callsign is in the range for.
//
// argument	1 (const char *) callsign string
//
// returns	(const char *) NULL : ambiguous (e.g. a callsign like "a")
//		(const char *) long name of country for this callsign range
//
// note		Do not modify the returned string.
//-----------------------------------------------------------------------------
#define callsign_to_country_name_long(c) ({					\
	const struct country_call * __country_p;				\
	__country_p = country_by_call((c));					\
	__country_p ? country_to_name_long( __country_p->code ) : NULL;		\
})

//-----------------------------------------------------------------------------
// macro	callsign_to_country_name_short
//
// purpose	Given a radio callsign, return the country short name for
//		the country the callsign is in the range for.
//
// argument	1 (const char *) callsign string
//
// returns	(const char *) NULL : ambiguous (e.g. a callsign like "a")
//		(const char *) short name of country for this callsign range
//
// note		Do not modify the returned string.
//-----------------------------------------------------------------------------
#define callsign_to_country_name_short(c) ({					\
	const struct country_call * __country_p;				\
	__country_p = country_by_call((c));					\
	__country_p ? country_to_name_short( __country_p->code ) : NULL;	\
})

//-----------------------------------------------------------------------------
// macro	callsign_to_country_name_sorting
//
// purpose	Given a radio callsign, return the country sorting name for
//		the country the callsign is in the range for.
//
// argument	1 (const char *) callsign string
//
// returns	(const char *) NULL : ambiguous (e.g. a callsign like "a")
//		(const char *) sorting name of country for this callsign range
//
// note		Do not modify the returned string.
//-----------------------------------------------------------------------------
#define callsign_to_country_name_sorting(c) ({					\
	const struct country_call * __country_p;				\
	__country_p = country_by_call((c));					\
	__country_p ? country_to_name_sorting( __country_p->code ) : NULL;	\
})

//-----------------------------------------------------------------------------
// macro	country_to_name_long
//
// purpose	Given a country code (or a special code for international
//		administrative units), return a pointer to the long name
//		of that country (or international administrative unit).
//
// argument	1 (const char *) country code string
//
// returns	(const char *) NULL : unknown code
//		(const char *) country long name string
//
// note		Do not modify the returned string.
//-----------------------------------------------------------------------------
#define country_to_name_long(c) ({						\
	struct country_name * __country_entry_p;				\
	__country_entry_p = country_to_name_entry( ( c ) );			\
	( __country_entry_p ) ? __country_entry_p->name_long : NULL;		\
})

//-----------------------------------------------------------------------------
// macro	country_to_name_short
//
// purpose	Given a country code (or a special code for international
//		administrative units), return a pointer to the short name
//		of that country (or international administrative unit).
//
// argument	1 (const char *) country code string
//
// returns	(const char *) NULL : unknown code
//		(const char *) country short name string
//
// note		Do not modify the returned string.
//-----------------------------------------------------------------------------
#define country_to_name_short(c) ({						\
	struct country_name * __country_entry_p;				\
	__country_entry_p = country_to_name_entry( ( c ) );			\
	( __country_entry_p ) ? __country_entry_p->name_short : NULL;		\
})

//-----------------------------------------------------------------------------
// macro	country_to_name_sorting
//
// purpose	Given a country code (or a special code for international
//		administrative units), return a pointer to the sorting name
//		of that country (or international administrative unit).
//
// argument	1 (const char *) country code string
//
// returns	(const char *) NULL : unknown code
//		(const char *) country sorting name string
//
// note		Do not modify the returned string.
//-----------------------------------------------------------------------------
#define country_to_name_sorting(c) ({						\
	struct country_name * __country_entry_p;				\
	__country_entry_p = country_to_name_entry( ( c ) );			\
	( __country_entry_p ) ? __country_entry_p->name_sorting : NULL;		\
})

//-----------------------------------------------------------------------------
// function	country_call_table
//
// purpose	Return a pointer to the callsign country table.
//
// arguments	-none-
//
// returns	(const struct country_call *) pointer to the callsign country
//		table
//
// note		A future version of this function will read the callsign to
//		country mapping table in from a shared resource file instead
//		of having it hard coded here.
//-----------------------------------------------------------------------------
const struct country_call *
country_call_table ()
;

//-----------------------------------------------------------------------------
// function	country_call_table_count
//
// purpose	Return a count of the number of entries in the callsign
//		country table.
//
// arguments	-none-
//
// returns	(int) count of the number of entries in the table
//-----------------------------------------------------------------------------
int
country_call_table_count ()
;

//-----------------------------------------------------------------------------
// function	country_by_call
//
// purpose	Given a radio callsign, return the country code and callsign
//		range it fits in.
//
// argument	1 (const char *) callsign string
//
// returns	(const struct country_call *) NULL : error, range not found
//		(const struct country_call *) pointer to country call entry
//
// note		Do not modify the referenced strings (const char *).
//-----------------------------------------------------------------------------
const struct country_call *
country_by_call (
    const char *	arg_call_p
    )
;

//-----------------------------------------------------------------------------
// function	country_name_table
//
// purpose	Return a pointer to the country code to name table.
//
// arguments	-none-
//
// returns	(struct country_name *) pointer to country code to name table
//
// note		A future version of this function will read the country name
//		mapping table in from a shared resource file instead of having
//		it hard coded here.
//-----------------------------------------------------------------------------
struct country_name *
country_name_table ()
;

//-----------------------------------------------------------------------------
// function	country_to_name_entry
//
// purpose	Given a country code (or a special code for international
//		administrative units), store a pointer to the entry of type
//		struct country_name which contains the short name, long name,
//		and sorting name for that country.
//
// argument	1 (const char *) country code string
//
// returns	(struct country_name *) NULL : country code not found
//		(struct country_name *) pointer to country entry
//
// note		Do not modify the strings pointed to.
//-----------------------------------------------------------------------------
struct country_name *
country_to_name_entry (
    const char *	arg_country_p
    )
;

#endif /* __CALLSIGN_H__ */
