//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
//-----------------------------------------------------------------------------
// package	libh/cgi
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		cgi.h
//
// purpose	Define resources used by and with the cgi library.
//-----------------------------------------------------------------------------
#ifndef __CGI_H__
#define __CGI_H__
#include <libh/map.h>
//-----------------------------------------------------------------------------
// function	cgi_form_parse_finish
//
// purpose	Finish the web page form parsing and release any resources
//		that might still be held, like the buffers for query string
//		and post content.
//
// arguments	-none-
//
// returns	(int) -1 = indicates an error occurred
//		(int)  0 = indicates no more variables available
//-----------------------------------------------------------------------------
#define cgi_form_parse_finish() cgi_form_parse((NULL),(NULL),(NULL),(NULL))

//-----------------------------------------------------------------------------
// function	cgi_form_fetch_done
//
// purpose	Release resources after all need for CGI form data is done.
//
// arguments	-none-
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
cgi_form_fetch_done ()
;

//-----------------------------------------------------------------------------
// function	cgi_form_fetch_init
//
// purpose	Initialize use of cgi_form_fetch functions.
//
// arguments	-none-
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
cgi_form_fetch_init ()
;

//-----------------------------------------------------------------------------
// function	cgi_form_fetch_post_content
//
// purpose	Fetch web page form variables ONLY from the post content,
//		in a CGI conforming web server environment (such as Apache).
//
// arguments	1 (const char *) variable name requested
//		2 (const char *) return this if variable is absent
//		3 (size_t *) where to store length in case value is binary
//
// returns	(const char *) pointer to value string, or NULL
//
// note		Use of cgi_form_fetch group functions is mutually exclusive
//		with use of cgi_form_map() or cgi_form_parse() functions.
//
// note		If the browser passes the same variable name multiple times,
//		only one of the values will be returned.  Usually it will be
//		the first one, but not always.  For access to multiple
//		variable instances, use cgi_form_parse().
//-----------------------------------------------------------------------------
const char *
cgi_form_fetch_post_content (
    const char *	arg_name_str
    ,
    const char *	arg_default_absent
    ,
    size_t *		arg_value_len_ptr
    )
;

//-----------------------------------------------------------------------------
// function	cgi_form_fetch_post_or_query
//
// purpose	Fetch web page form variables from the post content, or if
//		not present in the post content, then from the query string,
//		in a CGI conforming web server environment (such as Apache).
//
// arguments	1 (const char *) variable name requested
//		2 (const char *) return this if variable is absent
//		3 (size_t *) where to store length in case value is binary
//
// returns	(const char *) pointer to value string, or NULL
//
// note		Use of cgi_form_fetch group functions is mutually exclusive
//		with use of cgi_form_map() or cgi_form_parse() functions.
//
// note		If the browser passes the same variable name multiple times,
//		only one of the values will be returned.  Usually it will be
//		the first one, but not always.  For access to multiple
//		variable instances, use cgi_form_parse().
//-----------------------------------------------------------------------------
const char *
cgi_form_fetch_post_or_query (
    const char *	arg_name_str
    ,
    const char *	arg_default_absent
    ,
    size_t *		arg_value_len_ptr
    )
;

//-----------------------------------------------------------------------------
// function	cgi_form_fetch_query_or_post
//
// purpose	Fetch web page form variables from the query string, or if
//		not present in the query string, then from the post content,
//		in a CGI conforming web server environment (such as Apache).
//
// arguments	1 (const char *) variable name requested
//		2 (const char *) return this if variable is absent
//		3 (size_t *) where to store length in case value is binary
//
// returns	(const char *) pointer to value string, or NULL
//
// note		Use of cgi_form_fetch group functions is mutually exclusive
//		with use of cgi_form_map() or cgi_form_parse() functions.
//
// note		If the browser passes the same variable name multiple times,
//		only one of the values will be returned.  Usually it will be
//		the first one, but not always.  For access to multiple
//		variable instances, use cgi_form_parse().
//-----------------------------------------------------------------------------
const char *
cgi_form_fetch_query_or_post (
    const char *	arg_name_str
    ,
    const char *	arg_default_absent
    ,
    size_t *		arg_value_len_ptr
    )
;

//-----------------------------------------------------------------------------
// function	cgi_form_fetch_query_string
//
// purpose	Fetch web page form variables ONLY from the query string,
//		in a CGI conforming web server environment (such as Apache).
//
// arguments	1 (const char *) variable name requested
//		2 (const char *) return this if variable is absent
//		3 (size_t *) where to store length in case value is binary
//
// returns	(const char *) pointer to value string, or NULL
//
// note		Use of cgi_form_fetch group functions is mutually exclusive
//		with use of cgi_form_map() or cgi_form_parse() functions.
//
// note		If the browser passes the same variable name multiple times,
//		only one of the values will be returned.  Usually it will be
//		the first one, but not always.  For access to multiple
//		variable instances, use cgi_form_parse().
//-----------------------------------------------------------------------------
const char *
cgi_form_fetch_query_string (
    const char *	arg_name_str
    ,
    const char *	arg_default_absent
    ,
    size_t *		arg_value_len_ptr
    )
;

//-----------------------------------------------------------------------------
// function	cgi_form_map
//
// purpose	Fill mappings from web page form variables from either query
//		string or post request method or both, in a CGI conforming web
//		server environment (such as Apache).
//
// usage	Call cgi_form_map() to fill one or two mappings from web form
//		variables, then use the map_str_*() functions to access those
//		variables including multiple instances of the same variable
//		name.  When the mapping is no longer needed, its memory can
//		be recovered by calling map_destroy() for each map.
//
// arguments	1 (MAP) mapping to fill in with variables from query string
//		2 (MAP) mapping to fill in with variables from post content
//
// returns	(int) >=  0 : count of total variables from both sources
//		(int)  < -1 : error
//
// note		If cgi_form_map is used, do not use cgi_form_parse or any of
//		the cgi_form_fetch functions.
//
// note		If the browser passes the same variable name multiple times,
//		only one of the values will be returned.  Usually it will be
//		the first one, but not always.  For access to multiple
//		variable instances, do not use cgi_form_fetch(), but instead
//		use either cgi_form_map() in conjunction with the map library
//		functions, or use cgi_form_parse() in a loop to pick up all
//		the variables in sequence.
//
// note		No information is available to identify whether the variable
//		came from the query string or the post content.  If that
//		information is needed, use cgi_form_parse() directly instead.
//-----------------------------------------------------------------------------
int
cgi_form_map (
    MAP			arg_query_string_map
    ,
    MAP			arg_post_content_map
    )
;

//-----------------------------------------------------------------------------
// function	cgi_form_parse
//
// purpose	Get web page form variables from either query string or post
//		request method (MIME type application/x-www-form-urlencoded
//		only) or both, in a CGI conforming web server environment
//		(such as Apache).
//
//		Support for posted MIME type multipart/mixed and and uploaded
//		files (which use multipart/mixed) is NOT provided by this
//		function.
//
// usage	To get each form variable, call the cgi_form_parse() function
//		giving it pointers to where to store the following:
//			1.  where to store a pointer to the name string
//			2.  where to store the length of the name string
//			3.  where to store a pointer to the value string
//			4.  where to store the length of the value string
//		The return value indicates whether the variable is from the
//		query string (if environment variable QUERY_STRING is set
//		by the web server) or the posted content (if POST METHOD is
//		indicated in the environment variable REQUEST_METHOD and a
//		CONTENT_LENGTH is properly set by the web server).
//
//		The value returned by the function will be one of:
//			-1  indicates an error occurred
//			 0  indicates no more variables available
//			 1  indicates the variable is from the query string
//			 2  indicates the variable is from the post content
//
// arguments	1 (char * *) where to store pointer to name pointer
//		2 (size_t *) where to store pointer to name length
//		3 (char * *) where to store pointer to value pointer
//		4 (size_t *) where to store pointer to value length
//
// returns	(int) -1 = indicates an error occurred
//		(int)  0 = indicates no more variables available
//		(int)  1 = indicates the variable is from the query string
//		(int)  2 = indicates the variable is from the post content
//
// note		The use of cgi_form_map() and cgi_form_parse() in the same
//		program does not work.  if cgi_form_map() is used, it must
//		be the only caller of cgi_form_parse().
//
// note		Once a variable has been given to the caller, that variable
//		will not be given to the caller again.  Only one pass of the
//		variables is possible.  This is because the POST content
//		stream cannot be replayed and cgi_form_parse() does not keep
//		them.
//
// note		If the browser passes the same variable name multiple times,
//		which is valid for forms, then cgi_form_parse() will return
//		each of those variables with their values in the order the
//		browser sent them (usually in the same order they are in the
//		form).
//
// note		The same variable name may occur in both the query string as
//		as well as the post content.  Each will be returned separately
//		with the return value indicating which source the variable
//		came from.
//-----------------------------------------------------------------------------
int
cgi_form_parse (
    char * *	arg_name_str_ptr
    ,
    size_t *	arg_name_len_ptr
    ,
    char * *	arg_value_str_ptr
    ,
    size_t *	arg_value_len_ptr
    )
;

//-----------------------------------------------------------------------------
// function	cgi_is_nph
//
// purpose	Return true (1) if CGI is running in Non-Parsed-Header (NPH)
//		mode under Apache or an equivalently compatible web server.
//
// arguments	-none-
//
// returns	(int) -1 : error (maybe not CGI at all)
//		(int)  0 : not NPH mode
//		(int)  1 : NPH mode
//-----------------------------------------------------------------------------
int
cgi_is_nph ()
;

#endif /* __CGI_H__ */
