//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/ctools
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		ctools.h
//-----------------------------------------------------------------------------
#ifndef __CTOOLS_H__
#define __CTOOLS_H__

#include <stdio.h>
#include <limits.h>

//-----------------------------------------------------------------------------
// macros	decimal_max_signed_bits
//		decimal_max_signed_bits_big
//
// purpose	Yield the maximum number of bits that are supported by the
//		macros that calculate the number of digits, characters, or
//		memory requirements.
//-----------------------------------------------------------------------------
#define decimal_max_signed_bits		(uint_max(unsigned long)/55267UL)
#define decimal_max_signed_bits_big	(uint_max(unsigned long long)/3774669ULL)

//-----------------------------------------------------------------------------
// macros	decimal_digits_for_*
//
// purpose	Given a number of bits, calculate the number of decimal digits
//		that the largest possible number (known as a Mersenne Number)
//		that can be stored in that many bits, would have.
//
// limitation	In a 32 bit architecture, these macros will calculate the
//		number of decimal digits exactly for integers with up to 77713
//		bits for unsigned or 77714 bits for signed.  If the number of
//		bits could possibly exceed this limit, use the macros ending
//		with _big instead, which force 64 bit calculation, support up
//		to 4886983222558 bits for unsigned or 4886983222559 bits for
//		signed, and have fewer overestimates.
//
//		In a 64 bit architecture, these macros will calculate the
//		number of decimal digits exactly for integers with up to
//		333775020784728 bits for unsigned or 333775020784729 for
//		signed.  The macros ending with _big will also work with the
//		same limitations as in a 32 bit architecture.
//
// note		You would need much more than 32 terabytes of RAM to be able
//		to exceed the limitations of the _big versions.
//-----------------------------------------------------------------------------
#define decimal_digits_for_type(t)						\
	((((sizeof((t))*(CHAR_BIT))-int_is_signed((t)))*55267UL)/183593UL)+1UL)

#define decimal_digits_for_signed_bits(b)					\
	((((((unsigned long)(b))-1UL)*55267UL)/183593UL)+1UL)
#define decimal_digits_for_unsigned_bits(b)					\
	(((((unsigned long)(b))*55267UL)/183593UL)+1UL)

#define decimal_digits_for_signed_bits_big(b)					\
	((((((unsigned long long)(b))-1ULL)*3774669ULL)/12539179ULL)+1ULL)
#define decimal_digits_for_unsigned_bits_big(b)					\
	(((((unsigned long long)(b))*3774669ULL)/12539179ULL)+1ULL)

#define decimal_digits_for_signed_char						\
	((((CHAR_BIT)-1U)*55267UL)/183593UL)+1U)
#define decimal_digits_for_unsigned_char					\
	((((CHAR_BIT)*55267UL)/183593UL)+1U)
#define decimal_digits_for_signed_short						\
	((((sizeof(signed short)*(CHAR_BIT))-1U)*55267UL)/183593UL)+1U)
#define decimal_digits_for_unsigned_short					\
	((((sizeof(unsigned short)*(CHAR_BIT))*55267UL)/183593UL)+1U)
#define decimal_digits_for_signed_int						\
	((((sizeof(signed int)*(CHAR_BIT))-1U)*55267UL)/183593UL)+1U)
#define decimal_digits_for_unsigned_int						\
	((((sizeof(unsigned int)*(CHAR_BIT))*55267UL)/183593UL)+1U)
#define decimal_digits_for_signed_long						\
	((((sizeof(signed long)*(CHAR_BIT))-1U)*55267UL)/183593UL)+1U)
#define decimal_digits_for_unsigned_long					\
	((((sizeof(unsigned long)*(CHAR_BIT))*55267UL)/183593UL)+1U)
#define decimal_digits_for_signed_long_long					\
	((((sizeof(signed long long)*(CHAR_BIT))-1U)*55267UL)/183593UL)+1U)
#define decimal_digits_for_unsigned_long_long					\
	((((sizeof(unsigned long long)*(CHAR_BIT))*55267UL)/183593UL)+1U)

//-----------------------------------------------------------------------------
// macros	decimal_chars_for_*
//
// purpose	Given a number of bits, calculate the number of characters,
//		accounting for a possible sign character for signed types,
//		that the largest possible number (known as a Mersenne Number),
//		or also smallest possible number for signed types, that can be
//		stored in that many bits, would have.
//
// limitation	In a 32 bit architecture, these macros will calculate the
//		number of characters exactly for integers with up to 77713
//		bits for unsigned or 77714 bits for signed.  If the number of
//		bits could possibly exceed this limit, use the macros ending
//		with _big instead, which force 64 bit calculation, support up
//		to 4886983222558 bits for unsigned or 4886983222559 bits for
//		signed, and have fewer overestimates.
//
//		In a 64 bit architecture, these macros will calculate the
//		number of characters exactly for integers with up to
//		333775020784728 bits for unsigned or 333775020784729 for
//		signed.  The macros ending with _big will also work with the
//		same limitations as in a 32 bit architecture.
//
// note		You would need much more than 32 terabytes of RAM to be able
//		to exceed the limitations of the _big versions.
//-----------------------------------------------------------------------------
#define decimal_chars_for_type(t)						\
	((((sizeof((t))*(CHAR_BIT))-int_is_signed((t)))*55267UL)/183593UL)+int_is_signed((t))+1UL)

#define decimal_chars_for_signed_bits(b)					\
	((((((unsigned long)(b))-1UL)*55267UL)/183593UL)+2UL)
#define decimal_chars_for_unsigned_bits(b)					\
	(((((unsigned long)(b))*55267UL)/183593UL)+1UL)

#define decimal_chars_for_signed_bits_big(b)					\
	((((((unsigned long long)(b))-1ULL)*3774669ULL)/12539179ULL)+2ULL)
#define decimal_chars_for_unsigned_bits_big(b)					\
	(((((unsigned long long)(b))*3774669ULL)/12539179ULL)+1ULL)

#define decimal_chars_for_signed_char						\
	((((CHAR_BIT)-1U)*55267UL)/183593UL)+2U)
#define decimal_chars_for_unsigned_char						\
	((((CHAR_BIT)*55267UL)/183593UL)+1U)
#define decimal_chars_for_signed_short						\
	((((sizeof(signed short)*(CHAR_BIT))-1U)*55267UL)/183593UL)+2U)
#define decimal_chars_for_unsigned_short					\
	((((sizeof(unsigned short)*(CHAR_BIT))*55267UL)/183593UL)+1U)
#define decimal_chars_for_signed_int						\
	((((sizeof(signed int)*(CHAR_BIT))-1U)*55267UL)/183593UL)+2U)
#define decimal_chars_for_unsigned_int						\
	((((sizeof(unsigned int)*(CHAR_BIT))*55267UL)/183593UL)+1U)
#define decimal_chars_for_signed_long						\
	((((sizeof(signed long)*(CHAR_BIT))-1U)*55267UL)/183593UL)+2U)
#define decimal_chars_for_unsigned_long						\
	((((sizeof(unsigned long)*(CHAR_BIT))*55267UL)/183593UL)+1U)
#define decimal_chars_for_signed_long_long					\
	((((sizeof(signed long long)*(CHAR_BIT))-1U)*55267UL)/183593UL)+2U)
#define decimal_chars_for_unsigned_long_long					\
	((((sizeof(unsigned long long)*(CHAR_BIT))*55267UL)/183593UL)+1U)

//-----------------------------------------------------------------------------
// macros	decimal_space_for_*
//
// purpose	Given a number of bits, calculate the amount of memory space,
//		accounting for a possible sign character for signed types and
//		the null terminating character, that the largest possible
//		number (known as a Mersenne Number), or also smallest possible
//		number for signed types, that can be stored in that many bits,
//		would have.
//
// limitation	In a 32 bit architecture, these macros will calculate the
//		number of memory spaces exactly for integers with up to 77713
//		bits for unsigned or 77714 bits for signed.  If the number of
//		bits could possibly exceed this limit, use the macros ending
//		with _big instead, which force 64 bit calculation, support up
//		to 4886983222558 bits for unsigned or 4886983222559 bits for
//		signed, and have fewer overestimates.
//
//		In a 64 bit architecture, these macros will calculate the
//		number of memory spaces exactly for integers with up to
//		333775020784728 bits for unsigned or 333775020784729 for
//		signed.  The macros ending with _big will also work with the
//		same limitations as in a 32 bit architecture.
//
// note		You would need much more than 32 terabytes of RAM to be able
//		to exceed the limitations of the _big versions.
//-----------------------------------------------------------------------------
#define decimal_space_for_type(t)						\
	((((sizeof((t))*(CHAR_BIT))-int_is_signed((t)))*55267UL)/183593UL)+int_is_signed((t))+2UL)

#define decimal_space_for_signed_bits(b)					\
	((((((unsigned long)(b))-1UL)*55267UL)/183593UL)+3UL)
#define decimal_space_for_unsigned_bits(b)					\
	(((((unsigned long)(b))*55267UL)/183593UL)+2UL)

#define decimal_space_for_signed_bits_big(b)					\
	((((((unsigned long long)(b))-1ULL)*3774669ULL)/12539179ULL)+3ULL)
#define decimal_space_for_unsigned_bits_big(b)					\
	(((((unsigned long long)(b))*3774669ULL)/12539179ULL)+2ULL)

#define decimal_space_for_signed_char						\
	((((CHAR_BIT)-1U)*55267UL)/183593UL)+3U)
#define decimal_space_for_unsigned_char						\
	((((CHAR_BIT)*55267UL)/183593UL)+2U)
#define decimal_space_for_signed_short						\
	((((sizeof(signed short)*(CHAR_BIT))-1U)*55267UL)/183593UL)+3U)
#define decimal_space_for_unsigned_short					\
	((((sizeof(unsigned short)*(CHAR_BIT))*55267UL)/183593UL)+2U)
#define decimal_space_for_signed_int						\
	((((sizeof(signed int)*(CHAR_BIT))-1U)*55267UL)/183593UL)+3U)
#define decimal_space_for_unsigned_int						\
	((((sizeof(unsigned int)*(CHAR_BIT))*55267UL)/183593UL)+2U)
#define decimal_space_for_signed_long						\
	((((sizeof(signed long)*(CHAR_BIT))-1U)*55267UL)/183593UL)+3U)
#define decimal_space_for_unsigned_long						\
	((((sizeof(unsigned long)*(CHAR_BIT))*55267UL)/183593UL)+2U)
#define decimal_space_for_signed_long_long					\
	((((sizeof(signed long long)*(CHAR_BIT))-1U)*55267UL)/183593UL)+3U)
#define decimal_space_for_unsigned_long_long					\
	((((sizeof(unsigned long long)*(CHAR_BIT))*55267UL)/183593UL)+2U)

//-----------------------------------------------------------------------------
// macro	env_d
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type double, or use the default if not found.
//
// arguments	1 (double) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(double) the converted or default value.
//-----------------------------------------------------------------------------
#define env_d(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? strtod( libh__str, NULL )					\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_d
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type double, or use the default
//		if not found.
//
// arguments	1 (double) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(double) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_d(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_d)( pow, libh__str, NULL )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_ld
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type long double, or use the
//		default if not found.
//
// arguments	1 (long double) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(long double) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_ld(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_ld)( powl, libh__str, NULL )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_si
// alias	env_expr_i
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type signed int, or use the
//		default if not found.
//
// arguments	1 (signed int) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(signed int) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_i env_expr_si
#define env_expr_si(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_si)( libh__str, NULL, 10 )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_sl
// alias	env_expr_l
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type signed long, or use the
//		default if not found.
//
// arguments	1 (signed long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(signed long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_l env_expr_sl
#define env_expr_sl(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_sl)( libh__str, NULL, 10 )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_sll
// alias	env_expr_ll
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type signed long long, or use the
//		default if not found.
//
// arguments	1 (signed long long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(signed long long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_ll env_expr_sll
#define env_expr_sll(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_sll)( libh__str, NULL, 10 )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_ui
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type unsigned int, or use the
//		default if not found.
//
// arguments	1 (unsigned int) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(unsigned int) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_ui(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_ui)( libh__str, NULL, 10 )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_ul
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type unsigned long, or use the
//		default if not found.
//
// arguments	1 (unsigned long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(unsigned long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_ul(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_ul)( libh__str, NULL, 10 )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_expr_ull
//
// purpose	Select the first defined environment variable from a given list
//		and convert its expression to type unsigned long long, or use
//		the default if not found.
//
// arguments	1 (unsigned long long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(unsigned long long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_expr_ull(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (str_expr_to_ull)( libh__str, NULL, 10 )			\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// macro	env_ld
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type long double, or use the default if not
//		found.
//
// arguments	1 (long double) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(long double) the converted or default value.
//-----------------------------------------------------------------------------
#define env_ld(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? strtold( libh__str, NULL )					\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_si
// alias	env_i
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type signed int, or use the default if not
//		found.
//
// arguments	1 (signed int) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(signed int) the converted or default value.
//-----------------------------------------------------------------------------
#define env_i env_si
#define env_si(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (strtol)( libh__str, NULL, 10 )				\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_sl
// alias	env_l
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type signed long, or use the default if not
//		found.
//
// arguments	1 (signed long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(signed long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_l env_sl
#define env_sl(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (strtol)( libh__str, NULL, 10 )				\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_sll
// alias	env_ll
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type signed long long, or use the default if
//		not found.
//
// arguments	1 (signed long long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(signed long long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_ll env_sll
#define env_sll(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (strtoll)( libh__str, NULL, 10 )				\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// macro	env_str_dup
//
// purpose	Select the first defined environment variable from a given list
//		and return a pointer to an allocated copy of the value.
//
// arguments	1 (const char *) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(char *) pointer to duplicated value string
//-----------------------------------------------------------------------------
#define env_str_dup(d,n...) ({							\
	const char * libh__str;							\
	char * libh__new;							\
	size_t libh__len;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	if ( ! libh__str || ! * libh__str ) libh__str = ( d );			\
	libh__len = strlen( libh__str ) + 1;					\
	libh__new = malloc( libh__len );					\
	memcpy( libh__new, libh__str, libh__len );				\
	libh__new;								\
})

//-----------------------------------------------------------------------------
// macro	env_str_dupa
//
// purpose	Select the first defined environment variable from a given list
//		and return a pointer to an allocated copy of the value using
//		the alloca() function in the context of the function that uses
//		this macro.
//
// arguments	1 (const char *) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(char *) pointer to duplicated value string
//
// WARNING	All the restrictions for alloca() apply to env_str_dupa().
//-----------------------------------------------------------------------------
#define env_str_dupa(d,n...) ({							\
	const char * libh__str;							\
	char * libh__new;							\
	size_t libh__len;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	if ( ! libh__str || ! * libh__str ) libh__str = ( d );			\
	libh__len = strlen( libh__str ) + 1;					\
	libh__new = alloca( libh__len );					\
	memcpy( libh__new, libh__str, libh__len );				\
	libh__new;								\
})

//-----------------------------------------------------------------------------
// macro	env_str_ptr
//
// purpose	Select the first defined environment variable from a given list
//		and return a pointer to its value found in the environment.
//
// arguments	1 (const char *) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(const char *) pointer to value in the environment
//-----------------------------------------------------------------------------
#define env_str_ptr(d,n...) ({							\
	const char * libh__str;							\
	char * libh__new;							\
	size_t libh__len;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? libh__str							\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_ui
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type unsigned int, or use the default if not
//		found.
//
// arguments	1 (unsigned int) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(unsigned int) the converted or default value.
//-----------------------------------------------------------------------------
#define env_ui(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (strtoul)( libh__str, NULL, 10 )				\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_ul
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type unsigned long, or use the default if not
//		found.
//
// arguments	1 (unsigned long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(unsigned long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_ul(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (strtoul)( libh__str, NULL, 10 )				\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// function	env_ull
//
// purpose	Select the first defined environment variable from a given list
//		and convert it to type unsigned long long, or use the default if
//		not found.
//
// arguments	1 (unsigned long long) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(unsigned long long) the converted or default value.
//-----------------------------------------------------------------------------
#define env_ull(d,n...) ({							\
	const char * libh__str;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	( libh__str && * libh__str )						\
		? (strtoull)( libh__str, NULL, 10 )				\
		: ( d );							\
})

//-----------------------------------------------------------------------------
// macro	for_each_in
//
// purpose	Iterate a loop for each element in a NULL terminated array of
//		of pointers to some data.
//
// arguments	1 (name) variable name to be defined and points to each data
//		2 (name) name of or pointer to array of pointers
//
// yields	A for loop construct to be followed by code body
//-----------------------------------------------------------------------------
#define for_each_in(n,l) for(__typeof__(*(l))n,*__for_each_in_p=(l);(n=*__for_each_in_p);++__for_each_in_p)

//-----------------------------------------------------------------------------
// macro	for_each_of
//
// purpose	Iterate a loop for each of the given non-NULL pointers.
//
// arguments	1 (name) variable name to be defined and points to each string
//		2..N (const char *) strings to be iterated over
//
// yields	A for loop construct to be followed by code body
//
// example	for_each_of ( p, "foo", "bar" ) puts( p );
//-----------------------------------------------------------------------------
#define for_each_of(v,s...) for(const char*v,**__for_each_of_p=({static const char*__for_each_of_pp[]={s,(NULL)};__for_each_of_pp;});(v=*__for_each_of_p);++__for_each_of_p)

//-----------------------------------------------------------------------------
// macro	foreach
//
// purpose	Iterate a loop for each element in a NULL terminated array of
//		pointers to some data.
//
// arguments	1 (name) variable name to be defined and points to each data
//		2 (name) name of or pointer to array of pointers
//
// yields	A for loop construct to be followed by code body
//-----------------------------------------------------------------------------
#define foreach(n,l) for(__typeof__(*(l))n,*libh__foreach_p=(l);(n=*libh__foreach_p);++libh__foreach_p)

//-----------------------------------------------------------------------------
// macro	foreachstr
//
// purpose	Iterate a loop for each of the given strings.
//
// arguments	1 (name) variable name to be defined and points to each string
//		2..N (const char *) strings to be iterated over
//
// yields	A for loop construct to be followed by code body
//
// example	foreachstr ( p, "one", "two", "three", "four" ) puts( p );
//-----------------------------------------------------------------------------
#define foreachstr(v,s...) for(const char*v,**libh__foreachstr_p=({static const char*libh__foreachstr_pp[]={s,(NULL)};libh__foreachstr_pp;});(v=*libh__foreachstr_p);++libh__foreachstr_p)

//-----------------------------------------------------------------------------
// macro        getenv_or_default
//
// purpose      Get an environment variable (similar to getenv()) except that
//              if the variable is not defined, use a supplied default instead.
//
// argument     1 (const char *) environment variable name
//              2 (const char *) default value to use
//
// returns      (const char *) environment variable value or default value
//-----------------------------------------------------------------------------
#define getenv_or_default(n,d) ({						\
	const char *libh__str;							\
	libh__str=getenv((n));							\
	if(!libh__str)libh__str=(d);						\
	libh__str;								\
})

//-----------------------------------------------------------------------------
// macro	ifnot
//
// purpose	Invert the logic of if, for use in statements.
//
// arguments	1 (int) condition
//
// usage	As replacement for if statements.
//-----------------------------------------------------------------------------
#define ifnot(c) if(!(c))

//-----------------------------------------------------------------------------
// macro	int_is_signed
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to 1 (true) if the given integer type is signed, else
//		0 (false).
//
// arguments	1 (type) integer type
//
// yields	(int) 0: type is unsigned
//		(int) 1: type is signed
//-----------------------------------------------------------------------------
#define int_is_signed(t) ( ( ( (t)( ((t)0) - ((t)1) ) ) < ((t)0) ) ? 1 : 0 )

//-----------------------------------------------------------------------------
// macro	int_is_unsigned
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to 1 (true) if the given integer type is unsigned, else
//		0 (false).
//
// arguments	1 (type) integer type
//
// yields	(int) 0: type is signed
//		(int) 1: type is unsigned
//-----------------------------------------------------------------------------
#define int_is_unsigned(t) ( ( ( (t)( ((t)0) - ((t)1) ) ) < ((t)0) ) ? 0 : 1 )

//-----------------------------------------------------------------------------
// macro	int_max
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the maximum value of a given integer type, either
//		signed or unsigned.
//
// arguments	1 (type) integer type
//
// yields	(type) maximum value
//-----------------------------------------------------------------------------
#define int_max(t) ( (t) ~ int_min(t) )

//-----------------------------------------------------------------------------
// macro	int_min
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the minimum value of a given integer type, either
//		signed or unsigned.
//
// arguments	1 (type) integer type
//
// yields	(type) minimum value
//-----------------------------------------------------------------------------
#define int_min(t) ( int_is_signed(t) ? sint_min(t) : uint_min(t) )

//-----------------------------------------------------------------------------
// macro	limit_high
//
// purpose	Limit the given value to a specified range.  If the value is
//		greater than the high limit, return the high limit itself.
//		Otherwise return the actual value.
//
// arguments	1 (scalar) value to be limited
//		2 (scalar) high limit
//
// returns	(scalar) the limited value
//-----------------------------------------------------------------------------
#define limit_high(v,h) ({							\
	__typeof__( ( v ) ) __v;						\
	__typeof__( ( h ) ) __h;						\
	__v = ( v );								\
	__h = ( h );								\
	__v > __h ? __h : __v; })

//-----------------------------------------------------------------------------
// macro	limit_low
//
// purpose	Limit the given value to a specified range.  If the value is
//		lower than the low limit, return the low limit itself.
//		Otherwise return the actual value.
//
// arguments	1 (scalar) value to be limited
//		2 (scalar) low limit
//
// returns	(scalar) the limited value
//-----------------------------------------------------------------------------
#define limit_low(v,l) ({							\
	__typeof__( ( v ) ) __v;						\
	__typeof__( ( l ) ) __l;						\
	__v = ( v );								\
	__l = ( l );								\
	__v < __l ? __l : __v; })

//-----------------------------------------------------------------------------
// macro	limit_range
//
// purpose	Limit the given value to a specified range.  If the value is
//		lower than the low limit, return the low limit itself.  If the
//		value is greater than the high limit, return the high limit
//		itself.  Otherwise return the actual value.
//
// arguments	1 (scalar) value to be limited
//		2 (scalar) low limit
//		3 (scalar) high limit
//
// returns	(scalar) the limited value
//-----------------------------------------------------------------------------
#define limit_range(v,l,h) ({							\
	__typeof__( ( v ) ) __v;						\
	__typeof__( ( l ) ) __l;						\
	__typeof__( ( h ) ) __h;						\
	__v = ( v );								\
	__l = ( l );								\
	__h = ( h );								\
	__v < __l ? __l : __v > __h ? __h : __v; })

//-----------------------------------------------------------------------------
// macro	libh_max
//
// purpose	Select the maximum of two given like type values.
//
// arguments	1 (numeric) 1st value
//		2 (numeric) 2nd value
//
// returns	(numeric) maximum value
//-----------------------------------------------------------------------------
#define libh_max(a,b) ({							\
	__typeof__(a) libh__a;							\
	__typeof__(b) libh__b;							\
	(void) ( & libh__a == & libh__b );					\
	libh__a = (a);								\
	libh__b = (b);								\
	libh__a > libh__b ? libh__a : libh__b;					\
})

#ifndef LIBH_NO_max
//-----------------------------------------------------------------------------
// macro	max
//
// purpose	Select the maximum of two given like type values.
//
// arguments	1 (numeric) 1st value
//		2 (numeric) 2nd value
//
// returns	(numeric) maximum value
//-----------------------------------------------------------------------------
#define max(a,b) ({								\
	__typeof__(a) libh__a;							\
	__typeof__(b) libh__b;							\
	(void) ( & libh__a == & libh__b );					\
	libh__a = (a);								\
	libh__b = (b);								\
	libh__a > libh__b ? libh__a : libh__b;					\
})
#endif

//-----------------------------------------------------------------------------
// macro	libh_min
//
// purpose	Select the minimum of two given like type values.
//
// arguments	1 (numeric) 1st value
//		2 (numeric) 2nd value
//
// returns	(numeric) minimum value
//-----------------------------------------------------------------------------
#define libh_min(a,b) ({							\
	__typeof__(a) libh__a;							\
	__typeof__(b) libh__b;							\
	(void) ( & libh__a == & libh__b );					\
	libh__a = (a);								\
	libh__b = (b);								\
	libh__a < libh__b ? libh__a : libh__b;					\
})

#ifndef LIBH_NO_min
//-----------------------------------------------------------------------------
// macro	min
//
// purpose	Select the minimum of two given like type values.
//
// arguments	1 (numeric) 1st value
//		2 (numeric) 2nd value
//
// returns	(numeric) minimum value
//-----------------------------------------------------------------------------
#define min(a,b) ({								\
	__typeof__(a) libh__a;							\
	__typeof__(b) libh__b;							\
	(void) ( & libh__a == & libh__b );					\
	libh__a = (a);								\
	libh__b = (b);								\
	libh__a < libh__b ? libh__a : libh__b;					\
})
#endif

//-----------------------------------------------------------------------------
// macro	return_error
//
// purpose	Provide convenient coding for a frequent case of setting the
//		the errno variable to a particular value and returning a value
//		set in the macro THIS_ERROR_CODE.
//
// arguments	1 (int) value to set errno to
//
// results	Returns to caller with a return value of THIS_ERROR_CODE.
//-----------------------------------------------------------------------------
#define return_error(e) return (((errno)=(e)),(THIS_ERROR_CODE))

//-----------------------------------------------------------------------------
// macro	return_error_value
//
// purpose	Provide convenient coding for a frequent case of setting the
//		the errno variable to a particular value and returning the
//		given value.
//
// arguments	1 (int) value to set errno to
//		2 (undefined) value to yield
//
// results	Returns to caller with a specified return value.
//-----------------------------------------------------------------------------
#define return_error_value(e,v) return (((errno)=(e)),(v))

//-----------------------------------------------------------------------------
// macro	sint_max
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the maximum value of a given signed integer type.
//
// arguments	1 (type) signed integer type
//
// yields	(type) maximum value
//-----------------------------------------------------------------------------
#define sint_max(t) ( (t) ( ~ ( (t) ( ( (t) 1 ) << ( type_bits(t) - 1 ) ) ) ) )

//-----------------------------------------------------------------------------
// macro	sint_min
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the minimum value of a given signed integer type.
//
// arguments	1 (type) signed integer type
//
// yields	(type) minimum value
//-----------------------------------------------------------------------------
#define sint_min(t) ( (t) ( ( (t) 1 ) << ( type_bits(t) - 1 ) ) )

//-----------------------------------------------------------------------------
// macro	libh_swap
//
// purpose	Swap the values of two variables or structs of like type.
//		Both must be given as lvalues the & operator can be used on.
//
// arguments	1 (lvalue variable)
//		2 (lvalue variable)
//
// returns	(void)
//
// note		This is not optimized for large structs.
//-----------------------------------------------------------------------------
#define libh_swap(a,b) (void)({							\
	__typeof__(a) *libh__a;							\
	__typeof__(b) *libh__b;							\
	__typeof__(a) libh__t;							\
	(void) ( libh__a == libh__b );						\
	libh__a=&(a);								\
	libh__b=&(b);								\
	libh__t=*libh__a;							\
	*libh__a=*libh__b;							\
	*libh__b=libh__t;							\
})

#ifndef LIBH_NO_swap
//-----------------------------------------------------------------------------
// macro	swap
//
// purpose	Swap the values of two variables or structs of like type.
//		Both must be given as lvalues the & operator can be used on.
//
// arguments	1 (lvalue variable)
//		2 (lvalue variable)
//
// returns	(void)
//
// note		This is not optimized for large structs.
//-----------------------------------------------------------------------------
#define swap(a,b) (void)({							\
	__typeof__(a) *libh__a;							\
	__typeof__(b) *libh__b;							\
	__typeof__(a) libh__t;							\
	(void) ( libh__a == libh__b );						\
	libh__a=&(a);								\
	libh__b=&(b);								\
	libh__t=*libh__a;							\
	*libh__a=*libh__b;							\
	*libh__b=libh__t;							\
})
#endif

//-----------------------------------------------------------------------------
// macro	this_error
//
// purpose	Provide convenient coding for a frequent case of setting the
//		the errno variable to a particular value and returning a value
//		set in the macro THIS_ERROR_CODE.
//
// arguments	1 (int) value to set errno to
//
// yields	(undefined) THIS_ERROR_CODE
//-----------------------------------------------------------------------------
#define this_error(e) (((errno)=(e)),(THIS_ERROR_CODE))

//-----------------------------------------------------------------------------
// macro	this_error_value
//
// purpose	Provide convenient coding for a frequent case of setting the
//		the errno variable to a particular value and returning the
//		given value.
//
// arguments	1 (int) value to set errno to
//		2 (undefined) value to yield
//
// yields	(undefined) value from argument 2
//-----------------------------------------------------------------------------
#define this_error_value(e,v) (((errno)=(e)),(v))

//-----------------------------------------------------------------------------
// macro	type_bits
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the number of bits in a given type.
//
// arguments	1 (type) type
//
// yields	(int) number of bits
//-----------------------------------------------------------------------------
#define type_bits(t) ( sizeof(t) * (CHAR_BIT) )

//-----------------------------------------------------------------------------
// macro	uint_max
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the maximum value of a given unsigned integer type.
//
// arguments	1 (type) unsigned integer type
//
// yields	(type) maximum value
//-----------------------------------------------------------------------------
#define uint_max(t) ( (t) ~ ( (t) 0 ) )

//-----------------------------------------------------------------------------
// macro	uint_min
//
// purpose	Yield a constant expression that can be evaluated at compile
//		time to the minimum value of a given unsigned integer type.
//
// arguments	1 (type) unsigned integer type
//
// yields	(type) minimum value
//-----------------------------------------------------------------------------
#define uint_min(t) ( (t) 0 )

//-----------------------------------------------------------------------------
// macro	until
//
// purpose	Invert the logic of while, for use in statements.
//
// arguments	1 (int) condition
//
// usage	As replacement for while statements.
//-----------------------------------------------------------------------------
#define until(c) while(!(c))

//-----------------------------------------------------------------------------
// macro	zero_and_free
//
// purpose	Zero out memory then free it.  The caller must know the size.
//
// arguments	1 (void *) pointer to memory to zero and free
//		2 (size_t) size of memory to zero
//
// returns	(void)
//-----------------------------------------------------------------------------
#define zero_and_free(m,s) ({							\
	__typeof__((m)) libh__mem;						\
	libh__mem = (m);							\
	memset( libh__mem, 0, (s) );						\
	free( libh__mem );							\
})

//-----------------------------------------------------------------------------
// function	env_str
//
// purpose	Select the first defined environment variable from a given list
//		that is defined and non-empty, and return a pointer to the
//		environment value string.
//
// arguments	1..N (const char *) list of variable names, ending in NULL
//
// returns	(const char *) pointer to value string or NULL if not found
//-----------------------------------------------------------------------------
const char *
env_str (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	env_str_empty
//
// purpose	Select the first defined environment variable from a given list
//		that is defined, even if empty, and return a pointer to the
//		environment value string.
//
// arguments	1..N (const char *) list of variable names, ending in NULL
//
// returns	(const char *) pointer to value string or NULL if not found
//-----------------------------------------------------------------------------
const char *
env_str_empty (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_d
//
// purpose	Pick a double from the given arguments based on the value of
//		the first argument.  The first choice from argument 2 is
//		chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (double) values to choose one to be returned
//
// returns	(double) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
double
pick_d (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_ld
//
// purpose	Pick a long double from the given arguments based on the value
//		of the first argument.  The first choice from argument 2 is
//		chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (long double) values to choose one to be returned
//
// returns	(long double) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
long double
pick_ld (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_ptr
//
// purpose	Pick a pointer from the given arguments based on the value of
//		the first argument.  The first choice from argument 2 is
//		chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (void *) values to choose one to be returned
//
// returns	(void *) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
#define pick_char_p(i,a...) ((char *)pick_ptr((i),a))
#define pick_schar_p(i,a...) ((signed char *)pick_ptr((i),a))
#define pick_uchar_p(i,a...) ((unsigned char *)pick_ptr((i),a))
#define pick_short_p(i,a...) ((short *)pick_ptr((i),a))
#define pick_ushort_p(i,a...) ((unsigned short *)pick_ptr((i),a))
#define pick_int_p(i,a...) ((int *)pick_ptr((i),a))
#define pick_uint_p(i,a...) ((unsigned int *)pick_ptr((i),a))
#define pick_long_p(i,a...) ((long *)pick_ptr((i),a))
#define pick_ulong_p(i,a...) ((unsigned long *)pick_ptr((i),a))
#define pick_longlong_p(i,a...) ((long long *)pick_ptr((i),a))
#define pick_ulonglong_p(i,a...) ((unsigned long long *)pick_ptr((i),a))
#define pick_void_p(i,a...) ((void *)pick_ptr((i),a))
void *
pick_ptr (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_si
// alias	pick_i
//
// purpose	Pick a signed int from the given arguments based on the value
//		of the first argument.  The first choice from argument 2 is
//		chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (signed int) values to choose one to be returned
//
// returns	(int) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
#define pick_i pick_si
signed int
pick_si (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_sl
// alias	pick_l
//
// purpose	Pick a signed long from the given arguments based on the value of
//		the first argument.  The first choice from argument 2 is
//		chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (signed long) values to choose one to be returned
//
// returns	(signed long) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
#define pick_l pick_sl
signed long
pick_sl (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_sll
// alias	pick_ll
//
// purpose	Pick a signed long long from the given arguments based on the
//		value of the first argument.  The first choice from argument 2
//		is chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (signed long long) values to choose one to be returned
//
// returns	(signed long long) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
#define pick_ll pick_sll
signed long long
pick_sll (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_ui
//
// purpose	Pick an unsigned int from the given arguments based on the
//		value of the first argument.  The first choice from argument
//		2 is chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (unsigned int) values to choose one to be returned
//
// returns	(unsigned int) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
unsigned int
pick_ui (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_ul
//
// purpose	Pick an unsigned long from the given arguments based on the
//		value of the first argument.  The first choice from argument
//		2 is chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (unsigned long) values to choose one to be returned
//
// returns	(unsigned long) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
unsigned long
pick_ul (
    int			arg_choice
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	pick_ull
//
// purpose	Pick an unsigned long long from the given arguments based on
//		the value of the first argument.  The first choice from
//		argument 2 is chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (unsigned long long) values to choose one to be returned
//
// returns	(unsigned long long) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
unsigned long long
pick_ull (
    int			arg_choice
    ,
    ...
    )
;

#endif /* __CTOOLS_H__ */
