//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/daemon
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		daemon.h
//-----------------------------------------------------------------------------
#ifndef __DAEMON_H__
#define __DAEMON_H__

#include <stdio.h>
//-----------------------------------------------------------------------------
// function	daemon_set_start_return_fatal
//
// purpose	Set the return code for fatal errors from daemon_start().
//		The default value of zero means to exit.  A non-zero value
//		will be the negative return value.
//
// arguments	1 (int) return code value to use, or 0 to not return
//
// returns	(int) the given return code value
//-----------------------------------------------------------------------------
int
daemon_set_start_return_fatal (
    signed int		arg_code
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_master_return_fatal
//
// purpose	Set the return code for fatal errors from daemon_master().
//		The default value of zero means to exit.  A non-zero value
//		will be the negative return value.
//
// arguments	1 (int) return code value to use, or 0 to not return
//
// returns	(int) the given return code value
//-----------------------------------------------------------------------------
int
daemon_set_master_return_fatal (
    signed int		arg_code
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_server_return_fatal
//
// purpose	Set the return code for fatal errors from daemon_server().
//		The default value of zero means to exit.  A non-zero value
//		will be the negative return value.
//
// arguments	1 (int) return code value to use, or 0 to not return
//
// returns	(int) the given return code value
//-----------------------------------------------------------------------------
int
daemon_set_server_return_fatal (
    signed int		arg_code
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_server_return_restart
//
// purpose	Set the return code for restarts from daemon_server().
//		The default value of zero means to exit with a status of 0
//		to cause the master process to restart a new server process.
//		A non-zero value will be the negative return value.
//
// note		When the caller of daemon_server() gets a return value of
//		the specified code, it should eventually exit with a status
//		of 0 unless it decides the situation warrants a shutdown.
//
// arguments	1 (int) return code value to use, or 0 to not return
//
// returns	(int) the given return code value
//-----------------------------------------------------------------------------
int
daemon_set_server_return_restart (
    signed int		arg_code
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_server_return_shutdown
//
// purpose	Set the return code for shutdown errors from daemon_server().
//		The default value of zero means to exit with a status of 1
//		to cause the master process to exit.  A non-zero value will be
//		the negative return value.
//
// note		When the caller of daemon_server() gets a return value of
//		the specified code, it should eventually exit with a status
//		of 1 unless it decides the situation warrants a restart.
//
// arguments	1 (int) return code value to use, or 0 to not return
//
// returns	(int) the given return code value
//-----------------------------------------------------------------------------
int
daemon_set_server_return_shutdown (
    signed int		arg_code
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_program_name
//
// purpose	Set the program name for error messages.  This function picks
//		only the final part of the program name if it is a path.
//
// arguments	1 (const char *) program name.
//
// returns	1 (const char *) pointer to final part of program name used
//				 as a convenience for the caller.
//-----------------------------------------------------------------------------
const char *
daemon_set_program_name (
    const char *	arg_name
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_start_timeout
//
// purpose	Set the value of the start timeout for the 1st timeout cycle.
//
//		This timeout limits how long the startup process in the shell
//		foreground will wait for the background server to indicate
//		that it has successfully started running.
//
// arguments	1 (time_t) timeout value in seconds, or 0 to disable
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK
//-----------------------------------------------------------------------------
int
daemon_set_start_timeout (
    time_t	arg_timeout
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_restart_count_max
//
// purpose	Set the value of the restart count maximum.  The restart
//		process will not restart an exiting server more than this
//		many times.
//
// arguments	1 (int) count of times to restart, 0 for disabled.
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK
//-----------------------------------------------------------------------------
int
daemon_set_restart_count_max (
    signed int		arg_count
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_server_fast_time
//
// purpose	Set the value of the time difference in which server restart
//		times less than this amount are considered a fast restart.
//
// arguments	1 (time_t) minimum restart time in seconds
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
daemon_set_server_fast_time (
    time_t		arg_time
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_server_fast_max
//
// purpose	Set the maximum number of fast restarts in a row allowed for a
//		server process.  After this number is exceeded, output an error
//		message and return or abort.
//
// arguments	1 (int) maximum number of fast restarts
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
daemon_set_server_fast_max (
    int			arg_max
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_server_tick
//
// purpose	Set the value of the tick interval in seconds.
//
//		The server process will wake up periodically at intervals
//		specified by this value.  It can perform certain checks at
//		this time, such as making sure the master process is still
//		running.
//
// arguments	1 (unsigned long) interval
//		2 (unsigned long) interval offset
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK
//-----------------------------------------------------------------------------
int
daemon_set_server_tick (
    unsigned long	arg_interval
    ,
    unsigned long	arg_offset
    )
;

//-----------------------------------------------------------------------------
// function	daemon_set_worker_count_max
//
// purpose	Set the value of the maximum number of concurrent worker
//		processes to be allowed to run.  The server process will
//		stop accepting connections when this number of worker
//		processes are running.  It will resume accepting connections
//		when the number of worker processes running drops below this
//		number.
//
// arguments	1 (unsigned int) maximum worker count
//
// returns	(unsigned int) the previous setting
//-----------------------------------------------------------------------------
unsigned int
daemon_set_worker_count_max (
    unsigned int	arg_workers
    )
;

//-----------------------------------------------------------------------------
// function	daemon_clean_fd
//
// purpose	Clean up excess leftover file descriptors, usually inherited
//		from annoying shells.
//
// arguments	1 (int) highest fd to NOT close
//
// returns	(int) count of descriptors which successfully closed
//
// note		Some systems may have limits on the number of file descriptors
//		allowed that are so high that looping to attempt to close them
//		all is an unreasonable use of CPU time that may burden or slow
//		down the startup of the daemon.  This function places a ceiling
//		on that number to keep things from getting out of control.
//-----------------------------------------------------------------------------
int
daemon_clean_fd (
    int		arg_lo
    )
;

//-----------------------------------------------------------------------------
// function	daemon_success
//
// purpose	Send a signal to the startup process that the server has
//		successfully begun operations.  This informs the startup
//		process that the status it should exit with is 0.
//
//		Also close the write ends of the pipes now so the startup
//		process gets EOF.
//
// arguments	-none-
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
daemon_success ()
;

//-----------------------------------------------------------------------------
// function	daemon_start
//
// purpose	Run the startup loop in the foreground process and return to
//		the calling program in a child process, which will later become
//		the server process or the master process.
//
//		The inherited stdin, stdout, and stderr descriptors and the
//		controlling tty are isolated from the background processes.
//		Pipes are established in place of stdout and stderr and copied
//		by the startup loop to the real stdout and stderr so that the
//		startup or failure messages can be presented to the startup
//		session tty.
//
// arguments	-none-
//
// returns	(int) -1 : in caller: fork failed or other error before fork
//		(int)  0 : in child:  child should now run server
//
// note		When caller gets a return value of -1 it should do any
//		cleanups and additional error messages then exit.
//
// note		When caller gets a return value of 0 it should do any needed
//		preparations to run a daemon then start running it, perhaps
//		by calling daemon_master() to have automatic restarts or
//		daemon_server() to run a server once.
//-----------------------------------------------------------------------------
#define DAEMON_START_RETURN_ERROR(v)	((v)<0)
#define DAEMON_START_RETURN_IN_MASTER	0
int
daemon_start ()
;

//-----------------------------------------------------------------------------
// function	daemon_master
//
// purpose	1.  Start and restart as needed a process to run a server
//		    which will serve incoming network connections.
//
//		2.  Run the master loop which monitors and restarts the
//		    server process.
//
//		The daemon_master() function returns to the caller in the
//		same process it was called in if there is an error getting
//		started, or if there is an error in the master loop which
//		has been directed to return an error code.
//
//		The daemon_master() function returns to the caller in new
//		processes for each startup of the server process.  The main
//		caller should then call daemon_server() to actually run the
//		server after any configuration or other setup.
//
// arguments	1 (long *) where to store process restart sequence count
//
// returns	(int) == -1: in master process: fatal error
//		(int) ==  0: in server process: started
//
// note		Some errors in master loop will exit instead of return.
//
// note		When caller gets a return value of 0, it should proceed to
//		run the server, generally by calling daemon_server() to serve
//		network connections.
//-----------------------------------------------------------------------------
#define DAEMON_MASTER_RETURN_ERROR(v)	((v)<0)
#define DAEMON_MASTER_RETURN_IN_SERVER	0
long
daemon_master (
    long *	arg_count_ptr
    )
;

//-----------------------------------------------------------------------------
// function	daemon_server
//
// purpose	Run the server process accepting (via poll) incoming
//		connections, forking a new worker for each.
//
// details	The worker process is started as a child process which returns
//		to the calling program to run the worker program code.  The
//		new connection will be descriptors 0 and 1.  When the worker
//		is done, it can shutdown(), close() and exit().
//
// arguments	1 (int *) list of descriptors to accept connections on
//		2 (struct sockaddr *) socket address to be filled by accept
//		3 (socklen_t *) socket address length to be filled by accept
//
// returns	(int) <  0 : in caller: error
//		(int) >= 0 : index into list of descriptors for the specific
//				descriptor on which this connection arrived
//
// note		The list of descriptors to accept connections on should be
//		terminated by a negative value, but a value of 2 or less
//		will also terminate it since standard descriptors should not
//		be passed to the daemon library in this list.
//
// note		The 2nd and 3rd arguments are like the 2nd and 3rd arguments
//		of the accept() call.  The variable pointed to by the 3rd
//		argument must be filled in with the length of space available
//		in the 2nd argument.  Upon return, the 2nd argument will have
//		the connecting socket address filled in, and the 3rd argument
//		will have the actual address length filled in the same way
//		that accept() does.
//
// note		In order to accept IPv6 connections, the size of the socket
//		address area, and the value of the given length, must be at
//		least that of sizeof(sockaddr_in6).  See "man ipv6".
//
// note		The returned value is an index to the list of descriptors,
//		not the descriptor value itself.  This allows the caller to
//		keep another array of information related to each descriptor,
//		such as the type of service or protocol to carry out, which
//		can be easily indexed by this returned value.
//-----------------------------------------------------------------------------
#define DAEMON_SERVER_RETURN_ERROR(v)	((v)<0)
int
daemon_server (
    int *		arg_sockfd_list
    ,
    struct sockaddr *	arg_sock_addr
    ,
    socklen_t *		arg_sock_addrlen
    )
;

#endif /* __DAEMON_H__ */
