//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
//-----------------------------------------------------------------------------
// package	libh/http
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		http.h
//
// purpose	Define resources used by and with the http library.
//-----------------------------------------------------------------------------
#ifndef __HTTP_H__
#define __HTTP_H__

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <libh/time.h>

//-----------------------------------------------------------------------------
// Define HTTP response strings.
//-----------------------------------------------------------------------------
#define HTTP_RESPONSE_100	"100 Continue"
#define HTTP_RESPONSE_101	"101 Switching Protocols"
#define HTTP_RESPONSE_102	"102 Processing"

#define HTTP_RESPONSE_200	"200 OK"
#define HTTP_RESPONSE_201	"201 Created"
#define HTTP_RESPONSE_202	"202 Accepted"
#define HTTP_RESPONSE_203	"203 Non-Authoritative Information"
#define HTTP_RESPONSE_204	"204 No Content"
#define HTTP_RESPONSE_205	"205 Reset Content"
#define HTTP_RESPONSE_206	"206 Partial Content"
#define HTTP_RESPONSE_207	"207 Multi-Status"

#define HTTP_RESPONSE_300	"300 Multiple Choices"
#define HTTP_RESPONSE_301	"301 Moved Permanently"
#define HTTP_RESPONSE_302	"302 Found"
#define HTTP_RESPONSE_303	"303 See Other"
#define HTTP_RESPONSE_304	"304 Not Modified"
#define HTTP_RESPONSE_305	"305 Use Proxy"
#define HTTP_RESPONSE_307	"307 Temporary Redirect"

#define HTTP_RESPONSE_400	"400 Bad Request"
#define HTTP_RESPONSE_401	"401 Unauthorized"
#define HTTP_RESPONSE_402	"402 Payment Required"
#define HTTP_RESPONSE_403	"403 Forbidden"
#define HTTP_RESPONSE_404	"404 Not Found"
#define HTTP_RESPONSE_405	"405 Method Not Allowed"
#define HTTP_RESPONSE_406	"406 Not Acceptable"
#define HTTP_RESPONSE_407	"407 Proxy Authentication Required"
#define HTTP_RESPONSE_408	"408 Request Timeout"
#define HTTP_RESPONSE_409	"409 Conflict"
#define HTTP_RESPONSE_410	"410 Gone"
#define HTTP_RESPONSE_411	"411 Length Required"
#define HTTP_RESPONSE_412	"412 Precondition Failed"
#define HTTP_RESPONSE_413	"413 Request Entity Too Large"
#define HTTP_RESPONSE_414	"414 Request-URI Too Long"
#define HTTP_RESPONSE_415	"415 Unsupported Media Type"
#define HTTP_RESPONSE_416	"416 Requested Range Not Satisfiable"
#define HTTP_RESPONSE_417	"417 Expectation Failed"
#define HTTP_RESPONSE_422	"422 Unprocessable Entity"
#define HTTP_RESPONSE_423	"423 Locked"
#define HTTP_RESPONSE_424	"424 Failed Dependency"

#define HTTP_RESPONSE_500	"500 Internal Server Error"
#define HTTP_RESPONSE_501	"501 Not Implemented"
#define HTTP_RESPONSE_502	"502 Bad Gateway"
#define HTTP_RESPONSE_503	"503 Service Unavailable"
#define HTTP_RESPONSE_504	"504 Gateway Timeout"
#define HTTP_RESPONSE_505	"505 HTTP Version Not Supported"
#define HTTP_RESPONSE_506	"506 Variant Also Negotiates"
#define HTTP_RESPONSE_507	"507 Insufficient Storage"
#define HTTP_RESPONSE_510	"510 Not Extended"

//-----------------------------------------------------------------------------
// macro	http_content_type
//
// purpose	Queue an HTTP Content-Type header for output with the given
//		MIME type string.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) MIME type string
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define http_content_type(p,t) http_header((p),"Content-Type",(t))

//-----------------------------------------------------------------------------
// macro	http_date_now
//
// purpose	Put the current date as an HTTP date header in the queue.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) header prefix to use
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define http_date_now() http_date(0,NULL,0)

//-----------------------------------------------------------------------------
// macro	http_error_exit
//
// purpose	Handle a fatal error condition and exit the process.
//
// arguments	1 (int) exit status code to use
//		2 (const char *) HTTP error response string
//		3 (const char *) error message text
//
// returns	No return
//-----------------------------------------------------------------------------
#define http_error_exit(e,s,m...) ({(http_error)((s),m,(const char*)(NULL));exit((e));})

//-----------------------------------------------------------------------------
// macro	http_error__exit
//
// purpose	Handle a fatal error condition and exit the process quickly.
//
// arguments	1 (int) exit status code to use
//		2 (const char *) HTTP error response string
//		3 (const char *) error message text
//
// returns	No return
//-----------------------------------------------------------------------------
#define http_error__exit(e,s,m...) ({(http_error)((s),m,(const char*)(NULL));_exit((e));})

//-----------------------------------------------------------------------------
// macro	http_error_return
//
// purpose	Handle a fatal error condition and exit the process.
//
// arguments	1 (?) return value
//		2 (const char *) HTTP error response string
//		3 (const char *) error message text
//
// returns	Whatever is provided in argument 1.
//-----------------------------------------------------------------------------
#define http_error_return(r,s,m...) ({(http_error)((s),m,(const char*)(NULL));return (r);})

//-----------------------------------------------------------------------------
// macro	http_header
//
// purpose	Build an HTTP header string from given arguments and place it
//		in the HTTP header queue.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) header name
//		3..N (const char *) arguments of strings
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define http_header(p,h,a...) (http_header)((p),(h),a,(const char*)(NULL))

//-----------------------------------------------------------------------------
// function	http_auth_fail
//
// purpose	Handle HTTP authorization authentication failure by prompting
//		for a username and password.
//
// arguments	1 (const char *) NULL or alternate realm name to use.
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
http_auth_fail (
    const char *	arg_realm
)
;

//-----------------------------------------------------------------------------
// function	http_auth_get
//
// purpose	Extract and decode the username and the password from the
//		HTTP_AUTHORIZATION environment variable, which represents
//		The HTTP Authorization header.
//
// notice	This requires the SECURITY_HOLE_PASS_AUTHORIZATION option
//		to be enabled in Apache at compile time in order to get
//		the HTTP Authorization header in the HTTP_AUTHORIZATION
//		environment variable.  Otherwise Apache suppresses that
//		environment variable for security purposes.
//
// arguments	1 (const char *) NULL or alternate HTTP authorization string
//		2 (char *) where to store the username
//		3 (size_t) maximum length for username
//		4 (char *) where to store the password
//		5 (size_t) maximum length for password
//
// returns	(int) -1 : error
//		(int)  0 : OK (username and password are both OK)
//		+1 = username is truncated
//		+2 = password is truncated
//		+4 = password is absent
//-----------------------------------------------------------------------------
int
http_auth_get (
    const char *	arg_authstr
    ,
    char *		arg_ptr_username
    ,
    size_t		arg_len_username
    ,
    char *		arg_ptr_password
    ,
    size_t		arg_len_password
    )
;

//-----------------------------------------------------------------------------
// function	http_commit_put
//
// purpose	Commit the HTTP headers and output them to stdout.
//
// arguments	-none-
//
// returns	(int) -1 : error
//		(int)  0 : OK
//		(int)  1 : already done
//-----------------------------------------------------------------------------
int
http_commit_put()
;

//-----------------------------------------------------------------------------
// function	http_date
//
// purpose	Put an HTTP date header in the queue.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) header prefix to use
//		3 (time_t) time value to convert, or 0 for current time
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define http_date_now() http_date(0,NULL,0)
int
http_date (
    int			arg_priority
    ,
    const char *	arg_head
    ,
    time_t		arg_time
    )
;

//-----------------------------------------------------------------------------
// function	http_discard
//
// purpose	Discard the HTTP response and queued HTTP headers.
//
// arguments	-none-
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
http_discard ()
;

//-----------------------------------------------------------------------------
// function	http_error
//
// purpose	Handle a fatal error condition.
//
// arguments	1 (const char *) HTTP error response string
//		2..N (const char *) error message text
//
// returns	(int) -1 : error (imagine that)
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define http_error(s,m...) ((http_error)((s),m,(const char*)(NULL)))

int
(http_error) (
    const char *	arg_error_res
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	http_etime
//
// purpose	Put an HTTP date header in the queue with the date and time
//		taken from an etime value.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) header prefix to use
//		3 (etime_t) etime value to convert, or 0 for current time
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
http_etime (
    int			arg_priority
    ,
    const char *	arg_head
    ,
    etime_t		arg_etime
    )
;

//-----------------------------------------------------------------------------
// function	(http_header)
//
// purpose	Build an HTTP header string from given arguments and place it
//		in the HTTP header queue.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) header name
//		3..N (const char *) arguments of strings
//		N+1 (const char *) NULL to indicate end of arguments
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
(http_header) (
    int			arg_priority
    ,
    const char *	arg_header
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	http_header_str
//
// purpose	Place an arbitrary header string in the HTTP header queue.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) header string
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
http_header_str (
    int			arg_priority
    ,
    const char *	arg_string
    )
;

//-----------------------------------------------------------------------------
// function	http_header_var
//
// purpose	Build an HTTP header string from a va_list argument and place
//		it in the HTTP header queue.
//
// arguments	1 (int) priority (lowest first)
//		2 (const char *) header name
//		3 (va_list) va_list of strings ended with NULL
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
http_header_var (
    int			arg_priority
    ,
    const char *	arg_header
    ,
    va_list		arg_str_list
    )
;

//-----------------------------------------------------------------------------
// function	http_no_cache
//
// purpose	Output headers for a page with no caching.
//
// arguments	-none-
//
// returns	(int) 0
//-----------------------------------------------------------------------------
int
http_no_cache ()
;

//-----------------------------------------------------------------------------
// function	http_redirect
//
// purpose	Redirect HTTP request to a specified URL.
//
// arguments	1 (const char *) format string for URL
//		2..N (...) variable arguments for formatting URL
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
http_redirect (
    const char *	arg_url_format
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	http_response_str
//
// purpose	Specify the HTTP response code string to use.
//
// arguments	(const char *) response code string
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
http_response_str (
    const char *		arg_response
    )
;

//-----------------------------------------------------------------------------
// function	http_set_full
//
// purpose	Set the HTTP mode for full response, which outputs the HTTP
//		response line (instead of the Status header for Apache).
//		This would be used for CGI in Non-Parsed Header mode, or in
//		a custom web server.
//
// arguments	1 (int) 0 : disable full response mode
//		1 (int) 1 : enable full response mode
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
http_set_full (
    int		arg_mode
    )
;

#endif /* __HTTP_H__ */
