//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/io
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		io.h
//-----------------------------------------------------------------------------
#ifndef __IO_H__
#define __IO_H__

#include <dirent.h>
#include <dlfcn.h>
#include <fcntl.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <poll.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include <libh/map.h>

//-----------------------------------------------------------------------------
// struct	get_item_s
// typedef	get_item_t, get_item_p
//
// purpose	Hold the parameters and state for parsing an item stream.
//-----------------------------------------------------------------------------
struct get_item_s {

    unsigned long long		pos_cnt			;
    unsigned long		pos_col			;
    unsigned long		pos_row			;

    unsigned long long		beg_cnt			;
    unsigned long		beg_col			;
    unsigned long		beg_row			;

    unsigned long long		end_cnt			;
    unsigned long		end_col			;
    unsigned long		end_row			;

    //------------------------------------------------
    // Which of these union members is used depends on
    // which input source is designated via getc_p.
    //------------------------------------------------
    union get_item_s_input_sources {
	struct get_item_s_input_sources_fd {
	    size_t		size			;
	    int			fd			;
	}				fd		;

	struct get_item_s_input_sources_file {
	    FILE *		file			;
	}				file		;

	struct get_item_s_input_sources_str {
	    const char *	str			;
	    size_t		len			;
	}				str		;

	struct get_item_s_input_sources_call {
	    int		( *	fun	)(void*)	;
	    void	*	arg			;
	}				call		;
    }						in	;

    int		( *	getc_p	)(struct get_item_s*)	;

    char *		buf_get_p			;
    char *		buf_put_p			;
    char *		buf_end_p			;

    size_t		actual_len			;
    size_t		stored_len			;

    int			opt_hold			;

    int			opt_tab				;
    int			opt_bslash			;
    int			opt_carat			;
    int			opt_colon			;
    int			opt_quote2			;
    int			opt_equal			;
    int			opt_percent			;
    int			opt_quote1			;
    int			opt_hash			;
    int			opt_scolon			;
    int			opt_slash			;
    int			opt_bquote			;

    unsigned char	in_eof				;
    unsigned char	in_error			;
    unsigned char	in_newline			;
    unsigned char	in_comment			;
    unsigned char	in_hold				;

    char		buffer		[3]		;
};

typedef struct get_item_s	get_item_t		;
typedef struct get_item_s *	get_item_p		;

//-----------------------------------------------------------------------------
// macro	chdir_join
//
// purpose	Change the current directory to the given name where the
//		directory name is given as multiple string arguments to be
//		joined together.
//
// arguments	1..N (const char *) directory name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, directory changed] status
//-----------------------------------------------------------------------------
#define chdir_join(n...) ((chdir_join)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	chdir_join_sep
//
// purpose      Change the current directory to the given name where the
//              directory name is given as multiple string arguments to be
//              joined together with a directory separator between each
//		argument.
//
// arguments    1..N (const char *) directory name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, directory changed] status
//-----------------------------------------------------------------------------
#define chdir_join_sep(n...) ((chdir_join_sep)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	chmod_join
//
// purpose	Change the mode of a named file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (mode_t) new mode for file
//		2..N (const char *) file name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, mode changed] status
//-----------------------------------------------------------------------------
#define chmod_join(m,n...) ((chmod_join)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	chmod_join_sep
//
// purpose      Change the mode of a named file where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments    1 (mode_t) new mode for file
//              2..N (const char *) file name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, mode changed] status
//-----------------------------------------------------------------------------
#define chmod_join_sep(m,n...) ((chmod_join_sep)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	chown_join
//
// purpose	Change the owner and group of a named file where the file name
//		is given as multiple string arguments to be joined together.
//
// arguments	1 (uid_t) new owner id
//		2 (gid_t) new group id
//		3..N (const char *) file name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, owner/group changed] status
//-----------------------------------------------------------------------------
#define chown_join(u,g,n...) ((chown_join)((u),(g),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	chown_join_sep
//
// purpose	Change the owner and group of a named file where the file name
//		is given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1 (uid_t) new owner id
//		2 (gid_t) new group id
//		3..N (const char *) file name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, owner/group changed] status
//-----------------------------------------------------------------------------
#define chown_join_sep(u,g,n...) ((chown_join_sep)((u),(g),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	chroot_join
//
// purpose	Change the root directory to the given name where the
//		directory name is given as multiple string arguments to be
//		joined together.
//
// arguments	1..N (const char *) directory name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, root changed] status
//-----------------------------------------------------------------------------
#define chroot_join(n...) ((chroot_join)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	chroot_join_sep
//
// purpose      Change the root directory to the given name where the
//              directory name is given as multiple string arguments to be
//              joined together with a directory separator between each
//		argument.
//
// arguments    1..N (const char *) directory name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, root changed] status
//-----------------------------------------------------------------------------
#define chroot_join_sep(n...) ((chroot_join_sep)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	dlopen_join
//
// purpose	Open a dynamically linked library where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (int) option flags, e.g. RTLD_LAZY, RTLD_NOW, RTLD_GLOBAL
//		2..N (const char *) file name parts
//
// returns	(void *) == NULL : [failure, see errno]
//		(void *) != NULL : [success, library open] handle
//-----------------------------------------------------------------------------
#define dlopen_join(f,n...) ((dlopen_join)((dlopen),(f),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	dlopen_join_sep
//
// purpose      Open a dynamically linked library where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments    1 (int) option flags, e.g. RTLD_LAZY, RTLD_NOW, RTLD_GLOBAL
//              2..N (const char *) file name parts
//
// returns      (void *) == NULL : [failure, see errno]
//              (void *) != NULL : [success, library open] handle
//-----------------------------------------------------------------------------
#define dlopen_join_sep(f,n...) ((dlopen_join_sep)((dlopen),(f),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	dlsym_join
//
// purpose	Return the address of a symbol in the open library specified
//		by its handle, where the symbol name is given as multiple
//		string arguments to be joined together.
//
// arguments	1 (void *) open library handle
//		2..N (const char *) symbol name parts
//
// returns	(void *) == NULL : [failure, see errno]
//		(void *) != NULL : [success, symbol found] pointer
//-----------------------------------------------------------------------------
#define dlsym_join(f,n...) ((dlsym_join)((dlsym),(f),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	dlsym_join_sep
//
// purpose	Return the address of a symbol in the open library specified
//		by its handle, where the symbol name is given as multiple
//		string arguments to be joined together with an underscore
//		separator between each argument.
//
// arguments	1 (void *) open library handle
//		2..N (const char *) symbol name parts
//
// returns	(void *) == NULL : [failure, see errno]
//		(void *) != NULL : [success, symbol found] pointer
//-----------------------------------------------------------------------------
#define dlsym_join_sep(f,n...) ((dlsym_join_sep)((dlsym),(f),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// function	fopen_fork_exec
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (FILE * *) where to store FILE pointer of write end of pipe
//			which will be the child's stdin.
//		2 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stdout.
//		3 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stderr.
//		4 (const char *) filename of program to execute in child
//		5-N (const char *) arguments to program being executed
//
// returns	(int) -1 if there is an error (see errno)
//		(int) child PID
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same FILE pointer as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
#define fopen_fork_exec(i,o,e,p,a...) (fopen_fork_exec)((i),(o),(e),(p),a,(const char*)(NULL))

//-----------------------------------------------------------------------------
// macro	fopen_join
//
// purpose	Open a file and return a FILE pointer where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (const char *) mode, e.g. "r", "w+", etc.
//		2..N (const char *) file name parts
//
// returns	(FILE *) == NULL : [failure, see errno]
//		(FILE *) != NULL : [success, file open] FILE pointer
//-----------------------------------------------------------------------------
#define fopen_join(m,n...) ((fopen_join)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	fopen_join_sep
//
// purpose	Open a file and return a FILE pointer where the file name is
//		given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1 (const char *) mode, e.g. "r", "w+", etc.
//		2..N (const char *) file name parts
//
// returns	(FILE *) == NULL : [failure, see errno]
//		(FILE *) != NULL : [success, file open] FILE pointer
//-----------------------------------------------------------------------------
#define fopen_join_sep(m,n...) ((fopen_join_sep)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// function     fclose_proc
//
// purpose      Close a pipe to a process opened by fopen_proc.
//
// arguments    1 (FILE *) file descriptor to close
//
// returns      (int) final child process status
//-----------------------------------------------------------------------------
#define fclose_proc(f) fclose((f))

//-----------------------------------------------------------------------------
// macro	fopen_proc_arg
//
// purpose	Create a pipe to a child process to read from its stdout (and
//		optionally stderr) or to write to its stdin, with the child
//		process running the given command and arguments.
//
// arguments	1 (const char *) option string
//		2 (const char *) file name to execute
//		3-N (const char *) program arguments as function arguments
//
// returns	(FILE *) NULL : error
//		(FILE *) file pointer to pipe.
//
// options	"r" open to read from stdout (and stderr if not inherited)
//		"w" open to write to stdin
//		"i" inherit stdin from parent
//		"o" inherit stdout from parent
//		"e" inherit stderr from parent
//
// warning	Do not use pclose() on the stream returned by this function.
//		It works differently than popen() and is not compatible with
//		pclose().
//-----------------------------------------------------------------------------
#define fopen_proc_arg(o,f,a...) ((fopen_proc_arg)((o),(f),a,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	eput_byte
//
// purpose	Output a byte, escaped as needed, to stderr.
//
// arguments	1 (int) point to beginning of a byte
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_byte(b) fput_byte((b),(stderr))

//-----------------------------------------------------------------------------
// macro	put_byte
//
// purpose	Output a byte, escaped as needed, to stdout.
//
// arguments	1 (int) point to beginning of a byte
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_byte(b) fput_byte((b),(stdout))

//-----------------------------------------------------------------------------
// macro	eput_byte_xx
//
// purpose	Output a byte, escaped as needed, to stderr.
//
// arguments	1 (int) point to beginning of a byte
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_byte_xx(b) fput_byte_xx((b),(stderr))

//-----------------------------------------------------------------------------
// macro	put_byte_xx
//
// purpose	Output a byte, escaped as needed, to stdout.
//
// arguments	1 (int) point to beginning of a byte
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_byte_xx(b) fput_byte_xx((b),(stdout))

//-----------------------------------------------------------------------------
// macro	eput_bytes_end
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stderr.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (const char *) point to end of bytes (after last)
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_bytes_end(p,e) fput_bytes_end((p),(e),(stderr))

//-----------------------------------------------------------------------------
// macro	put_bytes_end
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stdout.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (const char *) point to end of bytes (after last)
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_bytes_end(p,e) fput_bytes_end((p),(e),(stdout))

//-----------------------------------------------------------------------------
// macro	eput_bytes_end_xx
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stderr.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (const char *) point to end of bytes (after last)
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_bytes_end_xx(p,e) fput_bytes_end_xx((p),(e),(stderr))

//-----------------------------------------------------------------------------
// macro	put_bytes_end_xx
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stdout.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (const char *) point to end of bytes (after last)
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_bytes_end_xx(p,e) fput_bytes_end_xx((p),(e),(stdout))

//-----------------------------------------------------------------------------
// macro	eput_bytes_len
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stderr.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_bytes_len(p,l) fput_bytes_len((p),(l),(stderr))

//-----------------------------------------------------------------------------
// macro	put_bytes_len
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stdout.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_bytes_len(p,l) fput_bytes_len((p),(l),(stdout))

//-----------------------------------------------------------------------------
// macro	eput_bytes_len_xx
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stderr.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_bytes_len_xx(p,l) fput_bytes_len_xx((p),(l),(stderr))

//-----------------------------------------------------------------------------
// macro	put_bytes_len_xx
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to stdout.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_bytes_len_xx(p,l) fput_bytes_len_xx((p),(l),(stdout))

//-----------------------------------------------------------------------------
// macro	eput_bytes_str
//
// purpose	Output bytes, escaped as needed, up to zero termination,
//		to stderr.
//
// arguments	1 (const char *) point to beginning of bytes
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_bytes_str(p) fput_bytes_len((p),~(size_t)0,(stderr))

//-----------------------------------------------------------------------------
// macro	put_bytes_str
//
// purpose	Output bytes, escaped as needed, up to zero termination,
//		to stdout.
//
// arguments	1 (const char *) point to beginning of bytes
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_bytes_str(p) fput_bytes_len((p),~(size_t)0,(stdout))

//-----------------------------------------------------------------------------
// macro	fput_bytes_str
//
// purpose	Output bytes, escaped as needed, up to zero termination,
//		to the specified open file.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
#define fput_bytes_str(p,f) fput_bytes_len((p),~(size_t)0,(f))

//-----------------------------------------------------------------------------
// macro	eput_bytes_str_xx
//
// purpose	Output bytes, escaped as needed, up to zero termination,
//		to stderr.
//
// arguments	1 (const char *) point to beginning of bytes
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_bytes_str_xx(p) fput_bytes_len_xx((p),~(size_t)0,(stderr))

//-----------------------------------------------------------------------------
// macro	put_bytes_str_xx
//
// purpose	Output bytes, escaped as needed, up to zero termination,
//		to stdout.
//
// arguments	1 (const char *) point to beginning of bytes
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_bytes_str_xx(p) fput_bytes_len_xx((p),~(size_t)0,(stdout))

//-----------------------------------------------------------------------------
// macro	fput_bytes_str_xx
//
// purpose	Output bytes, escaped as needed, up to zero termination,
//		to the specified open file.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
#define fput_bytes_str_xx(p,f) fput_bytes_len_xx((p),~(size_t)0,(f))

//-----------------------------------------------------------------------------
// macro	eput_cwd
//
// purpose	Output the current directory string to stderr.
//
// arguments	-none-
//
// returns	(size_t) length of string written
//-----------------------------------------------------------------------------
#define eput_cwd() (fput_cwd((stderr)))

//-----------------------------------------------------------------------------
// macro	put_cwd
//
// purpose	Output the current directory string to stdout.
//
// arguments	-none-
//
// returns	(size_t) length of string written
//-----------------------------------------------------------------------------
#define put_cwd() (fput_cwd((stdout)))

//-----------------------------------------------------------------------------
// function	eput_dirfd
//
// purpose	Output the path of a directory given as an open descriptor
//		to stderr.
//
// arguments	1 (int) file descriptor of directory
//		2 (FILE *) which file to write to, or NULL for stdout
//
// returns	(size_t) length of string written
//-----------------------------------------------------------------------------
#define eput_dirfd(d) (fput_dirfd((d),(stderr)))

//-----------------------------------------------------------------------------
// function	put_dirfd
//
// purpose	Output the path of a directory given as an open descriptor
//		to stdout.
//
// arguments	1 (int) file descriptor of directory
//		2 (FILE *) which file to write to, or NULL for stdout
//
// returns	(size_t) length of string written
//-----------------------------------------------------------------------------
#define put_dirfd(d) (fput_dirfd((d),(stdout)))

//-----------------------------------------------------------------------------
// macro	fstat_atime
//
// purpose	Return access time of an open file descriptor.
//
// arguments	1 (int) file descriptor of open file
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] access time
//-----------------------------------------------------------------------------
#define fstat_atime(f) ({							\
	struct stat libh__statbuf;						\
	fstat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_atime;					\
})

//-----------------------------------------------------------------------------
// macro	fstat_ctime
//
// purpose	Return change time of an open file descriptor.
//
// arguments	1 (int) file descriptor of open file
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] change time
//-----------------------------------------------------------------------------
#define fstat_ctime(f) ({							\
	struct stat libh__statbuf;						\
	fstat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_ctime;					\
})

//-----------------------------------------------------------------------------
// macro	fstat_gid
//
// purpose	Return group ID of an open file descriptor.
//
// arguments	1 (int) file descriptor of open file
//
// returns	(gid_t) == -1 : [failure, see errno]
//		(gid_t) >=  0 : [success] group ID number
//-----------------------------------------------------------------------------
#define fstat_gid(f) ({								\
	struct stat libh__statbuf;						\
	fstat( (f), & libh__statbuf ) < 0					\
		? (gid_t) -1							\
		: libh__statbuf.st_gid;						\
})

//-----------------------------------------------------------------------------
// macro	fstat_mode
//
// purpose	Return mode of an open file descriptor.
//
// arguments	1 (int) file descriptor of open file
//
// returns	(mode_t) == -1 : [failure, see errno]
//		(mode_t) >=  0 : [success] file mode
//-----------------------------------------------------------------------------
#define fstat_mode(f) ({							\
	struct stat libh__statbuf;						\
	fstat( (f), & libh__statbuf ) < 0					\
		? (mode_t) -1							\
		: libh__statbuf.st_mode;					\
})

//-----------------------------------------------------------------------------
// macro	fstat_mtime
//
// purpose	Return modification time of an open file descriptor.
//
// arguments	1 (int) file descriptor of open file
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] modification time
//-----------------------------------------------------------------------------
#define fstat_mtime(f) ({							\
	struct stat libh__statbuf;						\
	fstat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_mtime;					\
})

//-----------------------------------------------------------------------------
// macro	fstat_size
//
// purpose	Return size of an open file descriptor.
//
// arguments	1 (int) file descriptor of open file
//
// returns	(long long) == -1 : [failure, see errno]
//		(long long) >=  0 : [success] file size
//
// note		A (long long) type is returned so that if type (off_t) is a
//		smaller unsigned integer, it's full range of values will not
//		collide with the error return value.
//-----------------------------------------------------------------------------
#define fstat_size(f) ({							\
	struct stat libh__statbuf;						\
	fstat( (f), & libh__statbuf ) < 0					\
		? (long long) -1						\
		: (long long) libh__statbuf.st_size;				\
})

//-----------------------------------------------------------------------------
// macro	fstat_uid
//
// purpose	Return owner ID of an open file descriptor.
//
// arguments	1 (int) file descriptor of open file
//
// returns	(uid_t) == -1 : [failure, see errno]
//		(uid_t) >=  0 : [success] owner ID number
//-----------------------------------------------------------------------------
#define fstat_uid(f) ({								\
	struct stat libh__statbuf;						\
	fstat( (f), & libh__statbuf ) < 0					\
		? (uid_t) -1							\
		: libh__statbuf.st_uid;						\
})

//-----------------------------------------------------------------------------
// macro	link_join
//
// purpose	Link one name as another where the names are given as multiple
//		string arguments to be joined together.
//
// arguments	1..N (const char *) name parts for source name
//		N+1 (const char *) NULL to indicate end of source list
//		N+2..M (const char *) name parts for target name
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, names linked] status
//-----------------------------------------------------------------------------
#define link_join(n...) ((link_join)(n,(const char*)(NULL),(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	lock_read
//
// purpose	Simple interface to POSIX fcntl() file/record/range locking.
//		Lock a file/record/range for reading, without waiting.
//
// arguments	1 (int) file descriptor of open file to lock
//		2 (off_t) starting byte offset to begin locking
//		3 (off_t) length of record/range to lock
//
// returns	(int) -1 : error
//		(int)  0 : lock has been obtained
//		(int) >0 : process ID already holding lock
//-----------------------------------------------------------------------------
#define lock_read(f,s,l) ({							\
	struct flock libh__lock;						\
	int _libh_fd_;								\
	libh__lock.l_type = ( F_RDLCK );					\
	libh__lock.l_whence = ( SEEK_SET );					\
	libh__lock.l_start = ( s );						\
	libh__lock.l_len = ( l );						\
	libh__fd = ( f );							\
	fcntl( libh__fd, ( F_SETLK ), & libh__lock ) == 0			\
		? 0								\
		: ({								\
			int libh__errno;					\
			libh__errno = errno;					\
			fcntl( libh__fd, ( F_GETLK ), & libh__lock );		\
			errno = libh__errno;					\
			libh__lock.l_type == ( F_UNLCK )			\
				? -1						\
				: libh__lock.l_pid;				\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	lock_read_wait
//
// purpose	Simple interface to POSIX fcntl() file/record/range locking.
//		Lock a file/record/range for reading, waiting to get lock.
//
// arguments	1 (int) file descriptor of open file to lock
//		2 (off_t) starting byte offset to begin locking
//		3 (off_t) length of record/range to lock
//
// returns	(int) -1 : error
//		(int)  0 : lock has been obtained
//		(int) >0 : process ID already holding lock
//-----------------------------------------------------------------------------
#define lock_read_wait(f,s,l) ({						\
	struct flock libh__lock;						\
	int libh__fd;								\
	libh__lock.l_type = ( F_RDLCK );					\
	libh__lock.l_whence = ( SEEK_SET );					\
	libh__lock.l_start = ( s );						\
	libh__lock.l_len = ( l );						\
	libh__fd = ( f );							\
	fcntl( libh__fd, ( F_SETLKW ), & libh__lock ) == 0			\
		? 0								\
		: ({								\
			int libh__errno;					\
			libh__errno = errno;					\
			fcntl( libh__fd, ( F_GETLK ), & libh__lock );		\
			errno = libh__errno;					\
			libh__lock.l_type == ( F_UNLCK )			\
				? -1						\
				: libh__lock.l_pid;				\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	lock_write
//
// purpose	Simple interface to POSIX fcntl() file/record/range locking.
//		Lock a file/record/range for writing, without waiting.
//
// arguments	1 (int) file descriptor of open file to lock
//		2 (off_t) starting byte offset to begin locking
//		3 (off_t) length of record/range to lock
//
// returns	(int) -1 : error
//		(int)  0 : lock has been obtained
//		(int) >0 : process ID already holding lock
//-----------------------------------------------------------------------------
#define lock_write(f,s,l) ({							\
	struct flock libh__lock;						\
	int libh__fd;								\
	libh__lock.l_type = ( F_WRLCK );					\
	libh__lock.l_whence = ( SEEK_SET );					\
	libh__lock.l_start = ( s );						\
	libh__lock.l_len = ( l );						\
	libh__fd = ( f );							\
	fcntl( libh__fd, ( F_SETLK ), & libh__lock ) == 0			\
		? 0								\
		: ({								\
			int libh__errno;					\
			libh__errno = errno;					\
			fcntl( libh__fd, ( F_GETLK ), & libh__lock );		\
			errno = libh__errno;					\
			libh__lock.l_type == ( F_UNLCK )			\
				? -1						\
				: libh__lock.l_pid;				\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	lock_write_wait
//
// purpose	Simple interface to POSIX fcntl() file/record/range locking.
//		Lock a file/record/range for writing, waiting to get lock.
//
// arguments	1 (int) file descriptor of open file to lock
//		2 (off_t) starting byte offset to begin locking
//		3 (off_t) length of record/range to lock
//
// returns	(int) -1 : error
//		(int)  0 : lock has been obtained
//		(int) >0 : process ID already holding lock
//-----------------------------------------------------------------------------
#define lock_write_wait(f,s,l) ({						\
	struct flock libh__lock;						\
	int libh__fd;								\
	libh__lock.l_type = ( F_WRLCK );					\
	libh__lock.l_whence = ( SEEK_SET );					\
	libh__lock.l_start = ( s );						\
	libh__lock.l_len = ( l );						\
	libh__fd = ( f );							\
	fcntl( libh__fd, ( F_SETLKW ), & libh__lock ) == 0			\
		? 0								\
		: ({								\
			int libh__errno;					\
			libh__errno = errno;					\
			fcntl( libh__fd, ( F_GETLK ), & libh__lock );		\
			errno = libh__errno;					\
			libh__lock.l_type == ( F_UNLCK )			\
				? -1						\
				: libh__lock.l_pid;				\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	locked_read_pid
//
// purpose	Simple interface to POSIX fcntl() file/record/range locking.
//		Get the process ID which has the file locked for reading.
//
// arguments	1 (int) file descriptor of open file to check
//		2 (off_t) starting byte offset to record/range to check
//		3 (off_t) length of record/range to check
//
// returns	(int) -1 : error
//		(int)  0 : no process has file/record/range locked
//		(int) >0 : process ID already holding lock
//-----------------------------------------------------------------------------
#define locked_read_pid(f,s,l) ({						\
	struct flock libh__lock;						\
	libh__lock.l_type = ( F_RDLCK );					\
	libh__lock.l_whence = ( SEEK_SET );					\
	libh__lock.l_start = ( s );						\
	libh__lock.l_len = ( l );						\
	fcntl( ( f ), ( F_GETLK ), & libh__lock );				\
	libh__lock.l_type == ( F_UNLCK )					\
		? 0								\
		: libh__lock.l_pid;						\
	})

//-----------------------------------------------------------------------------
// macro	locked_write_pid
//
// purpose	Simple interface to POSIX fcntl() file/record/range locking.
//		Get the process ID which has the file locked for writing.
//
// arguments	1 (int) file descriptor of open file to check
//		2 (off_t) starting byte offset to record/range to check
//		3 (off_t) length of record/range to check
//
// returns	(int) -1 : error
//		(int)  0 : no process has file/record/range locked
//		(int) >0 : process ID already holding lock
//-----------------------------------------------------------------------------
#define locked_write_pid(f,s,l) ({						\
	struct flock libh__lock;						\
	libh__lock.l_type = ( F_WRLCK );					\
	libh__lock.l_whence = ( SEEK_SET );					\
	libh__lock.l_start = ( s );						\
	libh__lock.l_len = ( l );						\
	fcntl( ( f ), ( F_GETLK ), & libh__lock );				\
	libh__lock.l_type == ( F_UNLCK )					\
		? 0								\
		: libh__lock.l_pid;						\
	})

//-----------------------------------------------------------------------------
// macro	lstat_atime
//
// purpose	Return access time of a file.
//
// arguments	1 (const char *) file name
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] access time
//-----------------------------------------------------------------------------
#define lstat_atime(f) ({							\
	struct stat libh__statbuf;						\
	lstat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_atime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_atime_join
//
// purpose	Return the access time of a file where the file name is given
//		as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file access time
//-----------------------------------------------------------------------------
#define lstat_atime_join(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_atime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_atime_join_sep
//
// purpose	Return the access time of a file where the file name is given
//		as multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file access time
//-----------------------------------------------------------------------------
#define lstat_atime_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join_sep( & libh__statbuf, f ) < 0				\
		? (time_t) -1							\
		: libh__statbuf.st_atime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_ctime
//
// purpose	Return change time of a file.
//
// arguments	1 (const char *) file name
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] change time
//-----------------------------------------------------------------------------
#define lstat_ctime(f) ({							\
	struct stat libh__statbuf;						\
	lstat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_ctime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_ctime_join
//
// purpose	Return the inode change time of a file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file inode change time
//-----------------------------------------------------------------------------
#define lstat_ctime_join(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_ctime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_ctime_join_sep
//
// purpose	Return the inode change time of a file where the file name is
//		given as multiple string arguments to be joined together with
//		a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file inode change time
//-----------------------------------------------------------------------------
#define lstat_ctime_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join_sep( & libh__statbuf, f ) < 0				\
		? (time_t) -1							\
		: libh__statbuf.st_ctime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_gid
//
// purpose	Return group ID of a file.
//
// arguments	1 (const char *) file name
//
// returns	(gid_t) == -1 : [failure, see errno]
//		(gid_t) >=  0 : [success] group ID number
//-----------------------------------------------------------------------------
#define lstat_gid(f) ({								\
	struct stat libh__statbuf;						\
	lstat( (f), & libh__statbuf ) < 0					\
		? (gid_t) -1							\
		: libh__statbuf.st_gid;						\
})

//-----------------------------------------------------------------------------
// macro	lstat_gid_join
//
// purpose	Return the group ID number of a file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(gid_t) == -1 : [failure, see errno]
//		(gid_t) >=  0 : [success] file group ID number
//-----------------------------------------------------------------------------
#define lstat_gid_join(f...) ({							\
	struct stat libh__statbuf;						\
	lstat_join( & libh__statbuf, f ) < 0					\
		? (gid_t) -1							\
		: libh__statbuf.st_gid;						\
})

//-----------------------------------------------------------------------------
// macro	lstat_gid_join_sep
//
// purpose	Return the group ID number of a file where the file name is
//		given as multiple string arguments to be joined together with
//		a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(gid_t) == -1 : [failure, see errno]
//		(gid_t) >=  0 : [success] file group ID number
//-----------------------------------------------------------------------------
#define lstat_gid_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join_sep( & libh__statbuf, f ) < 0				\
		? (gid_t) -1							\
		: libh__statbuf.st_gid;						\
})

//-----------------------------------------------------------------------------
// macro	lstat_join
//
// purpose	Return information about a file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (struct stat *) buffer where to store file information
//		2..N (const char *) file name parts
//
// returns	(int) == -1 : [failure, see errno]
//		(int) ==  0 : [success, file info stored] status
//-----------------------------------------------------------------------------
#define lstat_join(b,n...) ((lstat_join)((b),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	lstat_join_sep
//
// purpose      Return information about a file where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments    1 (struct stat *) buffer where to store file information
//              2..N (const char *) file name parts
//
// returns      (int) == -1 : [failure, see errno]
//              (int) ==  0 : [success, file info stored] status
//-----------------------------------------------------------------------------
#define lstat_join_sep(b,n...) ((lstat_join_sep)((b),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	lstat_mode
//
// purpose	Return mode of a file.
//
// arguments	1 (const char *) file name
//
// returns	(mode_t) == -1 : [failure, see errno]
//		(mode_t) >=  0 : [success] file mode
//-----------------------------------------------------------------------------
#define lstat_mode(f) ({							\
	struct stat libh__statbuf;						\
	lstat( (f), & libh__statbuf ) < 0					\
		? (mode_t) -1							\
		: libh__statbuf.st_mode;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_mode_join
//
// purpose	Return mode of a file where the file name is given as
//		multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(mode_t) == -1 : [failure, see errno]
//		(mode_t) >=  0 : [success] file mode
//-----------------------------------------------------------------------------
#define lstat_mode_join(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join( & libh__statbuf, f ) < 0					\
		? (mode_t) -1							\
		: libh__statbuf.st_mode;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_mode_join_sep
//
// purpose      Return mode of a file where the file name is given as multiple
//		string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(mode_t) == -1 : [failure, see errno]
//		(mode_t) >=  0 : [success] file mode
//-----------------------------------------------------------------------------
#define lstat_mode_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join_sep( & libh__statbuf, f ) < 0				\
		? (mode_t) -1							\
		: libh__statbuf.st_mode;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_mtime
//
// purpose	Return modification time of a file.
//
// arguments	1 (const char *) file name
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] modification time
//-----------------------------------------------------------------------------
#define lstat_mtime(f) ({							\
	struct stat libh__statbuf;						\
	lstat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_mtime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_mtime_join
//
// purpose	Return the modification time of a file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file modification time
//-----------------------------------------------------------------------------
#define lstat_mtime_join(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_mtime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_mtime_join_sep
//
// purpose	Return the modification time of a file where the file name is
//		given as multiple string arguments to be joined together with
//		a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file modification time
//-----------------------------------------------------------------------------
#define lstat_mtime_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join_sep( & libh__statbuf, f ) < 0				\
		? (time_t) -1							\
		: libh__statbuf.st_mtime;					\
})

//-----------------------------------------------------------------------------
// macro	lstat_size
//
// purpose	Return size of a file.
//
// arguments	1 (const char *) file name
//
// returns	(long long) == -1 : [failure, see errno]
//		(long long) >=  0 : [success] file size
//
// note		A (long long) type is returned so that if type (off_t) is a
//		smaller unsigned integer, it's full range of values will not
//		collide with the error return value.
//-----------------------------------------------------------------------------
#define lstat_size(f) ({							\
	struct stat libh__statbuf;						\
	lstat( (f), & libh__statbuf ) < 0					\
		? (long long) -1						\
		: (long long) libh__statbuf.st_size;				\
})

//-----------------------------------------------------------------------------
// macro	lstat_size_join
//
// purpose	Return size of a file where the file name is given as multiple
//		string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(long long) == -1 : [failure, see errno]
//		(long long) >=  0 : [success] file size
//
// note		A (long long) type is returned so that if type (off_t) is a
//		smaller unsigned integer, it's full range of values will not
//		collide with the error return value.
//-----------------------------------------------------------------------------
#define lstat_size_join(f) ({							\
	struct stat libh__statbuf;						\
	lstat_join( & libh__statbuf, f ) < 0					\
		? (long long) -1						\
		: (long long) libh__statbuf.st_size;				\
})

//-----------------------------------------------------------------------------
// macro	lstat_size_join_sep
//
// purpose      Return size of a file where the file name is given as multiple
//		string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(long long) == -1 : [failure, see errno]
//		(long long) >=  0 : [success] file size
//
// note		A (long long) type is returned so that if type (off_t) is a
//		smaller unsigned integer, it's full range of values will not
//		collide with the error return value.
//-----------------------------------------------------------------------------
#define lstat_size_join_sep(f) ({						\
	struct stat libh__statbuf;						\
	lstat_join_sep( & libh__statbuf, f ) < 0				\
		? (long long) -1						\
		: (long long) libh__statbuf.st_size;				\
})

//-----------------------------------------------------------------------------
// macro	lstat_uid
//
// purpose	Return owner ID of a file.
//
// arguments	1 (const char *) file name
//
// returns	(uid_t) == -1 : [failure, see errno]
//		(uid_t) >=  0 : [success] owner ID number
//-----------------------------------------------------------------------------
#define lstat_uid(f) ({								\
	struct stat libh__statbuf;						\
	lstat( (f), & libh__statbuf ) < 0					\
		? (uid_t) -1							\
		: libh__statbuf.st_uid;						\
})

//-----------------------------------------------------------------------------
// macro	lstat_uid_join
//
// purpose	Return the owner user ID number of a file where the file name
//		is given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(uid_t) == -1 : [failure, see errno]
//		(uid_t) >=  0 : [success] file owner user ID number
//-----------------------------------------------------------------------------
#define lstat_uid_join(f...) ({							\
	struct stat libh__statbuf;						\
	lstat_join( & libh__statbuf, f ) < 0					\
		? (uid_t) -1							\
		: libh__statbuf.st_uid;						\
})

//-----------------------------------------------------------------------------
// macro	lstat_uid_join_sep
//
// purpose	Return the owner user ID number of a file where the file name
//		is given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(uid_t) == -1 : [failure, see errno]
//		(uid_t) >=  0 : [success] file owner user ID number
//-----------------------------------------------------------------------------
#define lstat_uid_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	lstat_join_sep( & libh__statbuf, f ) < 0				\
		? (uid_t) -1							\
		: libh__statbuf.st_uid;						\
})

//-----------------------------------------------------------------------------
// macro	mkdir_join
//
// purpose	Make a directory where the directory name is given as
//		multiple string arguments to be joined together.
//
// arguments	1 (mode_t) mode for new directory
//		2..N (const char *) directory name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, directory created] status
//-----------------------------------------------------------------------------
#define mkdir_join(m,n...) ((mkdir_join)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	mkdir_join_sep
//
// purpose      Make a directory where the directory name is given as
//		multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments    1 (mode_t) mode for new directory
//              2..N (const char *) directory name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, directory created] status
//-----------------------------------------------------------------------------
#define mkdir_join_sep(m,n...) ((mkdir_join_sep)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	mkfifo_join
//
// purpose	Make a fifo pipe node the fifo name is given as multiple
//		string arguments to be joined together.
//
// arguments	1 (mode_t) mode for new directory
//		2..N (const char *) directory name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, fifo created] status
//-----------------------------------------------------------------------------
#define mkfifo_join(m,n...) ((mkfifo_join)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	mkfifo_join_sep
//
// purpose      Make a fifo pipe node where the fifo name is given as
//		multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments    1 (mode_t) mode for new directory
//              2..N (const char *) directory name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, fifo created] status
//-----------------------------------------------------------------------------
#define mkfifo_join_sep(m,n...) ((mkfifo_join_sep)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	mknod_join
//
// purpose	Make a device node where the node name is given as multiple
//		string arguments to be joined together.
//
// arguments	1 (mode_t) mode for new directory
//		2 (dev_t) device code
//		3..N (const char *) node name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, device node created] status
//-----------------------------------------------------------------------------
#define mknod_join(m,d,n...) ((mknod_join)((m),(d),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	mknod_join_sep
//
// purpose      Make a device node where the node name is given as multiple
//		string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments    1 (mode_t) mode for new directory
//              2 (dev_t) device code
//              3..N (const char *) node name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, device node created] status
//-----------------------------------------------------------------------------
#define mknod_join_sep(m,d,n...) ((mknod_join_sep)((m),(d),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	msg_eprintf_return
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then return
//		to the calling context.
//
// arguments	1 (any) return value
//		2 (const char *) format for message content
//		3-N (...) variable argument list
//-----------------------------------------------------------------------------
#define msg_eprintf_return(r,a...) ({						\
	msg_eprintf(a);								\
	return (r);								\
})
//-----------------------------------------------------------------------------
// macro	msg_veprintf_return
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then return
//		to the calling context.
//
// arguments	1 (any) return value
//		2 (const char *) format for message content
//		3 (va_list) started variable argument list
//-----------------------------------------------------------------------------
#define msg_veprintf_return(r,f,d) ({						\
	msg_veprintf((f),(d));							\
	return (r);								\
})
//-----------------------------------------------------------------------------
// macro	msg_set_local_off
//
// purpose	Set the mode to use the local timezone for message timestamps
//		to off.
//
// arguments	-none-
//
// return	(int) current local mode setting, 0
//-----------------------------------------------------------------------------
#define msg_set_local_off() (msg_set_local(0))

//-----------------------------------------------------------------------------
// macro	msg_set_local_on
//
// purpose	Set the mode to use the local timezone for message timestamps
//		to on.
//
// arguments	-none-
//
// return	(int) current local mode setting, 1
//-----------------------------------------------------------------------------
#define msg_set_local_on() (msg_set_local(1))

//-----------------------------------------------------------------------------
// macro	msg_get_local
//
// purpose	Get the mode to use the local timezone for message timestamps.
//
// arguments	-none-
//
// return	(int) current local mode setting
//-----------------------------------------------------------------------------
#define msg_get_local() (msg_set_local(-1))

//-----------------------------------------------------------------------------
// macro	msg_clear_frac_digits
//
// purpose	Clear the number of fraction digits to be used for the
//		fractions of seconds to be appended to the time_format.
//
// arguments	-none-
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define msg_clear_frac_digits() (msg_set_frac_digits(0))

//-----------------------------------------------------------------------------
// macro	msg_get_frac_digits
//
// purpose	Get the minimum number of fraction digits to be used for the
//		fractions of seconds to be appended to the time_format.
//
// arguments	-none-
//
// returns	(int) minimum number of digits to use
//-----------------------------------------------------------------------------
#define msg_get_frac_digits() (msg_set_frac_digits(-1))

//-----------------------------------------------------------------------------
// macro	msg_clear_time_format
//
// purpose	Clear the time format to be included in messages.
//
// arguments	-none-
//
// returns	(char *) pointer to cleared time format.
//
// WARNING	This setting has process scope.  That means it will apply for
//		all threads in a multi-threading process.
//-----------------------------------------------------------------------------
#define msg_clear_time_format() (msg_set_time_format((const char*)""))

//-----------------------------------------------------------------------------
// macro	msg_get_time_format
//
// purpose	Get the time format to be included in messages.
//
// arguments	-none-
//
// returns	(char *) pointer to copied time format.
//
// WARNING	This setting has process scope.  That means it will apply for
//		all threads in a multi-threading process.
//-----------------------------------------------------------------------------
#define msg_get_time_format() (msg_set_time_format((const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	msg_clear_program_name
//
// purpose	Clear the program name to be included in messages.
//
// arguments	-none-
//
// returns	(char *) pointer to cleared program name
//
// WARNING	This setting has process scope.  That means it will apply for
//		all threads in a multi-threading process.
//-----------------------------------------------------------------------------
#define msg_clear_program_name() (msg_set_program_name((const char*)""))

//-----------------------------------------------------------------------------
// function	msg_get_program_name
//
// purpose	Get the program name to be included in messages.
//
// arguments	-none-
//
// returns	(char *) pointer to copied program name
//
// WARNING	This setting has process scope.  That means it will apply for
//		all threads in a multi-threading process.
//-----------------------------------------------------------------------------
#define msg_get_program_name() (msg_set_program_name((const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	multi_close
//
// purpose	Close multiple file descriptors from a variable argument list.
//
// arguments	1..N (int) file descriptor to close
//
// returns	(int) number of descriptors closed successfully
//-----------------------------------------------------------------------------
#define multi_close(f,a...) ((multi_close)((f),a,-1))

//-----------------------------------------------------------------------------
// macro	open_deep_join
//
// purpose	Open a file and return a file descriptor where the file name is
//		possibly longer than PATH_MAX allows, and is given as multiple
//		string arguments to be joined together.
//
// arguments	1 (int) option flags, e.g. O_RDONLY, O_WRONLY, etc
//		2 (mode_t) mode for created files
//		3..N (const char *) file name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) >= 0 : [success, file opened] file descriptor
//-----------------------------------------------------------------------------
#define open_deep_join(f,m,n...) ((open_deep_join)((f),(m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// function	open_fork_exec
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (int *) where to store file descriptor of write end of pipe
//			which will be the child's stdin.
//		2 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stdout.
//		3 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stderr.
//		4 (const char *) filename of program to execute in child
//		5-N (const char *) arguments to program being executed
//
// returns	(int) -1 if there is an error (see errno)
//		(int) child PID
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same file descriptor as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
#define open_fork_exec(i,o,e,p,a...) (open_fork_exec)((i),(o),(e),(p),a,(const char*)(NULL))

//-----------------------------------------------------------------------------
// macro	open_join
//
// purpose	Open a file and return a file descriptor where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (int) option flags, e.g. O_RDONLY, O_WRONLY, etc
//		2 (mode_t) mode for created files
//		3..N (const char *) file name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) >= 0 : [success, file opened] file descriptor
//-----------------------------------------------------------------------------
#define open_join(f,m,n...) ((open_join)((f),(m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	open_join_sep
//
// purpose	Open a file and return a file descriptor where the file name is
//		given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1 (int) option flags, e.g. O_RDONLY, O_WRONLY, etc
//		2 (mode_t) mode for created files
//		3..N (const char *) file name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) >= 0 : [success, file opened] file descriptor
//-----------------------------------------------------------------------------
#define open_join_sep(f,m,n...) ((open_join_sep)((f),(m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	open_proc_arg
//
// purpose	Open pipe(s) to a new process running the given command and
//		arguments.  The pipe must be closed via close_proc.
//
// arguments	1 (int *) pointer to store parent end of pipe to stdin
//		2 (int *) pointer to store parent end of pipe to stdout
//		3 (int *) pointer to store parent end of pipe to stderr
//		4 (int) file descriptor to use if no pipe for stdin
//		5 (int) file descriptor to use if no pipe for stdout
//		6 (int) file descriptor to use if no pipe for stderr
//		7 (int) highest descriptor to be closed in child
//		8 (int) 1 if wait for child now.
//		9 (const char *) file name to execute
//		10-N (const char *) program arguments as function arguments
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, child already exited, grandchild running
//		(int)  > 0 : OK, PID of child running, must wait later
//
// note		If the pointers given for stdout and stderr are the same,
//		then a single pipe is created and the child will have both
//		stdout and stderr sharing the write end of that pipe.
//
// note		Caller should keep track of the highest file descriptor it
//		has opened and pass it via argument 7 to ensure that the
//		child process closes everything but standard descriptors.
//
// note		If the caller needs to pass other file descriptors to the
//		child, then this function is not appropriate for use.
//-----------------------------------------------------------------------------
#define open_proc_arg(a,b,c,d,e,f,g,h,i,j...) ((open_proc_arg)((a),(b),(c),(d),(e),(f),(g),(h),(i),j,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	opendir_join
//
// purpose	Open a directory and return an open directory handle for
//		reading with readdir(), where the name is given as multiple
//		string arguments to be joined together.
//
// arguments	1..N (const char *) directory name parts
//
// returns	(DIR *) == NULL : [failure, see errno]
//		(DIR *) != NULL : [success, directory opened] handle
//-----------------------------------------------------------------------------
#define opendir_join(n...) ((opendir_join)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	opendir_join_sep
//
// purpose      Open a directory and return an open directory handle for
//              reading with readdir(), where the name is given as multiple
//              string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments    1..N (const char *) directory name parts
//
// returns      (DIR *) == NULL : [failure, see errno]
//              (DIR *) != NULL : [success, directory opened] handle
//-----------------------------------------------------------------------------
#define opendir_join_sep(n...) ((opendir_join_sep)(n,(const char*)(NULL)))


#define put_var_bash(f,e,p,n,s,v) put_var_shell((f),0,(e),(p),(n),(s),(v))
#define put_var_ksh(f,e,p,n,s,v) put_var_shell((f),0,(e),(p),(n),(s),(v))
#define put_var_csh(f,e,p,n,s,v) put_var_shell((f),1,(e),(p),(n),(s),(v))
#define put_var_tcsh(f,e,p,n,s,v) put_var_shell((f),1,(e),(p),(n),(s),(v))

//-----------------------------------------------------------------------------
// macro	readlink_join
//
// purpose	Read the reference data for symbolic link of the given name
//		where the symbolic link name is given as multiple string
//		arguments to be joined together.
//
// arguments	1 (char *) target buffer
//		2 (size_t) size of target buffer
//		3..N (const char *) symbolic link name parts
//
// returns	(ssize_t)  < 0 : [failure, see errno]
//		(ssize_t) >= 0 : [success] number of characters stored
//
// note		Unlike the POSIX/SUS standard readlink() function, this
//		function/macro will properly terminate the symlink data.
//		Thus it will be necessary to provide at least one additional
//		character of buffer space.
//-----------------------------------------------------------------------------
#define readlink_join(b,s,n...) ((readlink_join)((b),(s),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	readlink_join_sep
//
// purpose	Read the reference data for symbolic link of the given name
//		where the symbolic link name is given as multiple string
//		arguments to be joined together with a directory separator
//		between each argument.
//
// arguments	1 (char *) target buffer
//		2 (size_t) size of target buffer
//		3..N (const char *) symbolic link name parts
//
// returns	(ssize_t)  < 0 : [failure, see errno]
//		(ssize_t) >= 0 : [success] number of characters stored
//
// note		Unlike the POSIX/SUS standard readlink() function, this
//		function/macro will properly terminate the symlink data.
//		Thus it will be necessary to provide at least one additional
//		character of buffer space.
//-----------------------------------------------------------------------------
#define readlink_join_sep(b,s,n...) ((readlink_join_sep)((b),(s),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	rmdir_join
//
// purpose	Remove a directory of the given name where the directory name
//		is given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) directory name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, directory removed] status
//-----------------------------------------------------------------------------
#define rmdir_join(n...) ((rmdir_join)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	rmdir_join_sep
//
// purpose      Remove a directory of the given name where the directory name
//              is given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments    1..N (const char *) directory name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, directory removed] status
//-----------------------------------------------------------------------------
#define rmdir_join_sep(n...) ((rmdir_join_sep)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	stat_atime
//
// purpose	Return access time of a file.
//
// arguments	1 (const char *) file name
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] access time
//-----------------------------------------------------------------------------
#define stat_atime(f) ({							\
	struct stat libh__statbuf;						\
	stat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_atime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_atime_join
//
// purpose	Return the access time of a file where the file name is given
//		as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file access time
//-----------------------------------------------------------------------------
#define stat_atime_join(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_atime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_atime_join_sep
//
// purpose	Return the access time of a file where the file name is given
//		as multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file access time
//-----------------------------------------------------------------------------
#define stat_atime_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join_sep( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_atime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_ctime
//
// purpose	Return change time of a file.
//
// arguments	1 (const char *) file name
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] change time
//-----------------------------------------------------------------------------
#define stat_ctime(f) ({							\
	struct stat libh__statbuf;						\
	stat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_ctime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_ctime_join
//
// purpose	Return the inode change time of a file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file inode change time
//-----------------------------------------------------------------------------
#define stat_ctime_join(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_ctime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_ctime_join_sep
//
// purpose	Return the inode change time of a file where the file name is
//		given as multiple string arguments to be joined together with
//		a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file inode change time
//-----------------------------------------------------------------------------
#define stat_ctime_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join_sep( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_ctime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_gid
//
// purpose	Return group ID of a file.
//
// arguments	1 (const char *) file name
//
// returns	(gid_t) == -1 : [failure, see errno]
//		(gid_t) >=  0 : [success] group ID number
//-----------------------------------------------------------------------------
#define stat_gid(f) ({								\
	struct stat libh__statbuf;						\
	stat( (f), & libh__statbuf ) < 0					\
		? (gid_t) -1							\
		: libh__statbuf.st_gid;						\
})

//-----------------------------------------------------------------------------
// macro	stat_gid_join
//
// purpose	Return the group ID number of a file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(gid_t) == -1 : [failure, see errno]
//		(gid_t) >=  0 : [success] file group ID number
//-----------------------------------------------------------------------------
#define stat_gid_join(f...) ({							\
	struct stat libh__statbuf;						\
	stat_join( & libh__statbuf, f ) < 0					\
		? (gid_t) -1							\
		: libh__statbuf.st_gid;						\
})

//-----------------------------------------------------------------------------
// macro	stat_gid_join_sep
//
// purpose	Return the group ID number of a file where the file name is
//		given as multiple string arguments to be joined together with
//		a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(gid_t) == -1 : [failure, see errno]
//		(gid_t) >=  0 : [success] file group ID number
//-----------------------------------------------------------------------------
#define stat_gid_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join_sep( & libh__statbuf, f ) < 0					\
		? (gid_t) -1							\
		: libh__statbuf.st_gid;						\
})

//-----------------------------------------------------------------------------
// macro	stat_join
//
// purpose	Return information about a file where the file name is given
//		as multiple string arguments to be joined together.
//
// arguments	1 (struct stat *) buffer where to store file information
//		2..N (const char *) file name parts
//
// returns	(int) == -1 : [failure, see errno]
//		(int) >=  0 : [success, file info stored] status
//-----------------------------------------------------------------------------
#define stat_join(b,n...) ((stat_join)((b),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	stat_join_sep
//
// purpose      Return information about a file where the file name is given
//		as multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments    1 (struct stat *) buffer where to store file information
//              2..N (const char *) file name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, file info stored] status
//-----------------------------------------------------------------------------
#define stat_join_sep(b,n...) ((stat_join_sep)((b),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	stat_mode
//
// purpose	Return mode of a file.
//
// arguments	1 (const char *) file name
//
// returns	(mode_t) == -1 : [failure, see errno]
//		(mode_t) >=  0 : [success] file mode
//-----------------------------------------------------------------------------
#define stat_mode(f) ({								\
	struct stat libh__statbuf;						\
	stat( (f), & libh__statbuf ) < 0					\
		? (mode_t) -1							\
		: libh__statbuf.st_mode;					\
})

//-----------------------------------------------------------------------------
// macro	stat_mode_join
//
// purpose	Return mode of a file where the file name is given as
//		multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(mode_t) == -1 : [failure, see errno]
//		(mode_t) >=  0 : [success] file mode
//-----------------------------------------------------------------------------
#define stat_mode_join(f...) ({							\
	struct stat libh__statbuf;						\
	stat_join( & libh__statbuf, f ) < 0					\
		? (mode_t) -1							\
		: libh__statbuf.st_mode;					\
})

//-----------------------------------------------------------------------------
// macro	stat_mode_join_sep
//
// purpose      Return mode of a file where the file name is given as multiple
//		string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(mode_t) == -1 : [failure, see errno]
//		(mode_t) >=  0 : [success] file mode
//-----------------------------------------------------------------------------
#define stat_mode_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join_sep( & libh__statbuf, f ) < 0					\
		? (mode_t) -1							\
		: libh__statbuf.st_mode;					\
})

//-----------------------------------------------------------------------------
// macro	stat_mtime
//
// purpose	Return modification time of a file.
//
// arguments	1 (const char *) file name
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) != -1 : [success] modification time
//-----------------------------------------------------------------------------
#define stat_mtime(f) ({							\
	struct stat libh__statbuf;						\
	stat( (f), & libh__statbuf ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_mtime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_mtime_join
//
// purpose	Return the modification time of a file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file modification time
//-----------------------------------------------------------------------------
#define stat_mtime_join(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_mtime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_mtime_join_sep
//
// purpose	Return the modification time of a file where the file name is
//		given as multiple string arguments to be joined together with
//		a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(time_t) == -1 : [failure, see errno]
//		(time_t) >=  0 : [success] file modification time
//-----------------------------------------------------------------------------
#define stat_mtime_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join_sep( & libh__statbuf, f ) < 0					\
		? (time_t) -1							\
		: libh__statbuf.st_mtime;					\
})

//-----------------------------------------------------------------------------
// macro	stat_size
//
// purpose	Return size of a file.
//
// arguments	1 (const char *) file name
//
// returns	(long long) == -1 : [failure, see errno]
//		(long long) >=  0 : [success] file size
//
// note		A (long long) type is returned so that if type (off_t) is a
//		smaller unsigned integer, it's full range of values will not
//		collide with the error return value.
//-----------------------------------------------------------------------------
#define stat_size(f) ({								\
	struct stat libh__statbuf;						\
	stat( (f), & libh__statbuf ) < 0					\
		? (long long) -1						\
		: (long long) libh__statbuf.st_size;				\
})

//-----------------------------------------------------------------------------
// macro	stat_size_join
//
// purpose	Return size of a file where the file name is given as multiple
//		string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(long long) == -1 : [failure, see errno]
//		(long long) >=  0 : [success] file size
//
// note		A (long long) type is returned so that if type (off_t) is a
//		smaller unsigned integer, it's full range of values will not
//		collide with the error return value.
//-----------------------------------------------------------------------------
#define stat_size_join(f) ({							\
	struct stat libh__statbuf;						\
	stat_join( & libh__statbuf, f ) < 0					\
		? (long long) -1						\
		: (long long) libh__statbuf.st_size;				\
})

//-----------------------------------------------------------------------------
// macro	stat_size_join_sep
//
// purpose      Return size of a file where the file name is given as multiple
//		string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(long long) == -1 : [failure, see errno]
//		(long long) >=  0 : [success] file size
//
// note		A (long long) type is returned so that if type (off_t) is a
//		smaller unsigned integer, it's full range of values will not
//		collide with the error return value.
//-----------------------------------------------------------------------------
#define stat_size_join_sep(f) ({						\
	struct stat libh__statbuf;						\
	stat_join_sep( & libh__statbuf, f ) < 0					\
		? (long long) -1						\
		: (long long) libh__statbuf.st_size;				\
})

//-----------------------------------------------------------------------------
// macro	stat_uid
//
// purpose	Return owner ID of a file.
//
// arguments	1 (const char *) file name
//
// returns	(uid_t) == -1 : [failure, see errno]
//		(uid_t) >=  0 : [success] owner ID number
//-----------------------------------------------------------------------------
#define stat_uid(f) ({								\
	struct stat libh__statbuf;						\
	stat( (f), & libh__statbuf ) < 0					\
		? (uid_t) -1							\
		: libh__statbuf.st_uid;						\
})

//-----------------------------------------------------------------------------
// macro	stat_uid_join
//
// purpose	Return the owner user ID number of a file where the file name
//		is given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) file name parts
//
// returns	(uid_t) == -1 : [failure, see errno]
//		(uid_t) >=  0 : [success] file owner user ID number
//-----------------------------------------------------------------------------
#define stat_uid_join(f...) ({							\
	struct stat libh__statbuf;						\
	stat_join( & libh__statbuf, f ) < 0					\
		? (uid_t) -1							\
		: libh__statbuf.st_uid;						\
})

//-----------------------------------------------------------------------------
// macro	stat_uid_join_sep
//
// purpose	Return the owner user ID number of a file where the file name
//		is given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1..N (const char *) file name parts
//
// returns	(uid_t) == -1 : [failure, see errno]
//		(uid_t) >=  0 : [success] file owner user ID number
//-----------------------------------------------------------------------------
#define stat_uid_join_sep(f...) ({						\
	struct stat libh__statbuf;						\
	stat_join_sep( & libh__statbuf, f ) < 0					\
		? (uid_t) -1							\
		: libh__statbuf.st_uid;						\
})

//-----------------------------------------------------------------------------
// macro	time_msec
//
// purpose	Return the current time in milliseconds since the epoch.
//
// arguments	-none-
//
// returns	(long long)  < 0 : [failure, see errno]
//		(long long) >= 0 : [success] time in milliseconds
//-----------------------------------------------------------------------------
#define time_msec(f) ({								\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: ({								\
			long long libh__time;					\
			libh__time = libh__timeval.tv_sec;			\
			libh__time *= 1000LL;					\
			libh__time += ( libh__timeval.tv_usec / 1000L );	\
		})								\
	})

//-----------------------------------------------------------------------------
// macro	time_nsec
//
// purpose	Return the current time in nanoseconds since the epoch.
//
// arguments	-none-
//
// returns	(long long)  < 0 : [failure, see errno]
//		(long long) >= 0 : [success] time in nanoseconds
//-----------------------------------------------------------------------------
#define time_nsec(f) ({								\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: ({								\
			long long libh__time;					\
			libh__time = libh__timeval.tv_sec;			\
			libh__time *= 1000000LL;				\
			libh__time += libh__timeval.tv_usec;			\
			libh__time *= 1000LL;					\
		})								\
	})

//-----------------------------------------------------------------------------
// macro	time_sec
//
// purpose	Return the current time in seconds since the epoch.
//
// arguments	-none-
//
// returns	(long)  < 0 : [failure, see errno]
//		(long) >= 0 : [success] time in seconds
//-----------------------------------------------------------------------------
#define time_sec(f) ({								\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1L								\
		: libh__timeval.tv_sec;						\
	})

//-----------------------------------------------------------------------------
// macro	time_usec
//
// purpose	Return the current time in microseconds since the epoch.
//
// arguments	-none-
//
// returns	(long long)  < 0 : [failure, see errno]
//		(long long) >= 0 : [success] time in microseconds
//-----------------------------------------------------------------------------
#define time_usec(f) ({								\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: ({								\
			long long libh__time;					\
			libh__time = libh__timeval.tv_sec;			\
			libh__time *= 1000000LL;				\
			libh__time += libh__timeval.tv_usec;			\
		})								\
	})

//-----------------------------------------------------------------------------
// macro	truncate_join
//
// purpose	Truncate the named file where the file name is given as
//		multiple string arguments to be joined together.
//
// arguments	1 (off_t) new length for file
//		2..N (const char *) file name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, file truncated] status
//-----------------------------------------------------------------------------
#define truncate_join(m,n...) ((truncate_join)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	truncate_join_sep
//
// purpose	Truncate the named file where the file name is given as
//		multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments    1 (off_t) new length for file
//              2..N (const char *) file name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, mode changed] status
//-----------------------------------------------------------------------------
#define truncate_join_sep(m,n...) ((truncate_join_sep)((m),n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	unlink_join
//
// purpose	Unlink a name where the name is given as multiple string
//		arguments to be joined together.
//
// arguments	1..N (const char *) name parts
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, name unlinked] status
//-----------------------------------------------------------------------------
#define unlink_join(n...) ((unlink_join)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// macro	unlink_join_sep
//
// purpose	Unlink a name where the name is given as multiple string
//		arguments to be joined together with a directory separator
//		between each argument.
//
// arguments    1..N (const char *) name parts
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, name unlinked] status
//-----------------------------------------------------------------------------
#define unlink_join_sep(n...) ((unlink_join_sep)(n,(const char*)(NULL)))

//-----------------------------------------------------------------------------
// function	(chdir_join)
//
// purpose	Change the current directory to the given name where the
//		directory name is given as multiple string arguments to be
//		joined together.
//
// arguments	1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, directory changed] status
//-----------------------------------------------------------------------------
int
(chdir_join) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(chdir_join_sep)
//
// purpose      Change the current directory to the given name where the
//              directory name is given as multiple string arguments to be
//              joined together with a directory separator between each
//		argument.
//
// arguments    1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, directory changed] status
//-----------------------------------------------------------------------------
int
(chdir_join_sep) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(chmod_join)
//
// purpose	Change the mode of a named file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (mode_t) new mode for file
//		2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, mode changed] status
//-----------------------------------------------------------------------------
int
(chmod_join) (
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(chmod_join_sep)
//
// purpose      Change the mode of a named file where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments    1 (mode_t) new mode for file
//              2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, mode changed] status
//-----------------------------------------------------------------------------
int
(chmod_join_sep) (
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(chown_join)
//
// purpose	Change the mode of a named file where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (uid_t) new owner id
//		2 (gid_t) new group id
//		3..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, owner/group changed] status
//-----------------------------------------------------------------------------
int
(chown_join) (
    uid_t	arg_owner
    ,
    gid_t	arg_group
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(chown_join_sep)
//
// purpose      Change the mode of a named file where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1 (uid_t) new owner id
//		2 (gid_t) new group id
//		3..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, owner/group changed] status
//-----------------------------------------------------------------------------
int
(chown_join_sep) (
    uid_t	arg_owner
    ,
    gid_t	arg_group
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(chroot_join)
//
// purpose	Change the root directory to the given name where the
//		directory name is given as multiple string arguments to be
//		joined together.
//
// arguments	1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, root changed] status
//-----------------------------------------------------------------------------
int
(chroot_join) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(chroot_join_sep)
//
// purpose      Change the root directory to the given name where the
//              directory name is given as multiple string arguments to be
//              joined together with a directory separator between each
//		argument.
//
// arguments    1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, root changed] status
//-----------------------------------------------------------------------------
int
(chroot_join_sep) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	close_all_above
//
// purpose	Mass close file descriptors above a specified value.
//
// arguments	1 (int) highest file descriptor to not close
//
// returns	(int) number of descriptors closed successfully
//-----------------------------------------------------------------------------
int
close_all_above (
    int		arg_fd
    )
;

//-----------------------------------------------------------------------------
// macro	close_all_between
//
// purpose	Mass close file descriptors between given values, inclusively.
//
// arguments	1 (int) first file descriptor to close
//		2 (int) last file descriptor to close
//
// returns	(int) number of descriptors closed successfully
//-----------------------------------------------------------------------------
#define close_all_between(first,last) ({					\
	int libh__this_fd;							\
	int libh__last_fd;							\
	int libh__count;							\
	libh__this_fd = (first);						\
	libh__last_fd = (last);							\
	while ( libh__this_fd <= libh__last_fd ) {				\
		if ( close( libh__this_fd ) == 0 ) ++ libh__count; 		\
	}									\
	libh__count;								\
})
//-----------------------------------------------------------------------------
// function	(dlopen_join)
//
// purpose	Open a dynamically linked library where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 ((*)()) function pointer to dlopen passed by macro
//		2 (int) option flags, e.g. RTLD_LAZY, RTLD_NOW, RTLD_GLOBAL
//		3..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(void *) == NULL : [failure, see errno]
//		(void *) != NULL : [success, library open] handle
//
// note		The reference to dlopen is passed from the calling program via
//		the wrapper macro in argument one so that LIBH itself does not
//		generate a reference to libdl.
//-----------------------------------------------------------------------------
void *
(dlopen_join) (
    void *	( *	arg_dlopen	)( const char *, int )
    ,
    int			arg_flags
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(dlopen_join_sep)
//
// purpose      Open a dynamically linked library where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1 ((*)()) function pointer to dlopen passed by macro
//		2 (int) option flags, e.g. RTLD_LAZY, RTLD_NOW, RTLD_GLOBAL
//		3..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (void *) == NULL : [failure, see errno]
//              (void *) != NULL : [success, library open] handle
//
// note		The reference to dlopen is passed from the calling program via
//		the wrapper macro in argument one so that LIBH itself does not
//		generate a reference to libdl.
//-----------------------------------------------------------------------------
void *
(dlopen_join_sep) (
    void *	( *	arg_dlopen	)( const char *, int )
    ,
    int			arg_flags
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(dlsym_join)
//
// purpose	Return the address of a symbol in the open library specified
//		by its handle, where the symbol name is given as multiple
//		string arguments to be joined together.
//
// arguments	1 ((*)()) function pointer to dlsym passed by macro
//		2 (void *) open library handle
//		3..N (const char *) symbol name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(void *) == NULL : [failure, see errno]
//		(void *) != NULL : [success, symbol found] pointer
//
// note		The reference to dlsym is passed from the calling program via
//		the wrapper macro in argument one so that LIBH itself does not
//		generate a reference to libdl.
//-----------------------------------------------------------------------------
void *
(dlsym_join) (
    void *	( *	arg_dlsym	)( void *, char * )
    ,
    void *		arg_lib
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(dlsym_join_sep)
//
// purpose	Return the address of a symbol in the open library specified
//		by its handle, where the symbol name is given as multiple
//		string arguments to be joined together with an underscore
//		separator between each argument.
//
// arguments	1 ((*)()) function pointer to dlopen passed by macro
//		2 (void *) open library handle
//		3..N (const char *) symbol name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(void *) == NULL : [failure, see errno]
//		(void *) != NULL : [success, symbol found] pointer
//
// note		The reference to dlsym is passed from the calling program via
//		the wrapper macro in argument one so that LIBH itself does not
//		generate a reference to libdl.
//-----------------------------------------------------------------------------
void *
(dlsym_join_sep) (
    void *	( *	arg_dlsym	)( void *, char * )
    ,
    void *		arg_lib
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	fopen_fork
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (FILE * *) where to store FILE pointer of write end of pipe
//			which will be the child's stdin.
//		2 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stdout.
//		3 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stderr.
//
// returns	(int) -1 if there is an error (see errno)
//		(int) 0 when returning in the child process
//		(int) child PID when returning in the parent process
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same FILE pointer as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
int
fopen_fork (
    FILE * *		arg_file_ptr_stdin
    ,
    FILE * *		arg_file_ptr_stdout
    ,
    FILE * *		arg_file_ptr_stderr
    )
;

//-----------------------------------------------------------------------------
// function	(fopen_fork_exec)
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (FILE * *) where to store FILE pointer of write end of pipe
//			which will be the child's stdin.
//		2 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stdout.
//		3 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stderr.
//		4 (const char *) filename of program to execute in child
//		5-N (const char *) arguments to program being executed ended
//			with a NULL argument
//
// returns	(int) -1 if there is an error (see errno)
//		(int) child PID
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same FILE pointer as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
int
(fopen_fork_exec) (
    FILE * *		arg_file_ptr_stdin
    ,
    FILE * *		arg_file_ptr_stdout
    ,
    FILE * *		arg_file_ptr_stderr
    ,
    const char *	arg_execute
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	fopen_fork_execv
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (FILE * *) where to store FILE pointer of write end of pipe
//			which will be the child's stdin.
//		2 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stdout.
//		3 (FILE * *) where to store FILE pointer of read end of pipe
//			which will be the child's stderr.
//		4 (const char *) filename of program to execute in child
//		5 (char * const *) arguments to program being executed
//		6 (char * const *) environment variables to pass to program
//			or NULL to pass current environment
//
// returns	(int) -1 if there is an error (see errno)
//		(int) child PID
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same FILE pointer as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
int
fopen_fork_execv (
    FILE * *		arg_file_ptr_stdin
    ,
    FILE * *		arg_file_ptr_stdout
    ,
    FILE * *		arg_file_ptr_stderr
    ,
    const char *	arg_execute
    ,
    char * const *	arg_argv
    ,
    char * const *	arg_envp
    )
;

//-----------------------------------------------------------------------------
// function	(fopen_join)
//
// purpose	Open a file and return a FILE pointer where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (const char *) mode, e.g. "r", "w+", etc.
//		2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(FILE *) == NULL : [failure, see errno]
//		(FILE *) != NULL : [success, file open] FILE pointer
//-----------------------------------------------------------------------------
FILE *
(fopen_join) (
    const char *	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(fopen_join_sep)
//
// purpose	Open a file and return a FILE pointer where the file name is
//		given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1 (const char *) mode, e.g. "r", "w+", etc.
//		2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(FILE *) == NULL : [failure, see errno]
//		(FILE *) != NULL : [success, file open] FILE pointer
//-----------------------------------------------------------------------------
FILE *
(fopen_join_sep) (
    const char *	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	fopen_proc
//
// purpose	Create a pipe to a child process to read from its stdout (and
//		optionally stderr) or to write to its stdin, with the child
//		process running the given command and arguments.
//
// arguments	1 (const char *) option string
//		2 (const char *) file name to execute
//		3 (const char * const *) array of arguments
//
// returns	(FILE *) NULL : error
//		(FILE *) file pointer to pipe.
//
// options	"r" open to read from stdout (and stderr if not inherited)
//		"w" open to write to stdin
//		"i" inherit stdin from parent
//		"o" inherit stdout from parent
//		"e" inherit stderr from parent
//
// warning	Do not use pclose() on the stream returned by this function.
//		It works differently than popen() and is not compatible with
//		pclose().
//-----------------------------------------------------------------------------
FILE *
fopen_proc (
    const char *		arg_options
    ,
    const char *		arg_exec_name
    ,
    char * const *		arg_exec_args
    )
;

//-----------------------------------------------------------------------------
// function	fopen_proc_arg
//
// purpose	Create a pipe to a child process to read from its stdout (and
//		optionally stderr) or to write to its stdin, with the child
//		process running the given command and arguments.
//
// arguments	1 (const char *) option string
//		2 (const char *) file name to execute
//		3-N (const char *) program arguments as function arguments
//		N+1 (const char *) NULL to indicate end of arguments
//
// returns	(FILE *) NULL : error
//		(FILE *) file pointer to pipe.
//
// options	"r" open to read from stdout (and stderr if not inherited)
//		"w" open to write to stdin
//		"i" inherit stdin from parent
//		"o" inherit stdout from parent
//		"e" inherit stderr from parent
//
// warning	Do not use pclose() on the stream returned by this function.
//		It works differently than popen() and is not compatible with
//		pclose().
//-----------------------------------------------------------------------------
FILE *
(fopen_proc_arg) (
    const char *		arg_options
    ,
    const char *		arg_exec_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	fput_byte
//
// purpose	Output a byte, escaped as needed, to the specified open file.
//
// arguments	1 (int) point to beginning of a byte
//		2 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_byte (
    int			arg_byte
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	fput_byte_xx
//
// purpose	Output a byte, escaped as needed, to the specified open file.
//
// arguments	1 (int) point to beginning of a byte
//		2 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_byte_xx (
    int			arg_byte
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	fput_bytes_end
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to the specified open file.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (const char *) point to end of bytes (after last)
//		3 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_bytes_end (
    const char *	arg_ptr
    ,
    const char *	arg_end
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	fput_bytes_end_xx
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to the specified open file.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (const char *) point to end of bytes (after last)
//		3 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_bytes_end_xx (
    const char *	arg_ptr
    ,
    const char *	arg_end
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	fput_bytes_len
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to the specified open file.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//		3 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_bytes_len (
    const char *	arg_ptr
    ,
    size_t		arg_len
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	fput_bytes_len_xx
//
// purpose	Output bytes, escaped as needed, for the specified length,
//		to the specified open file.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//		3 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_bytes_len_xx (
    const char *	arg_ptr
    ,
    size_t		arg_len
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	fput_cwd
//
// purpose	Output the current directory string to the specified file.
//
// arguments	1 (FILE *) which file to write to, or NULL for stdout
//
// returns	(size_t) length of string written
//-----------------------------------------------------------------------------
int
fput_cwd (
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	fput_dirfd
//
// purpose	Output the path of a directory given as an open descriptor
//		to the specified file.
//
// arguments	1 (int) file descriptor of directory
//		2 (FILE *) which file to write to, or NULL for stdout
//
// returns	(size_t) length of string written
//-----------------------------------------------------------------------------
int
fput_dirfd (
    int			arg_dirfd
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	ft_copy_set_program_name
//
// purpose	Set the program name for error messages in ft_copy().
//		Otherwise the error messages will indicate the function name.
//
// arguments	1 (const char *) name of program
//
// returns	(int)  0 : OK
//		(int) -1 : error
//-----------------------------------------------------------------------------
int
ft_copy_set_program_name (
    const char *	arg_program_name
    )
;

//-----------------------------------------------------------------------------
// function	ft_copy_set_buffer_size
//
// purpose	Set the buffer size for regular files to be copied with.  If
//		the specified size is not exactly a multiple of the page size,
//		then the size will be rounded up to such a multiple.  If no
//		page size is specified, or if 0 is specified, a default value
//		is used.
//
// arguments	1 (size_t) buffer size
//
// returns	(int)  0 : OK
//		(int) -1 : error
//-----------------------------------------------------------------------------
int
ft_copy_set_buffer_size (
    size_t		arg_buffer_size
    )
;

//-----------------------------------------------------------------------------
// function	ft_copy_set_continue
//
// purpose	Set the continue option to on or off.
//
// arguments	1 (int) 0 for no continue, 1 for continue
//
// returns	(int)  0 : OK
//		(int) -1 : error
//-----------------------------------------------------------------------------
#define ft_copy_set_continue_off()	ft_copy_set_continue(0)
#define ft_copy_set_continue_on()	ft_copy_set_continue(1)
int
ft_copy_set_continue (
    int			arg_cont
    )
;

//-----------------------------------------------------------------------------
// function	ft_copy_set_list
//
// purpose	Set the list option to on or off.
//
// arguments	1 (int) 0 for no list, 1 for list
//
// returns	(int)  0 : OK
//		(int) -1 : error
//-----------------------------------------------------------------------------
#define ft_copy_set_list_off()	ft_copy_set_list(0)
#define ft_copy_set_list_on()	ft_copy_set_list(1)
int
ft_copy_set_list (
    int			arg_list
    )
;

//-----------------------------------------------------------------------------
// function	ft_copy_set_sort
//
// purpose	Set the sort option to on or off.
//
// arguments	1 (int) 0 for no sort, 1 for sort
//
// returns	(int)  0 : OK
//		(int) -1 : error
//-----------------------------------------------------------------------------
#define ft_copy_set_sort_off()	ft_copy_set_sort(0)
#define ft_copy_set_sort_on()	ft_copy_set_sort(1)
int
ft_copy_set_sort (
    int			arg_sort
    )
;

//-----------------------------------------------------------------------------
// function	ft_copy_set_mmap
//
// purpose	Set the mmap option to on or off.
//
// arguments	1 (int) 0 for no mmap, 1 for mmap
//
// returns	(int)  0 : OK
//		(int) -1 : error
//-----------------------------------------------------------------------------
#define ft_copy_set_mmap_off()	ft_copy_set_mmap(0)
#define ft_copy_set_mmap_on()	ft_copy_set_mmap(1)
int
ft_copy_set_mmap (
    int			arg_mmap
    )
;

//-----------------------------------------------------------------------------
// function	ft_copy
//
// purpose	Copy a file tree from source to target.
//
// arguments	1 (int) open file descriptor of source directory
//		2 (const char *) name of source object
//		3 (int) open file descriptor of target directory
//		4 (const char *) name of target object
//		
// returns	(int)  0 : object copied successfully
//		(int) -1 : error copying object
//
// WARNING	This function is NOT thread safe for POSIX threads because
//		it changes the current directory and POSIX threads does not
//		provide for each thread to have its own current directory.
//-----------------------------------------------------------------------------
int
ft_copy (
    int			arg_src_fd
    ,
    const char *	arg_src_name
    ,
    int			arg_tgt_fd
    ,
    const char *	arg_tgt_name
    )
;

//-----------------------------------------------------------------------------
// function	get_config_item
//
// purpose	Get the next config item, storing the name and data into the
//		given targets.
//
// arguments	1 (char *) pointer to name target string
//		2 (size_t) maximum space in name target string
//		3 (size_t *) where to store actual name length
//		4 (char *) pointer to data target string
//		5 (size_t) maximum space in data target string
//		6 (size_t *) where to store actual data length
//		7 (FILE *) pointer to open file to read from
//		8 (int *) where to store truncation status
//
// returns	(int) == -3 : error
//		(int) == -2 : config item not read due to end of file
//		(int) ==  1 : config item read, name only
//		(int) ==  2 : config item read, name and data
//
// note		Lengths are stored for the caller because the name and/or data
//		may have any octet code value, including 0.  A 0 octet is stored
//		at the end of the name and data for the usual case of non-binary
//		null-terminated strings.
//
// note		Single or double quotes are supported without nesting them.
//		Quoted strings must be ended by the same quote.  Mismatched
//		quotes within just become data.
//
// note		Sequences of space characters (space and tab) are reduced to
//		a single space character, unless enclosed in quotes.
//
// note		Conversion of character sequences beginning with backslash are
//		supported for both certain escape codes as well as conversion
//		of octal.
//
// note		Conversion of character sequences beginning with percent sign
//		are supported for conversion of cetal/hexadecimal.
//
// note		Conversion of character sequences beginning with carat are
//		supported for encoding of control characters.
//
// note		An end of line is considered to be any combination and order
//		of not more than one instance each of the four line breaking
//		characters:
//			'\n' (newline)
//			'\r' (carriage return)
//			'\f' (form feed)
//			'\v' (vertical tab)
//		If a character is encountered a 2nd time, it is considered to
//		begin a 2nd line ending, and hence an empty line.
//
// note		If the very first character of a line is a '+' then it is a
//		continuation of the previous line as if the preceeding new
//		line sequence and the following '+' were not present.
//-----------------------------------------------------------------------------
int
get_config_item (
    char *	arg_name_ptr
    ,
    size_t	arg_name_len
    ,
    size_t *	arg_name_len_ptr
    ,
    char *	arg_data_ptr
    ,
    size_t	arg_data_len
    ,
    size_t *	arg_data_len_ptr
    ,
    FILE *	arg_file
    ,
    int *	arg_trunc_ptr
    )
;

//-----------------------------------------------------------------------------
// function	get_config_map
//
// purpose	Get a config stream and return a loaded mapping.
//
// arguments	1 (MAP *) optional mapping to load, else one is created
//		2 (FILE *) pointer to open file to read from
//		3 (size_t) size to buffer item name
//		4 (size_t) size to buffer item data
//		5 (int) options as follows:
//			GET_CONFIG_MAP_DUP : allow duplicate entries
//
// returns	(MAP) the given or created mapping
//		(MAP) NULL : error
//
// note		Single or double quotes are supported without nesting them.
//		Quoted strings must be ended by the same quote.  Mismatched
//		quotes within just become data.
//
// note		Sequences of space characters (space and tab) are reduced to
//		a single space character, unless enclosed in quotes.
//
// note		Conversion of character sequences beginning with backslash are
//		supported for both certain escape codes as well as conversion
//		of octal.
//
// note		Conversion of character sequences beginning with percent sign
//		are supported for conversion of cetal/hexadecimal.
//
// note		Conversion of character sequences beginning with carat are
//		supported for encoding of control characters.
//
// note		An end of line is considered to be any combination and order
//		of not more than one instance each of the four line breaking
//		characters:
//			'\n' (newline)
//			'\r' (carriage return)
//			'\f' (form feed)
//			'\v' (vertical tab)
//		If a character is encountered a 2nd time, it is considered to
//		begin a 2nd line ending, and hence an empty line.
//
// note		If the very first character of a line is a '+' then it is a
//		continuation of the previous line as if the preceeding new
//		line sequence and the following '+' were not present.
//-----------------------------------------------------------------------------
#define GET_CONFIG_MAP_DUP	0x0001

MAP
get_config_map (
    MAP		arg_map
    ,
    FILE *	arg_file
    ,
    size_t	arg_name_len
    ,
    size_t	arg_data_len
    ,
    int		arg_options
    )
;

//-----------------------------------------------------------------------------
// function	get_item_add_options
//
// purpose	Specify a set of options to enable, adding to any that are
//		already enabled.  Twice enabling an option is not an error.
//		A NULL string is like an empty string and adds nothing.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//		2 (const char *) option string of options to add
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
get_item_add_options (
    get_item_p		arg_handle
    ,
    const char *	arg_options
    )
;

//-----------------------------------------------------------------------------
// function	get_item_set_options
//
// purpose	Specify or change scanning options via an option character
//		string.  All previous options are replaced by the given
//		options.  A NULL string restores the defaults.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//		2 (const char *) option string
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
get_item_set_options (
    get_item_p		arg_handle
    ,
    const char *	arg_options
    )
;

//-----------------------------------------------------------------------------
// function	get_item_del_options
//
// purpose	Specify a set of options to disable, if already enabled.
//		Disabling an option that is already disabled is not an error.
//		A NULL string is like an empty string and deletes nothing.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//		2 (const char *) option string of options to delete.
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
get_item_del_options (
    get_item_p		arg_handle
    ,
    const char *	arg_options
    )
;

//-----------------------------------------------------------------------------
// function	get_item_set_hold
//
// purpose	Set the hold-at-eol mode, which causes get_item_next() to
//		always return an EOL status on the same line once an EOL
//		is reached, in effect being stuck on the same line and not
//		getting past the newline, until after get_item_flush_line()
//		is called.  This mode is useful for certain multi-context
//		parsing where a nested context needs to exit back to the
//		previous context when it reaches EOL in such a way that
//		previous context sees the same EOL.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
get_item_set_hold (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_unset_hold
//
// purpose	UNset the hold-at-eol mode, which causes get_item_next() to
//		no longer always return an EOL status on the same line once
//		an EOL is reached, in effect no longer being stuck on the
//		same line and now being able to get past the newline without
//		having to call get_item_flush_line().  This mode is not
//		useful most of the time other that for certain multi-context
//		parsing where a nested context needs to exit back to the
//		previous context when it reaches EOL in such a way that
//		previous context sees the same EOL.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
get_item_unset_hold (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_new_fd
//
// purpose	Create a new handle to an item reading instance to keep the
//		reading state, and associate it with an open file descriptor.
//
// arguments	1 (int) open file descriptor to read from
//
// returns	(get_item_p) pointer handle to new item reading instance.
//-----------------------------------------------------------------------------
get_item_p
get_item_new_fd (
    int			arg_fd
    )
;

//-----------------------------------------------------------------------------
// function	get_item_new_file
//
// purpose	Create a new handle to an item reading instance to keep the
//		reading state, and associate it with an open standard file.
//
// arguments	1 (FILE *) open file to read from
//
// returns	(get_item_p) pointer handle to new item reading instance.
//-----------------------------------------------------------------------------
get_item_p
get_item_new_file (
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	get_item_new_string
//
// purpose	Create a new handle to an item reading instance to keep the
//		reading state, and associate it with a character string that
//		contains the data to be treated as input.
//
// arguments	1 (const char *) string to read from
//		2 (size_t) length of string to read from (or ~0 with null)
//
// returns	(get_item_p) pointer handle to new item reading instance.
//-----------------------------------------------------------------------------
get_item_p
get_item_new_string (
    const char *	arg_str
    ,
    size_t		arg_len
    )
;

//-----------------------------------------------------------------------------
// function	get_item_new_call
//
// purpose	Create a new handle to an item reading instance to keep the
//		reading state, and associate it with a function callback.
//
// arguments	1 (int (*)(void *)) pointer to callback function
//		2 (void *) arbitrary pointer to pass to callback function
//
// returns	(get_item_p) pointer handle to new item reading instance.
//-----------------------------------------------------------------------------
get_item_p
get_item_new_call (
    int ( *		arg_fun		)(void *)
    ,
    void *		arg_arg
    )
;

//-----------------------------------------------------------------------------
// function	get_item_destroy
//
// purpose	Destroy a get_item instance that is no longer needed.
//		The handle is no longer valid after this function is called.
//
// arguments	1 (get_item_p) pointer handle to instance to destroy
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
get_item_destroy (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_begin_cnt
//
// purpose	Get the beginning position of the most recently retrieved item
//		as the number of characters from the beginning of the input
//		file as read by get_item_next().  The first character will be
//		returned as the value 0.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(long long) == -1 : error
//		(long long) >=  0 : item position in input
//-----------------------------------------------------------------------------
long long
get_item_begin_cnt (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_begin_col
//
// purpose	Get the beginning position of the most recently retrieved item
//		as the number of characters from the beginning of the line the
//		item began on.  The first column will be returned as the value
//		0.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(long) == -1 : error
//		(long) >=  0 : item position in input
//-----------------------------------------------------------------------------
long
get_item_begin_col (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_begin_row
//
// purpose	Get the beginning position of the most recently retrieved item
//		as the number of the row the item began on.  The first row will
//		be returned as the value 0.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(long) == -1 : error
//		(long) >=  0 : item position in input
//-----------------------------------------------------------------------------
long
get_item_begin_row (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_end_cnt
//
// purpose	Get the ending position of the most recently retrieved item
//		as the number of characters from the beginning of the input
//		file as read by get_item_next().  The first character will be
//		returned as the value 0.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(long long) == -1 : error
//		(long long) >=  0 : item position in input
//-----------------------------------------------------------------------------
long long
get_item_end_cnt (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_end_col
//
// purpose	Get the ending position of the most recently retrieved item
//		as the number of characters from the beginning of the line the
//		item began on.  The first column will be returned as the value
//		0.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(long) == -1 : error
//		(long) >=  0 : item position in input
//-----------------------------------------------------------------------------
long
get_item_end_col (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_end_row
//
// purpose	Get the ending position of the most recently retrieved item
//		as the number of the row the item began on.  The first row will
//		be returned as the value 0.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(long) == -1 : error
//		(long) >=  0 : item position in input
//-----------------------------------------------------------------------------
long
get_item_end_row (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_actual_len
//
// purpose	Get the actual data length of the most recently retrieved item.
//		The actual data length is the length of the data item after
//		conversion regardless of the size of the space it was being
//		stored in.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(size_t) == ~0 : length unavailable, bad handle
//		(size_t) != ~0 : actual data item length
//-----------------------------------------------------------------------------
size_t
get_item_actual_len (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_stored_len
//
// purpose	Get the stored data length of the most recently retrieved item.
//		The stored data length is the length of the data item after
//		conversion, as truncated to fit the available space it was
//		being stored in.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(size_t) == ~0 : length unavailable, bad handle
//		(size_t) != ~0 : stored data item length
//-----------------------------------------------------------------------------
size_t
get_item_stored_len (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_item_next
//
// purpose	Get the next input space delimited item, storing the item
//		string into the given target.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//		2 (char *) pointer to target string to copy item string to
//		3 (size_t) maximum available target space
//
// returns	(int) == -4 : error, invalid arguments
//		(int) == -3 : item not read due to file I/O error
//		(int) == -2 : item not read due to end of file
//		(int) == -1 : item not read due to end of line
//		(int) ==  0 : item read and stored complete
//		(int) ==  1 : item read and stored truncated
//-----------------------------------------------------------------------------
// note		If the item space target is not large enough, then truncation
//		will occur and the return code will be 1 (instead of 0).
//		The complete item will be read from the file as if the space
//		were large enough, so the next call to get_item_next() will
//		correctly return the next item.  The truncated item will be
//		properly terminated at the end of the available space.
//-----------------------------------------------------------------------------
int
get_item_next (
    get_item_p		arg_handle
    ,
    char *		arg_item_ptr
    ,
    size_t		arg_item_len
    )
;

//-----------------------------------------------------------------------------
// function	get_item_flush_line
//
// purpose	Flush the current line.
//
// arguments	1 (get_item_p) handle pointer to get_item instance
//
// returns	(int) == -4 : error, invalid arguments
//		(int) == -3 : item not read due to file I/O error
//		(int) == -2 : item not read due to end of file
//		(int) ==  0 : line flushed OK
//-----------------------------------------------------------------------------
int
get_item_flush_line (
    get_item_p		arg_handle
    )
;

//-----------------------------------------------------------------------------
// function	get_line_file
//
// purpose	Get the next line, ended by the newline character (or formfeed
//		or vertical tab), from an open FILE, with any carriage returns
//		truncated, copied to the specified target buffer up to one
//		one character less than the size given (plus a null character
//		always stored at the end of the stored string even if the line
//		is too long and the stored string is truncated).
//
// arguments	1 (char *) pointer to target buffer
//		2 (size_t) size of target buffer
//		3 (FILE *) pointer to open file to read from, or stdin
//
// returns	(ssize_t) == -2 : error
//		(ssize_t) == -1 : end of file
//              (ssize_t) >=  0 : length of line
//
// note		The length returned is the length of the actual line, up to
//		any trailing newline-class characters, NOT the length of the
//		string actually stored.  Use strlen() to get the stored
//		length.
//-----------------------------------------------------------------------------
ssize_t
get_line_file (
    char *	arg_target
    ,
    size_t	arg_length
    ,
    FILE *	arg_file
    )
;

//-----------------------------------------------------------------------------
// function	(link_join)
//
// purpose	Link one name as another where the names are given as multiple
//		string arguments to be joined together.
//
// arguments	1..N (const char *) name parts for source name
//		N+1 (const char *) NULL to indicate end of source list
//		N+2..M (const char *) name parts for target name
//		M+1 (const char *) NULL to indicate end of target list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, name unlinked] status
//-----------------------------------------------------------------------------
int
(link_join) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(lstat_join)
//
// purpose	Return information about a file where the file name is
//		given as multiple string arguments to be joined together
//		ended by a NULL argument.
//
// arguments	1 (struct stat *) buffer where to store file information
//		2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int) == -1 : [failure, see errno]
//		(int) ==  0 : [success, file info stored] status
//-----------------------------------------------------------------------------
int
(lstat_join) (
    struct stat *	arg_buf
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(lstat_join_sep)
//
// purpose      Return information about a file where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument ended by a
//		NULL argument.
//
// arguments    1 (struct stat *) buffer where to store file information
//              2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int) == -1 : [failure, see errno]
//              (int) ==  0 : [success, file info stored] status
//-----------------------------------------------------------------------------
int
(lstat_join_sep) (
    struct stat *	arg_buf
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	make_parent_dir
//
// purpose	Make sure the parent directory exists for the given path.
//
// arguments	1 (const char *) pathname of object a parent is needed for
//		2 (int) directory access mode
//
// returns	(int)  <  0 : error returned by first mkdir()
//		(int) == -1 : error
//		(int) ==  0 : parent directory is created
//		(int) ==  1 : parent directory existed
//		(int) ==  2 : path does not need a parent
//
// WARNING	This function may be unsafe when executed as root for the
//		purpose of creating directories for the real user.  Swap
//		the euid and ruid first.
//-----------------------------------------------------------------------------
int
make_parent_dir (
    const char *	arg_path
    ,
    int			arg_mode
    )
;

//-----------------------------------------------------------------------------
// function	(mkdir_join)
//
// purpose	Make a directory where the directory name is given as
//		multiple string arguments to be joined together.
//
// arguments	1 (mode_t) mode for new directory
//		2..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, directory created] status
//-----------------------------------------------------------------------------
int
(mkdir_join) (
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(mkdir_join_sep)
//
// purpose      Make a directory where the directory name is given as
//		multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments    1 (mode_t) mode for new directory
//              2..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, directory created] status
//-----------------------------------------------------------------------------
int
(mkdir_join_sep) (
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(mkfifo_join)
//
// purpose	Make a fifo pipe node the fifo name is given as multiple
//		string arguments to be joined together.
//
// arguments	1 (mode_t) mode for new directory
//		2..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, fifo created] status
//-----------------------------------------------------------------------------
int
(mkfifo_join) (
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(mkfifo_join_sep)
//
// purpose      Make a fifo pipe node where the fifo name is given as
//		multiple string arguments to be joined together with a
//		directory separator between each argument.
//
// arguments    1 (mode_t) mode for new directory
//              2..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, fifo created] status
//-----------------------------------------------------------------------------
int
(mkfifo_join_sep) (
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(mknod_join)
//
// purpose	Make a device node where the node name is given as multiple
//		string arguments to be joined together.
//
// arguments	1 (mode_t) mode for new directory
//		2 (dev_t) device code
//		3..N (const char *) node name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, device node created] status
//-----------------------------------------------------------------------------
int
(mknod_join) (
    mode_t	arg_mode
    ,
    dev_t	arg_dev
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(mknod_join_sep)
//
// purpose      Make a device node where the node name is given as multiple
//		string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments    1 (mode_t) mode for new directory
//              2 (dev_t) device code
//              3..N (const char *) node name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, device node created] status
//-----------------------------------------------------------------------------
int
(mknod_join_sep) (
    mode_t      arg_mode
    ,
    dev_t       arg_dev
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	msg_eprintf
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr.
//
// arguments	1 (const char *) format for message content
//		2 (...) variable argument list
//
// returns	(int) number of characters printed
//-----------------------------------------------------------------------------
int
msg_eprintf (
    const char *	arg_format
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	msg_eprintf__exit
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then exit the
//		process.
//
// arguments	1 (int) exit code
//		2 (const char *) format for message content
//		3-N (...) variable argument list
//-----------------------------------------------------------------------------
void
msg_eprintf__exit (
    int			arg_exit
    ,
    const char *	arg_format
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	msg_eprintf_abort
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then abort the
//		process.
//
// arguments	1 (const char *) format for message content
//		2-N (...) variable argument list
//-----------------------------------------------------------------------------
void
msg_eprintf_abort (
    const char *	arg_format
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	msg_eprintf_exit
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then exit the
//		process.
//
// arguments	1 (int) exit code
//		2 (const char *) format for message content
//		3-N (...) variable argument list
//-----------------------------------------------------------------------------
void
msg_eprintf_exit (
    int			arg_exit
    ,
    const char *	arg_format
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	msg_fprintf
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to the specified file.
//
// arguments	1 (FILE *) destination output stream
//		2 (const char *) format for message content
//		3 (...) variable argument list
//
// returns	(int) number of characters printed
//-----------------------------------------------------------------------------
int
msg_fprintf (
    FILE *		arg_file
    ,
    const char *	arg_format
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	msg_printf
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stdout.
//
// arguments	1 (const char *) format for message content
//		2 (...) variable argument list
//
// returns	(int) number of characters printed
//-----------------------------------------------------------------------------
int
msg_printf (
    const char *	arg_format
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	msg_veprintf
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr.
//
// arguments	1 (const char *) format for message content
//		2 (va_list) started variable argument list
//
// returns	(int) number of characters printed
//-----------------------------------------------------------------------------
int
msg_veprintf (
    const char *	arg_format
    ,
    va_list		arg_va_list
    )
;

//-----------------------------------------------------------------------------
// function	msg_veprintf_exit
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then exit the
//		process.
//
// arguments	1 (int) exit code
//		2 (const char *) format for message content
//		3 (va_list) started variable argument list
//-----------------------------------------------------------------------------
void
msg_veprintf__exit (
    int			arg_exit
    ,
    const char *	arg_format
    ,
    va_list		arg_va_list
    )
;

//-----------------------------------------------------------------------------
// function	msg_veprintf_abort
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then abort the
//		process.
//
// arguments	1 (const char *) format for message content
//		2 (va_list) started variable argument list
//-----------------------------------------------------------------------------
void
msg_veprintf_abort (
    const char *	arg_format
    ,
    va_list		arg_va_list
    )
;

//-----------------------------------------------------------------------------
// function	msg_veprintf_exit
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stderr, then exit the
//		process.
//
// arguments	1 (int) exit code
//		2 (const char *) format for message content
//		3 (va_list) started variable argument list
//-----------------------------------------------------------------------------
void
msg_veprintf_exit (
    int			arg_exit
    ,
    const char *	arg_format
    ,
    va_list		arg_va_list
    )
;

//-----------------------------------------------------------------------------
// function	msg_set_local
//
// purpose	Set the mode to use the local timezone for message timestamps
//		to on or off based on the given value.
//
// arguments	(int) -1: do not change, 0: set mode off, else set mode on
//
// return	(int) current local mode setting
//-----------------------------------------------------------------------------
int
msg_set_local (
    int		arg_local
    )
;

//-----------------------------------------------------------------------------
// function	msg_set_frac_digits
//
// purpose	Set the number of fraction digits to be used for the
//		fractions of seconds to be appended to the time_format.
//
// arguments	1 (int) number of digits to use, or negative to not change.
//
// returns	(int) number of digits to use
//-----------------------------------------------------------------------------
int
msg_set_frac_digits (
    int		arg_digits
    )
;

//-----------------------------------------------------------------------------
// function	msg_set_time_format
//
// purpose	Set the prefix format to be included in messages.  If the value
//		given is NULL, then unset the program name.
//
// arguments	1 (const char *) time format string to be set, empty string to
//				 unset, or NULL to leave unchanged
//
// returns	(char *) pointer to copied prefix format.
//
// WARNING	This setting has process scope.  That means it will apply for
//		all threads in a multi-threading process.
//
// WARNING	While this setting is being done, other threads should not
//		use any functions that use this setting since the string is
//		not updated atomically.
//-----------------------------------------------------------------------------
char *
msg_set_time_format (
    const char *	arg_format
    )
;

//-----------------------------------------------------------------------------
// function	msg_set_program_name
//
// purpose	Set the program name to be included in messages to the last
//		component of the name or full path given.  If the argument is
//		NULL, then unset the program name.
//
// arguments	1 (const char *) program name to be set, empty string to
//				 unset, or NULL to leave unchanged
//
// returns	(char *) pointer to copied program name.
//
// WARNING	This setting has process scope.  That means it will apply for
//		all threads in a multi-threading process.
//
// WARNING	While this setting is being done, other threads should not
//		use any functions that use this setting since the string is
//		not updated atomically.
//-----------------------------------------------------------------------------
char *
msg_set_program_name (
    const char *	arg_program
    )
;

//-----------------------------------------------------------------------------
// function	msg_set_program_path
//
// purpose	Set the program name to be included in messages to the full
//		program path as given.  If the argument is NULL, then unset
//		the program name.
//
// arguments	1 (const char *) program path to be set, empty string to
//				 unset, or NULL to leave unchanged
//
// returns	(char *) pointer to copied program path.
//
// WARNING	This setting has process scope.  That means it will apply for
//		all threads in a multi-threading process.
//
// WARNING	While this setting is being done, other threads should not
//		use any functions that use this setting since the string is
//		not updated atomically.
//-----------------------------------------------------------------------------
char *
msg_set_program_path (
    const char *	arg_program
    )
;

//-----------------------------------------------------------------------------
// function	msg_vfprintf
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to the specified file.
//
// arguments	1 (FILE *) destination output stream
//		2 (const char *) format for message content
//		3 (va_list) started variable argument list
//
// returns	(int) number of characters printed
//-----------------------------------------------------------------------------
int
msg_vfprintf (
    FILE *		arg_file
    ,
    const char *	arg_format
    ,
    va_list		arg_va_list
    )
;

//-----------------------------------------------------------------------------
// function	msg_vprintf
//
// purpose	Format an output message according to registered and given
//		format specifications and output it to stdout.
//
// arguments	1 (const char *) format for message content
//		2 (va_list) started variable argument list
//
// returns	(int) number of characters printed
//-----------------------------------------------------------------------------
int
msg_vprintf (
    const char *	arg_format
    ,
    va_list		arg_va_list
    )
;

//-----------------------------------------------------------------------------
// function	(multi_close)
//
// purpose	Close multiple file descriptors from a variable argument list
//		ending in a negative number.
//
// arguments	1..N (int) file descriptor to close
//		N+1 (int) -1 to indicate end of list
//
// returns	(int) number of descriptors closed successfully
//-----------------------------------------------------------------------------
int
(multi_close) (
    int		arg_fd
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	open_deep
//
// purpose	Open a file and return a file descriptor where the file name is
//		possibly longer than PATH_MAX allows.
//
// arguments	1 (const char *) file name, possibly longer than PATH_MAX
//		2 (int) option flags, e.g. O_RDONLY, O_WRONLY, etc
//		3 (mode_t) mode for created files
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) >= 0 : [success, file opened] file descriptor
//-----------------------------------------------------------------------------
int
open_deep (
    const char *	arg_name
    ,
    int			arg_flags
    ,
    mode_t		arg_mode
    )
;

//-----------------------------------------------------------------------------
// function	(open_deep_join)
//
// purpose	Open a file and return a file descriptor where the file name is
//		possibly longer than PATH_MAX allows, and is given as multiple
//		string arguments to be joined together.
//
// arguments	1 (int) option flags, e.g. O_RDONLY, O_WRONLY, etc
//		2 (mode_t) mode for created files
//		3..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) >= 0 : [success, file opened] file descriptor
//-----------------------------------------------------------------------------
int
(open_deep_join) (
    int		arg_flags
    ,
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	open_fork
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (int *) where to store file descriptor of write end of pipe
//			which will be the child's stdin.
//		2 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stdout.
//		3 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stderr.
//
// returns	(int) -1 if there is an error (see errno)
//		(int) 0 when returning in the child process
//		(int) child PID when returning in the parent process
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same file descriptor as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
int
open_fork (
    int *		arg_fd_ptr_stdin
    ,
    int *		arg_fd_ptr_stdout
    ,
    int *		arg_fd_ptr_stderr
    )
;

//-----------------------------------------------------------------------------
// function	(open_fork_exec)
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (int *) where to store file descriptor of write end of pipe
//			which will be the child's stdin.
//		2 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stdout.
//		3 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stderr.
//		4 (const char *) filename of program to execute in child
//		5-N (const char *) arguments to program being executed ended
//			with a NULL argument
//
// returns	(int) -1 if there is an error (see errno)
//		(int) child PID
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same file descriptor as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
int
(open_fork_exec) (
    int *		arg_fd_ptr_stdin
    ,
    int *		arg_fd_ptr_stdout
    ,
    int *		arg_fd_ptr_stderr
    ,
    const char *	arg_execute
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	open_fork_execv
//
// purpose	Fork a child process with one or more pipes between parent and
//		child which will be stdin, stdout, and/or stderr for child.
//
// arguments	1 (int *) where to store file descriptor of write end of pipe
//			which will be the child's stdin.
//		2 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stdout.
//		3 (int *) where to store file descriptor of read end of pipe
//			which will be the child's stderr.
//		4 (const char *) filename of program to execute in child
//		5 (char * const *) arguments to program being executed
//		6 (char * const *) environment variables to pass to program
//			or NULL to pass current environment
//
// returns	(int) -1 if there is an error (see errno)
//		(int) child PID
//
// note		If a pointer has the value NULL, then no pipe will be created
//		for that particular purpose, and the child will inherit the
//		same file descriptor as the parent.
//
// note		If the pointers given in arguments 2 and 3 are the same, then
//		only a single pipe will be created for child output, and that
//		pipe will be duplicated for both stdout and stderr.
//
// warning	Care should be taken when transferring data both to and from
//		a child process at the same time to ensure that the parent and
//		child do not get into deadlock condition waiting for buffered
//		data from each other.  This is especially important when the
//		child process executes another program which may not handle
//		buffering as expected (such as a data compression program).
//-----------------------------------------------------------------------------
int
open_fork_execv (
    int *		arg_fd_ptr_stdin
    ,
    int *		arg_fd_ptr_stdout
    ,
    int *		arg_fd_ptr_stderr
    ,
    const char *	arg_execute
    ,
    char * const *	arg_argv
    ,
    char * const *	arg_envp
    )
;

//-----------------------------------------------------------------------------
// function	(open_join)
//
// purpose	Open a file and return a file descriptor where the file name is
//		given as multiple string arguments to be joined together.
//
// arguments	1 (int) option flags, e.g. O_RDONLY, O_WRONLY, etc
//		2 (mode_t) mode for created files
//		3..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) >= 0 : [success, file opened] file descriptor
//-----------------------------------------------------------------------------
int
(open_join) (
    int		arg_flags
    ,
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(open_join_sep)
//
// purpose	Open a file and return a file descriptor where the file name is
//		given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments	1 (int) option flags, e.g. O_RDONLY, O_WRONLY, etc
//		2 (mode_t) mode for created files
//		3..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) >= 0 : [success, file opened] file descriptor
//-----------------------------------------------------------------------------
int
(open_join_sep) (
    int		arg_flags
    ,
    mode_t	arg_mode
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	open_proc
//
// purpose	Open pipe(s) to a new process running the given command and
//		arguments.  The pipe must be closed via close_proc.
//
// arguments	1 (int *) pointer to store parent end of pipe to stdin
//		2 (int *) pointer to store parent end of pipe to stdout
//		3 (int *) pointer to store parent end of pipe to stderr
//		4 (int) file descriptor to use if no pipe for stdin
//		5 (int) file descriptor to use if no pipe for stdout
//		6 (int) file descriptor to use if no pipe for stderr
//		7 (int) highest descriptor to be closed in child
//		8 (int) 1 fork grandchild and wait for child now
//		9 (const char *) file name to execute
//		10 (const char * const *) array of arguments
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, child already exited, grandchild running
//		(int)  > 0 : OK, PID of child running, must wait later
//
// note		If the pointers given for stdout and stderr are the same,
//		then a single pipe is created and the child will have both
//		stdout and stderr sharing the write end of that pipe.
//
// note		Caller should keep track of the highest file descriptor it
//		has opened and pass it via argument 7 to ensure that the
//		child process closes everything but standard descriptors.
//
// note		If the caller needs to pass other file descriptors to the
//		child, then this function is not appropriate for use since it
//		closes all descriptors but the standard ones.  Most programs
//		executed via execve() only need to inherit standard files.
//-----------------------------------------------------------------------------
int
open_proc (
    int *		arg_fd_ptr_stdin
    ,
    int *		arg_fd_ptr_stdout
    ,
    int *		arg_fd_ptr_stderr
    ,
    int			arg_inh_fd_stdin
    ,
    int			arg_inh_fd_stdout
    ,
    int			arg_inh_fd_stderr
    ,
    int			arg_fd_high
    ,
    int			arg_wait_now
    ,
    const char *	arg_exec_name
    ,
    char * const *	arg_exec_args
    )
;

//-----------------------------------------------------------------------------
// function	open_proc_arg
//
// purpose	Open pipe(s) to a new process running the given command and
//		arguments.  The pipe must be closed via close_proc.
//
// arguments	1 (int *) pointer to store parent end of pipe to stdin
//		2 (int *) pointer to store parent end of pipe to stdout
//		3 (int *) pointer to store parent end of pipe to stderr
//		4 (int) file descriptor to use if no pipe for stdin
//		5 (int) file descriptor to use if no pipe for stdout
//		6 (int) file descriptor to use if no pipe for stderr
//		7 (int) highest descriptor to be closed in child
//		8 (int) 1 if wait for child now.
//		9 (const char *) file name to execute
//		10-N (const char *) program arguments as function arguments
//		N+1 (const char *) NULL to indicate end of arguments
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, child already exited, grandchild running
//		(int)  > 0 : OK, PID of child running, must wait later
//
// note		If the pointers given for stdout and stderr are the same,
//		then a single pipe is created and the child will have both
//		stdout and stderr sharing the write end of that pipe.
//
// note		Caller should keep track of the highest file descriptor it
//		has opened and pass it via argument 7 to ensure that the
//		child process closes everything but standard descriptors.
//
// note		If the caller needs to pass other file descriptors to the
//		child, then this function is not appropriate for use.
//-----------------------------------------------------------------------------
int
(open_proc_arg) (
    int *		arg_fd_ptr_stdin
    ,
    int *		arg_fd_ptr_stdout
    ,
    int *		arg_fd_ptr_stderr
    ,
    int			arg_inh_fd_stdin
    ,
    int			arg_inh_fd_stdout
    ,
    int			arg_inh_fd_stderr
    ,
    int			arg_fd_high
    ,
    int			arg_wait_now
    ,
    const char *	arg_exec_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	open_unique
//
// purpose	Create and open a new unique file for output.
//
// arguments	1 (const char *) filename format pattern (etime format specs)
//		2 (int) non-zero to also read
//		3 (int) open mode
//
// returns	(int)  < 0 : error returned by last open() after many tries
//		(int) >= 0 : open file descriptor handle
//-----------------------------------------------------------------------------
int
open_unique (
    const char *	arg_format
    ,
    int			arg_flags
    ,
    int			arg_mode
    )
;

//-----------------------------------------------------------------------------
// function	(opendir_join)
//
// purpose	Open a directory and return an open directory handle for
//		reading with readdir(), where the name is given as multiple
//		string arguments to be joined together.
//
// arguments	1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(DIR *) == NULL : [failure, see errno]
//		(DIR *) != NULL : [success, directory opened] handle
//-----------------------------------------------------------------------------
DIR *
(opendir_join) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(opendir_join_sep)
//
// purpose      Open a directory and return an open directory handle for
//              reading with readdir(), where the name is given as multiple
//              string arguments to be joined together with a directory
//		separator between each argument.
//
// arguments    1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (DIR *) == NULL : [failure, see errno]
//              (DIR *) != NULL : [success, directory opened] handle
//-----------------------------------------------------------------------------
DIR *
(opendir_join_sep) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	put_config_item
//
// purpose	Put a config item (name and data strings) to an output file.
//
// arguments	1 (const char *) pointer to name string
//		2 (size_t) length of name string
//		3 (const char *) pointer to data string
//		4 (size_t) length of name string
//		5 (FILE *) pointer to open file to write to
//
// returns	(int) == -3 : error
//		(int) ==  0 : config item written OK
//-----------------------------------------------------------------------------
#define PUT_CONFIG_NEST_BEGIN	0x0001
#define PUT_CONFIG_NEST_END	0x0002
int
put_config_item (
    const char *	arg_name_ptr
    ,
    size_t		arg_name_len
    ,
    const char *	arg_data_ptr
    ,
    size_t		arg_data_len
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	put_config_map
//
// purpose	Output a mapping into a config file.
//
// arguments	1 (MAP) mapping to be output
//		2 (FILE *) pointer to open file to output to
//
// returns	(long) == -1 : error
//		(long) >=  0 : number of items output
//-----------------------------------------------------------------------------
long
put_config_map (
    MAP			arg_map
    ,
    FILE *		arg_file
    )
;

//-----------------------------------------------------------------------------
// function	put_var_shell
//
// purpose	Output name and value strings in a form for shell variable
//		assignment.
//
// arguments	1 (FILE *) which file to output to
//		2 (int) code for which shell type:
//			0, 'b' bourne, bash, ksh shells
//			1, 'c' csh, tcsh shells
//		3 (int) 0 for normal variable, 1 for environment variable
//		4 (const char *) name prefix
//		5 (const char *) name
//		6 (const char *) name suffix
//		7 (const char *) value
//
// returns	(void)
//-----------------------------------------------------------------------------
void
put_var_shell (
    FILE *		arg_file
    ,
    int			arg_shell
    ,
    int			arg_env
    ,
    const char *	arg_prefix
    ,
    const char *	arg_name
    ,
    const char *	arg_suffix
    ,
    const char *	arg_value
    )
;

//-----------------------------------------------------------------------------
// function	read_block
//
// purpose	Read a full block of data, repeating until done.
//
// arguments	1 (int) file descriptor
//		2 (void *) data pointer
//		3 (size_t) data length
//
// returns	(ssize_t)  < 0 : error, see errno
//		(ssize_t) == 0 : end of file
//		(ssize_t)  > 0 : bytes read
//-----------------------------------------------------------------------------
ssize_t
read_block (
    int			arg_fd
    ,
    void *		arg_ptr
    ,
    size_t		arg_len
    )
;

//-----------------------------------------------------------------------------
// function	(readlink_join)
//
// purpose	Read the reference data for symbolic link of the given name
//		where the symbolic link name is given as multiple string
//		arguments to be joined together.
//
// arguments	1 (char *) target buffer
//		2 (size_t) size of target buffer
//		3..N (const char *) symbolic link name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(ssize_t)  < 0 : [failure, see errno]
//		(ssize_t) >= 0 : [success] number of characters stored
//
// note		Unlike the POSIX/SUS standard readlink() function, this
//		function/macro will properly terminate the symlink data.
//		Thus it will be necessary to provide at least one additional
//		character of buffer space.
//-----------------------------------------------------------------------------
ssize_t
(readlink_join) (
    char *		arg_buffer
    ,
    size_t		arg_size
    ,
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(readlink_join_sep)
//
// purpose	Read the reference data for symbolic link of the given name
//		where the symbolic link name is given as multiple string
//		arguments to be joined together with a directory separator
//		between each argument.
//
// arguments	1 (char *) target buffer
//		2 (size_t) size of target buffer
//		3..N (const char *) symbolic link name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(ssize_t)  < 0 : [failure, see errno]
//		(ssize_t) >= 0 : [success] number of characters stored
//
// note		Unlike the POSIX/SUS standard readlink() function, this
//		function/macro will properly terminate the symlink data.
//		Thus it will be necessary to provide at least one additional
//		character of buffer space.
//-----------------------------------------------------------------------------
ssize_t
(readlink_join_sep) (
    char *		arg_buffer
    ,
    size_t		arg_size
    ,
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(rmdir_join)
//
// purpose	Remove a directory of the given name where the directory name
//		is given as multiple string arguments to be joined together.
//
// arguments	1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, directory removed] status
//-----------------------------------------------------------------------------
int
(rmdir_join) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(rmdir_join_sep)
//
// purpose      Remove a directory of the given name where the directory name
//              is given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments    1..N (const char *) directory name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, directory removed] status
//-----------------------------------------------------------------------------
int
(rmdir_join_sep) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	skip_line_file
//
// purpose	Skip to the next line delimited by a combination of end of line
//		characters, discarding the contents.
//
// arguments	1 (FILE *) pointer to open file to read from
//
// returns	(int) -3 : error
//		(int) -2 : line not skipped due to immediate end of file
//              (int)  0 : line skipped OK
//
// note		An end of line is considered to be any combination of the four
//		line breaking characters:
//			'\n' (newline)
//			'\r' (carriage return)
//			'\f' (form feed)
//			'\v' (vertical tab)
//		with not more than one instance of any one character making up
//		one line ending.  If a character is encountered a 2nd time, it
//		is considered to begin a 2nd line ending.
//-----------------------------------------------------------------------------
int
skip_line_file (
    FILE *	arg_file
    )
;

//-----------------------------------------------------------------------------
// function	(stat_join)
//
// purpose	Return information about a file where the file name is given
//		as multiple string arguments to be joined together ended by a
//		NULL argument.
//
// arguments	1 (struct stat *) buffer where to store file information
//		2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int) == -1 : [failure, see errno]
//		(int) >=  0 : [success, file info stored] status
//-----------------------------------------------------------------------------
int
(stat_join) (
    struct stat *	arg_buf
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(stat_join_sep)
//
// purpose      Return information about a file where the file name is given
//		as multiple string arguments to be joined together with a
//		directory separator between each argument ended by a NULL
//		argument.
//
// arguments    1 (struct stat *) buffer where to store file information
//              2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, file info stored] status
//-----------------------------------------------------------------------------
int
(stat_join_sep) (
    struct stat *	arg_buf
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	stderr_to_file
//
// purpose	Change the C standard stderr stream to write to a specific
//		file.
//
// arguments	1 (const char *) file to write stderr to
//		2 (int) open flags
//		3 (int) open mode
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
stderr_to_file (
    const char *	arg_filename
    ,
    int			arg_flags
    ,
    int			arg_mode
    )
;

//-----------------------------------------------------------------------------
// function	stderr_to_unique_file
//
// purpose	Change the C standard stderr stream to write to a uniquely
//		created file according to the given pattern.
//
// arguments	1 (const char *) filename format pattern
//		2 (int) open flags
//		3 (int) open mode
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
stderr_to_unique_file (
    const char *	arg_pattern
    ,
    int			arg_flags
    ,
    int			arg_mode
    )
;

//-----------------------------------------------------------------------------
// function	sub_copy
//
// purpose	Copy an open file to another open file, with parsing for
//		substitutions that begin and end with specified strings,
//		returning to the caller for each substitution encountered.
//		The caller can output whatever it needs to output, and
//		resume by calling sub_copy() again.
//
// arguments	1 (FILE *) open input file
//		2 (FILE *) open output file
//		3 (char *) pointer to where to store substitution string
//		4 (size_t) size of space to store substitution string
//		5 (const char *) substitution marker prefix string
//		6 (const char *) substitution marker suffix string
//
// returns	(int) -2 : error
//		(int) -1 : EOF collecting substitution (stored)
//		(int)  0 : EOF complete
//		(int)  1 : substitution found, complete
//		(int)  2 : substitution found, truncated
//
// note		The marker prefix and suffix are not store with the string to
//		be substituted.
//-----------------------------------------------------------------------------
int
sub_copy (
    FILE *		arg_file_in
    ,
    FILE *		arg_file_out
    ,
    char *		arg_ptr
    ,
    size_t		arg_len
    ,
    const char *	arg_prefix
    ,
    const char *	arg_suffix
    )
;

//-----------------------------------------------------------------------------
// function	sub_copy_map
//
// purpose	Copy an open file to another open file, with parsing for
//		substitutions that begin and end with specified strings,
//		substituting each with the string found in the given mapping.
//
// arguments	1 (FILE *) open input file
//		2 (FILE *) open output file
//		3 (MAP) mapping to get replacement data from
//		4 (const char *) substitution marker prefix string
//		5 (const char *) substitution marker suffix string
//
// returns	(int) -2 : error
//		(int) -1 : EOF collecting substitution
//		(int)  0 : EOF complete
//-----------------------------------------------------------------------------
int
sub_copy_map (
    FILE *		arg_file_in
    ,
    FILE *		arg_file_out
    ,
    MAP			arg_map
    ,
    const char *	arg_prefix
    ,
    const char *	arg_suffix
    )
;

//-----------------------------------------------------------------------------
// function	(truncate_join)
//
// purpose	Truncate the named file where the file name is given as
//		multiple string arguments to be joined together.
//
// arguments	1 (off_t) new length for file
//		2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, file truncated] status
//-----------------------------------------------------------------------------
int
(truncate_join) (
    off_t	arg_len
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(truncate_join_sep)
//
// purpose      Change the mode of a named file where the file name is
//              given as multiple string arguments to be joined together
//		with a directory separator between each argument.
//
// arguments    1 (off_t) new length for file
//              2..N (const char *) file name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, mode changed] status
//-----------------------------------------------------------------------------
int
(truncate_join_sep) (
    off_t	arg_len
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(unlink_join)
//
// purpose	Unlink a name where the name is given as multiple string
//		arguments to be joined together.
//
// arguments	1..N (const char *) name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns	(int)  < 0 : [failure, see errno]
//		(int) == 0 : [success, name unlinked] status
//-----------------------------------------------------------------------------
int
(unlink_join) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	(unlink_join_sep)
//
// purpose	Unlink a name where the name is given as multiple string
//		arguments to be joined together with a directory separator
//		between each argument.
//
// arguments    1..N (const char *) name parts
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int)  < 0 : [failure, see errno]
//              (int) == 0 : [success, name unlinked] status
//-----------------------------------------------------------------------------
int
(unlink_join_sep) (
    const char *	arg_name
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	write_block
//
// purpose	Write a full block of data, repeating until done.
//
// arguments	1 (int) file descriptor
//		2 (const void *) data pointer
//		3 (size_t) data length
//
// returns	(ssize_t)  < 0 : error, see errno
//		(ssize_t) == 0 : special status
//		(ssize_t)  > 0 : bytes written
//-----------------------------------------------------------------------------
ssize_t
write_block (
    int			arg_fd
    ,
    const void *	arg_ptr
    ,
    size_t		arg_len
    )
;

#endif /* __IO_H__ */
