//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/net
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		libh/net.h
//-----------------------------------------------------------------------------
#ifndef __NET_H__
#define __NET_H__
#include <netdb.h>
#include <netinet/in.h>
#include <stdarg.h>

#include <libh/map.h>

//-----------------------------------------------------------------------------
// struct	ipv4_table_node
// typedef	ipv4_table_node_p
//
// purpose	This is a node struct of an IPv4 lookup table.
//-----------------------------------------------------------------------------
struct ipv4_table_node {
    struct ipv4_table_node *	next	[2]	;
    struct ipv4_table_node *	prev		;
    struct ipv4_table_node *	data		;
};
typedef struct ipv4_table_node *	ipv4_table_node_p	;

//-----------------------------------------------------------------------------
// struct	ipv4_table
// typedef	ipv4_table_p
//
// purpose	This is the head struct of an IPv4 lookup table, including
//		the apex node itself.
//-----------------------------------------------------------------------------
struct ipv4_table {
    struct ipv4_table_node	apex		;
    struct ipv4_table_node *	last		;
    int				prefix		;
};
typedef struct ipv4_table *		ipv4_table_p		;

#define IPV4_STR_TO_ADDR_ADDR           0x00000001
#define IPV4_STR_TO_ADDR_ADDR2          0x00000002
#define IPV4_STR_TO_ADDR_COUNT          0x00000004
#define IPV4_STR_TO_ADDR_PORT           0x00000008
#define IPV4_STR_TO_ADDR_PREFIX         0x00000010
#define IPV4_STR_TO_ADDR_NETMASK        0x00000020

#define IPV4_STR_TO_ADDR_BAD_ADDR       0x00010000
#define IPV4_STR_TO_ADDR_BAD_ADDR2      0x00020000
#define IPV4_STR_TO_ADDR_BAD_COUNT      0x00040000
#define IPV4_STR_TO_ADDR_BAD_PORT       0x00080000
#define IPV4_STR_TO_ADDR_BAD_PREFIX     0x00100000

#define IPV4_STR_TO_ADDR_BAD_CHAR       0x40000000

#define IPV4_STR_TO_ADDR_ERROR          0x80000000

//-----------------------------------------------------------------------------
// structs and typedefs
//-----------------------------------------------------------------------------
typedef uint32_t	ipv4_t		;

struct ipv4_netinfo {
    ipv4_t		address		;
    ipv4_t		address2	;
    ipv4_t		netfirst	;
    ipv4_t		netlast		;
    ipv4_t		netmask		;
    uint32_t		count		;
    unsigned int	prefix		;
    unsigned int	port		;
    unsigned int	flags		;
};

//-----------------------------------------------------------------------------
// macro	hbo_to_sockaddr_in
//
// purpose	Combine port and address in host byte order to make a struct
//		sockaddr_in to be assigned to a target lvalue.
//
// arguments	1 (unsigned long) address in host byte order
//		2 (unsigned int) port number in host byte order
//
// returns	(struct sockaddr_in) yielded struct
//-----------------------------------------------------------------------------
#define hbo_to_sockaddr_in(a,p) ({						\
	struct sockaddr_in	libh__addr;					\
	memset( & libh__addr, 0, sizeof libh__addr );				\
	libh__addr.sin_family = AF_INET;					\
	libh__addr.sin_port = htons( (p) );					\
	libh__addr.sin_addr.s_addr = htonl( (a) );				\
	libh__addr;								\
})

//-----------------------------------------------------------------------------
// macro	ipv4_addr_in_cidr
//
// purpose	Yield true or false indicating if the specified IP address is
//		in the CIDR block specified by address and prefix length.
//
// arguments	1 (ipv4_t) address to define CIDR block
//		2 (int) prefix length (1..32)
//		3 (ipv4_t) address to check
//
// yields	(int) true or false
//-----------------------------------------------------------------------------
#define ipv4_addr_in_cidr(n,p,a) ((((~(ipv4_t)0)<<(32-(p)))&(n))==(((~(ipv4_t)0)<<(32-(p)))&(a)))

//-----------------------------------------------------------------------------
// macro	ipv4_addr_in_cidr_count_arg
//
// purpose	Given a CIDR network block (address and prefix), and a list of
//		addresses in variable arguments (stopping at address 0.0.0.0),
//		return the count of how many of those addresses are in the
//		specified network block.
//
// arguments	1 (ipv4_t) address to define CIDR block
//		2 (int) prefix length (1..32)
//		3..N (ipv4_t) addresses to check
//
// returns	(int) count of addresses in the given CIDR block.
//-----------------------------------------------------------------------------
#define ipv4_addr_in_cidr_count_arg(n,p,a...) ((ipv4_addr_in_cidr_count_arg)((n),(p),a,((ipv4_t)0)))

//-----------------------------------------------------------------------------
// macro	ipv4_addrprefix_to_broadcast
//
// purpose	Convert an IPv4 address plus network prefix length into a
//		network broadcast address.
//
// arguments	1 (ipv4_t) IPv4 address value in integer form
//		2 (unsigned int) network prefix length (0..32)
//
// returns	(ipv4_t) IPv4 network broadcast address
//
// note		There are no possible error indications from this function.
//		If the network prefix length is out of range, results are
//		undefined.
//-----------------------------------------------------------------------------
#define ipv4_addrprefix_to_broadcast(a,p)					\
	(((ipv4_t)(a))|((((ipv4_t)1U)<<(32U-((unsigned int)(p))))-1U))

//-----------------------------------------------------------------------------
// macro	ipv4_addrprefix_to_netbase
//
// purpose	Convert an IPv4 address plus network prefix length into a
//		network base address.
//
// arguments	1 (ipv4_t) IPv4 address value in integer form
//		2 (unsigned int) network prefix length (0..32)
//
// returns	(ipv4_t) IPv4 network base address
//
// note		There are no possible error indications from this function.
//		If the network prefix length is out of range, results are
//		undefined.
//-----------------------------------------------------------------------------
#define ipv4_addrprefix_to_netbase(a,p)						\
	(((ipv4_t)(a))&((~(ipv4_t)0U)<<(32U-((unsigned int)(p)))))

//-----------------------------------------------------------------------------
// macro	ipv4_combine_octets
//
// purpose	Combine an IPv4 address in four separate octet parts into a
//		single 32-bit integer result.
//
// arguments	1 (ipv4_t) highest order octet
//		2 (ipv4_t) second highest order octet
//		3 (ipv4_t) second lowest order octet
//		4 (ipv4_t) lowest order octet
//
// returns	(ipv4_t) one complete IPv4 address
//
// note		This macro assumes the given octets are already limited to
//		values in the range 0..255.  If they are outside this range,
//		the results are undefined.
//-----------------------------------------------------------------------------
#define ipv4_combine_octets(a,b,c,d)						\
	((((ipv4_t)(a))<<24)|(((ipv4_t)(b))<<16)|(((ipv4_t)(c))<<8)|((ipv4_t)(d))<<0)

//-----------------------------------------------------------------------------
// macro	ipv4_table_count
//
// purpose	Count the number of entries in an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) pointer to table to count entries in.
//
// returns	(unsigned long) number of entries
//-----------------------------------------------------------------------------
#define ipv4_table_count(t) ipv4_table_count_node((t),32)

//-----------------------------------------------------------------------------
// macro	ipv4_table_node_alloc
//
// purpose	Create a new node for an IPv4 lookup table.
//
// arguments	-none-
//
// returns	(ipv4_table_p) pointer to new IPv4 lookup table
//-----------------------------------------------------------------------------
#define ipv4_table_node_alloc() ({						\
	ipv4_table_node_p libh__node_p;						\
	libh__node_p = malloc( sizeof (struct ipv4_table_node) );		\
	if ( libh__node_p ) {							\
		libh__node_p->next[0] = NULL;					\
		libh__node_p->next[1] = NULL;					\
		libh__node_p->prev = NULL;					\
		libh__node_p->data = NULL;					\
	}									\
	libh__node_p;								\
})

//-----------------------------------------------------------------------------
// macro	ipv4_table_create
//
// purpose	Create a new instance of an IPv4 lookup table.
//
// arguments	-none-
//
// returns	(ipv4_table_p) pointer to new IPv4 lookup table
//-----------------------------------------------------------------------------
#define ipv4_table_create() ({							\
	ipv4_table_p libh__table_p;						\
	libh__table_p = malloc( sizeof (struct ipv4_table) );			\
	if ( libh__table_p ) {							\
		libh__table_p->apex.next[0] = NULL;				\
		libh__table_p->apex.next[1] = NULL;				\
		libh__table_p->apex.prev = NULL;				\
		libh__table_p->apex.data = NULL;				\
		libh__table_p->last = NULL;					\
		libh__table_p->prefix = 0;					\
	}									\
	libh__table_p;								\
})
//-----------------------------------------------------------------------------
// macro	ipv4_table_destroy
//
// purpose	Destroy an instance of an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) pointer to table to destroy
//
// returns	(void)
//-----------------------------------------------------------------------------
#define ipv4_table_destroy(t) ({						\
	ipv4_table_p libh__table_p;						\
	libh__table_p = (t);							\
	if ( libh__table_p ) {							\
		ipv4_table_empty_node( & libh__table_p->apex, 32 );		\
		libh__table_p->apex.next[0] = NULL;				\
		libh__table_p->apex.next[1] = NULL;				\
		libh__table_p->apex.prev = NULL;				\
		libh__table_p->apex.data = NULL;				\
		libh__table_p->last = NULL;					\
		libh__table_p->prefix = 0;					\
		free( libh__table_p );						\
	}									\
})
//-----------------------------------------------------------------------------
// macro	ipv4_table_empty
//
// purpose	Empty a no longer needed instance of an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) pointer to table to empty
//
// returns	(void)
//-----------------------------------------------------------------------------
#define ipv4_table_empty(t) ({							\
	ipv4_table_p libh__table_p;						\
	libh__table_p = (t);							\
	if ( libh__table_p && libh__table_p->apex ) {				\
		ipv4_table_empty_node( libh__table_p->apex.next[0], 31 );	\
		ipv4_table_empty_node( libh__table_p->apex.next[1], 31 );	\
	}									\
})

//-----------------------------------------------------------------------------
// macro	ipv4_table_iterate
//
// purpose	Iterate through all the entries in an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) pointer to IPv4 lookup table to iterate on
//		2 (void *) function to call for each entry
//		3 (void *) extra pointer to pass to this function
//
// returns	(unsigned long) sum of iteration returns
//-----------------------------------------------------------------------------
#define ipv4_table_iterate(t,f,p) ipv4_table_iterate_node((ipv4_table_node_p)(t),(ipv4_t)0,0,(f),(p))

//-----------------------------------------------------------------------------
// macro	nbo_to_sockaddr_in
//
// purpose	Combine port and address in network byte order to make a struct
//		sockaddr_in to be assigned to a target lvalue.
//
// arguments	1 (struct in_addr) address in network byte order
//		2 (u_int16_t) port number in network byte order
//
// returns	(struct sockaddr_in) yielded struct
//-----------------------------------------------------------------------------
#define nbo_to_sockaddr_in(a,p) ({						\
	struct sockaddr_in	libh__addr;					\
	libh__addr.sin_family = AF_INET;					\
	libh__addr.sin_port = (p);						\
	libh__addr.sin_addr = (a);						\
	libh__addr;								\
})

//-----------------------------------------------------------------------------
// macro	nbo_to_sockaddr_in6
//
// purpose	Combine port and address in network byte order to make a struct
//		sockaddr_in to be assigned to a target lvalue.
//
// arguments	1 (struct sin6_addr) address in network byte order
//		2 (u_int16_t) port number in network byte order
//
// returns	(struct sockaddr_in6) yielded struct
//-----------------------------------------------------------------------------
#define nbo_to_sockaddr_in6(a,p) ({						\
	struct sockaddr_in6	libh__addr;					\
	libh__addr.sin6_family = AF_INET6;					\
	libh__addr.sin6_port = (p);						\
	libh__addr.sin6_addr = (a);						\
	libh__addr;								\
})

//-----------------------------------------------------------------------------
// function	get_port_by_name
//
// purpose	Get the port number, in host byte order, from a service name,
//		using getservbyname(), correcting for known errors in the way
//		the s_port member is defined in struct servent.
//
// arguments    1 (const char *) service name or numeric port string.
//		2 (const char *) protocol, "tcp" or "udp".
//
// returns      (int) < 0 : error
//              (int) > 0 : port number in host byte order
//-----------------------------------------------------------------------------
int
get_port_by_name (
    const char *        arg_service
    ,
    const char *        arg_protocol
    )
;

//-----------------------------------------------------------------------------
// function	(ipv4_addr_in_cidr_count_arg)
//
// purpose	Given a CIDR network block (address and prefix), and a list of
//		addresses in variable arguments (stopping at address 0.0.0.0),
//		return the count of how many of those addresses are in the
//		specified network block.
//
// arguments	1 (ipv4_t) address to define CIDR block
//		2 (int) prefix length (1..32)
//		3..N (ipv4_t) addresses to check
//		N+1 (ipv4_t) 0 to indicate end of list
//
// returns	(int) count of addresses in the given CIDR block.
//-----------------------------------------------------------------------------
int
(ipv4_addr_in_cidr_count_arg) (
    ipv4_t		arg_network
    ,
    int			arg_prefix
    ,
    ...
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_range_to_cidr_in_addr
//
// purpose	Convert a range of IPv4 addresses, in network byte order,
//		into a group of CIDR block addresses, returned one at a time.
//		This variety of the function uses addresses in struct in_addr
//		format.  See also ipv4_int_range_to_cidr() for int format.
//
// arguments	1 (struct in_addr *) pointer to range first address
//		2 (struct in_addr *) pointer to range last address
//		3 (struct in_addr *) where to store CIDR block address
//		4 (struct in_addr *) where to store CIDR block netmask
//		5 (struct in_addr *) where to store CIDR block size
//
// returns	(int) 1..32 : CIDR block prefix length
//		(int)     0 : the whole IPv4 address space
//		(int)    -1 : no (more) CIDR blocks
//
// note		The locations pointed to are modified in place to reduce the
//		range by removing each CIDR block returned.
//-----------------------------------------------------------------------------
int
ipv4_range_to_cidr_in_addr (
    struct in_addr *	arg_range_addr
    ,
    struct in_addr *	arg_range_last
    ,
    struct in_addr *	arg_cidr_addr
    ,
    struct in_addr *	arg_cidr_mask
    ,
    struct in_addr *	arg_cidr_size
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_range_to_cidr_int
//
// purpose	Convert a range of IPv4 addresses, in host byte order,
//		into a group of CIDR block addresses, returned one at a time.
//		This variety of the function uses addresses in int format.
//		See also ipv4_range_to_cidr_in_addr() for in_addr format.
//
// arguments	1 (ipv4_t *) pointer to range first address
//		2 (ipv4_t *) pointer to range last address
//		3 (ipv4_t *) where to store CIDR block address
//		4 (ipv4_t *) where to store CIDR block netmask
//		5 (ipv4_t *) where to store CIDR block size
//
// returns	(int) 1..32 : CIDR block prefix length
//		(int)     0 : the whole IPv4 address space
//		(int)    -1 : no (more) CIDR blocks
//
// note		The locations pointed to are modified in place to reduce the
//		range by removing each CIDR block returned.
//-----------------------------------------------------------------------------
int
ipv4_range_to_cidr_int (
    ipv4_t *		arg_range_first
    ,
    ipv4_t *		arg_range_last
    ,
    ipv4_t *		arg_cidr_addr
    ,
    ipv4_t *		arg_cidr_mask
    ,
    ipv4_t *		arg_cidr_size
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_range_to_class_in_addr
//
// purpose	Convert a range of IPv4 addresses, in network byte order,
//		into a group of traditional classful network block addresses,
//		returned one at a time.  This variety of the function uses
//		addresses in in_addr format.  See ipv4_range_to_class_int()
//		for int format.
//
// arguments	1 (struct in_addr *) pointer to range first address
//		2 (struct in_addr *) pointer to range last address
//		3 (struct in_addr *) where to store CIDR block address
//		4 (struct in_addr *) where to store CIDR block netmask
//		5 (struct in_addr *) where to store CIDR block size
//
// returns	(int) 8,16,24,32 : CIDR-style block prefix length
//		(int)          0 : the whole IPv4 address space
//		(int)         -1 : no (more) net blocks
//
// note		The locations pointed to are modified in place to reduce the
//		range by removing each network block returned.
//-----------------------------------------------------------------------------
int
ipv4_range_to_class_in_addr (
    struct in_addr *	arg_range_addr
    ,
    struct in_addr *	arg_range_last
    ,
    struct in_addr *	arg_class_addr
    ,
    struct in_addr *	arg_class_mask
    ,
    struct in_addr *	arg_class_size
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_range_to_class_int
//
// purpose	Convert a range of IPv4 addresses, in host byte order,
//		into a group of traditional classful network block addresses,
//		returned one at a time.  This variety of the function uses
//		addresses in int format.  See ipv4_range_to_class_in_addr()
//		for in_addr format.
//
// arguments	1 (ipv4_t *) pointer to range first address
//		2 (ipv4_t *) pointer to range last address
//		3 (ipv4_t *) where to store CIDR block address
//		4 (ipv4_t *) where to store CIDR block netmask
//		5 (ipv4_t *) where to store CIDR block size
//
// returns	(int) 8,16,24,32 : CIDR-style block prefix length
//		(int)          0 : the whole IPv4 address space
//		(int)         -1 : no (more) net blocks
//
// note		The locations pointed to are modified in place to reduce the
//		range by removing each network block returned.
//-----------------------------------------------------------------------------
int
ipv4_range_to_class_int (
    ipv4_t *		arg_range_addr
    ,
    ipv4_t *		arg_range_last
    ,
    ipv4_t *		arg_class_addr
    ,
    ipv4_t *		arg_class_mask
    ,
    ipv4_t *		arg_class_size
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_str_to_addr
//
// purpose	Convert an IPv4 address or network string in dotted decimal or
//		cetal (hexadecimal) format in one of these forms:
//			address[:port]
//			address-address[:port]
//			address/netmask[:port]
//			address/prefix[:port]
//			address#count[:port]
//		into corresponding numeric values stored to the caller and
//		return the type of format.
//
//		Combinations of these forms can also be handled with different
//		addresses stored in the the appropriate place depending on what
//		format designation it was found with.  The returned value is a
//		combination of flags.
//
// arguments	1 (const char *) string to be converted
//		2 (char * *) where to store pointer to end char
//		3 (ipv4_t *) where to store 1st address
//		4 (ipv4_t *) where to store 2nd address
//		5 (ipv4_t *) where to store netmask
//		6 (unsigned int *) where to store prefix
//		7 (unsigned int *) where to store count
//		8 (unsigned int *) where to store port
//
// return	(int) < 0 : error
//		(int)   0 : nothing found
//		(int)   1 : valid 1st IPv4 address
//		(int)   2 : valid 2nd IPv4 address
//		(int)   4 : valid count
//		(int)   8 : valid port
//		(int)  16 : valid netmask
//		(int)  32 : valid prefix
//-----------------------------------------------------------------------------
//	#define IPV4_STR_TO_ADDR_ADDR		0x00000001
//	#define IPV4_STR_TO_ADDR_ADDR2		0x00000002
//	#define IPV4_STR_TO_ADDR_COUNT		0x00000004
//	#define IPV4_STR_TO_ADDR_PORT		0x00000008
//	#define IPV4_STR_TO_ADDR_PREFIX		0x00000010
//	#define IPV4_STR_TO_ADDR_NETMASK	0x00000020
//
//	#define IPV4_STR_TO_ADDR_BAD_ADDR	0x00010000
//	#define IPV4_STR_TO_ADDR_BAD_ADDR2	0x00020000
//	#define IPV4_STR_TO_ADDR_BAD_COUNT	0x00040000
//	#define IPV4_STR_TO_ADDR_BAD_PORT	0x00080000
//	#define IPV4_STR_TO_ADDR_BAD_PREFIX	0x00100000
//
//	#define IPV4_STR_TO_ADDR_BAD_CHAR	0x40000000
//
//	#define IPV4_STR_TO_ADDR_ERROR		0x80000000
//-----------------------------------------------------------------------------

int
ipv4_str_to_addr (
    const char *	arg_str
    ,
    char * *		arg_endp_p
    ,
    ipv4_t *		arg_addr_p
    ,
    ipv4_t *		arg_addr2_p
    ,
    ipv4_t *		arg_netmask_p
    ,
    unsigned int *	arg_prefix_p
    ,
    unsigned int *	arg_count_p
    ,
    unsigned int *	arg_port_p
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_str_to_in_addr
//
// purpose	Convert an IPv4 address or network string in dotted decimal or
//		cetal (hexadecimal) format in one of these forms:
//			address[:port]
//			address-address[:port]
//			address/netmask[:port]
//			address/prefix[:port]
//			address#count[:port]
//		into corresponding numeric values stored to the caller and
//		return the type of format.
//
//		Combinations of these forms can also be handled with different
//		addresses stored in the the appropriate place depending on what
//		format designation it was found with.  The returned value is a
//		combination of flags.
//
// arguments	1 (const char *) string to be converted
//		2 (char * *) where to store pointer to end char
//		3 (struct in_addr *) where to store 1st address
//		4 (struct in_addr *) where to store 2nd address
//		5 (struct in_addr *) where to store netmask
//		6 (unsigned int *) where to store prefix
//		7 (unsigned int *) where to store count
//		8 (unsigned int *) where to store port
//
// return	(int) < 0 : error
//		(int)   0 : nothing found
//		(int)   1 : valid 1st IPv4 address
//		(int)   2 : valid 2nd IPv4 address
//		(int)   4 : valid count
//		(int)   8 : valid port
//		(int)  16 : valid netmask
//		(int)  32 : valid prefix
//-----------------------------------------------------------------------------
#define IPV4_STR_TO_ADDR_ADDR		0x00000001
#define IPV4_STR_TO_ADDR_ADDR2		0x00000002
#define IPV4_STR_TO_ADDR_COUNT		0x00000004
#define IPV4_STR_TO_ADDR_PORT		0x00000008
#define IPV4_STR_TO_ADDR_PREFIX		0x00000010
#define IPV4_STR_TO_ADDR_NETMASK	0x00000020

#define IPV4_STR_TO_ADDR_BAD_ADDR	0x00010000
#define IPV4_STR_TO_ADDR_BAD_ADDR2	0x00020000
#define IPV4_STR_TO_ADDR_BAD_COUNT	0x00040000
#define IPV4_STR_TO_ADDR_BAD_PORT	0x00080000
#define IPV4_STR_TO_ADDR_BAD_PREFIX	0x00100000

#define IPV4_STR_TO_ADDR_BAD_CHAR	0x40000000

#define IPV4_STR_TO_ADDR_ERROR		0x80000000

int
ipv4_str_to_in_addr (
    const char *	arg_str
    ,
    char * *		arg_endp_p
    ,
    struct in_addr *	arg_addr_p
    ,
    struct in_addr *	arg_addr2_p
    ,
    struct in_addr *	arg_netmask_p
    ,
    unsigned int *	arg_prefix_p
    ,
    unsigned int *	arg_count_p
    ,
    unsigned int *	arg_port_p
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_table_count_node (internal only)
//
// purpose	Recursively count the number of entries in an IPv4 branch.
//
// arguments	1 (ipv4_table_node_p) pointer to table node
//		2 (int) depth
//
// returns	(unsigned long) number of entries
//-----------------------------------------------------------------------------
unsigned long
ipv4_table_count_node (
    ipv4_table_node_p	arg_node
    ,
    int			arg_depth
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_table_delete
//
// purpose	Delete an IPv4 address entry from an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) IPv4 table to delete from
//		2 (ipv4_t) IPv4 (network base) address to delete
//		3 (int) prefix length of network (32 for single IP)
//
// returns	(int) -1 : error
//		(int)  0 : deleted
//		(int)  1 : not found
//-----------------------------------------------------------------------------
int
ipv4_table_delete (
    ipv4_table_p	arg_table
    ,
    ipv4_t		arg_addr
    ,
    int			arg_prefix
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_table_empty_node (internal only)
//
// purpose	Recursively empty each node of an IPv4 lookup table.
//
// arguments	1 (ipv4_table_node_p) pointer to table to empty
//		2 (int) depth
//
// returns	(void)
//-----------------------------------------------------------------------------
void
ipv4_table_empty_node (
    ipv4_table_node_p	arg_node
    ,
    int			arg_depth
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_table_find
//
// purpose	Find an IPv4 address entry in an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) pointer to table to look in
//		2 (ipv4_t) IPv4 address to look up
//
// returns	(void *) pointer associated with entry, or NULL if not found.
//-----------------------------------------------------------------------------
void *
ipv4_table_find (
    ipv4_table_p	arg_table
    ,
    ipv4_t		arg_addr
    )

;

//-----------------------------------------------------------------------------
// function	ipv4_table_find_all
//
// purpose	Find all IPv4 address entries in an IPv4 lookup table that
//		match a given IP address.
//
// arguments	1 (ipv4_table_p) pointer to table to look in
//		2 (ipv4_t) IPv4 address to look up
//		3 ((void *)[33]) array of associated data pointers found
//
// returns	(int) number of data pointers found
//
// note		The array pointed to in argument 3 is filled in indexed by
//		the prefix value from 0 to 32.
//-----------------------------------------------------------------------------
unsigned long
ipv4_table_find_all (
    ipv4_table_p	arg_table
    ,
    ipv4_t		arg_addr
    ,
    void * *		arg_list_p
    )

;

//-----------------------------------------------------------------------------
// function	ipv4_table_insert
//
// purpose	Insert an IPv4 address entry into an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) IPv4 table to insert into
//		2 (ipv4_t) IPv4 (network base) address to insert
//		3 (int) prefix length of network (32 for single IP)
//		4 (void *) arbitrary pointer to associate with this address
//
// note		Only the number of bits specified in the given prefix length
//		will be used from the given IPv4 address.  So any address in
//		the subnet will effectively specify the subnet.
//
// returns	(int) == -1 : error
//		(int) ==  0 : inserted
//		(int) ==  1 : already exists (unchanged)
//-----------------------------------------------------------------------------
int
ipv4_table_insert (
    ipv4_table_p	arg_table
    ,
    ipv4_t		arg_addr
    ,
    int			arg_prefix
    ,
    void *		arg_data_p
    )
;

//-----------------------------------------------------------------------------
// function	ipv4_table_iterate_node (internal only)
//
// purpose	Recursively iterate through all the nodes in an IPv4 lookup
//		table.
//
// arguments	1 (ipv4_table_p) pointer to IPv4 lookup table to iterate on
//		2 (ipv4_t) address of this branch
//		3 (int) prefix of this branch
//		4 (unsigned long(*)(ipv4_t,int,void*,void*)) function
//		5 (void *) extra pointer to pass to this function
//
// returns	(unsigned long) sum of iteration returns
//
// arguments to called function
//		1 (ipv4_t) subnet base IPv4 address
//		2 (int) CIDR prefix length of subnet
//		3 (void *) pointer associated with subnet
//		4 (void *) pointer passed to iteration
//
// return from called function
//		(unsigned long) number to accumulate for the iteration return
//-----------------------------------------------------------------------------
unsigned long long
ipv4_table_iterate_node (
    ipv4_table_node_p		arg_node
    ,
    ipv4_t			arg_addr
    ,
    int				arg_prefix
    ,
    unsigned long ( *		arg_fun		)( ipv4_t, int, void *, void * )
    ,
    void *			arg_ptr
    )
;

//-----------------------------------------------------------------------------
// function	is_invalid_hostname
//
// purpose	Check if the given string represents an invalid hostname.
//		If it is invalid, return a pointer to the first invalid
//		character.  Otherwise return a NULL pointer.
//
// arguments	1 (const char *) string to check for hostname
//
// returns	(char *) pointer to first invalid character
//		(char *) NULL if string is a valid hostname
//-----------------------------------------------------------------------------
char *
is_invalid_hostname (
    const char *	arg_host_ptr
    )
;

//-----------------------------------------------------------------------------
// function	map_hostent_addr_str
//
// purpose	Add entries from a hostent structure (from gethostbyname() and
//		other functions) to the specified mapping with the address as
//		the mapping key, and optional caller supplied string data.
//
// arguments	1 (MAP) mapping to insert addresses into
//		2 (struct hostent *) pointer to hostent structure
//		3 (const char *) string to store in mapping as data
//
// returns	(int) == -1 : error
//		(int) >=  0 : number of addresses inserted
//
// note		If an address is in struct hostent more than one, then each
//		instance will be entered in the mapping as a duplicate.
//-----------------------------------------------------------------------------
int
map_hostent_addr_str (
    MAP			arg_map
    ,
    struct hostent *	arg_hostent
    ,
    const char *	arg_data_str
    )
;

//-----------------------------------------------------------------------------
// function	map_seq_int_hostent
//
// purpose	Add entries from a hostent structure, from gethostbyname() and
//		other functions, to the specified mapping in sequential order
//		with an increasing integer as key and the address as data.
//		This preserves the original sequential order.
//
// arguments	1 (MAP) mapping to insert addresses into
//		2 (struct hostent *) pointer to hostent structure
//
// returns	(int) == -1 : error
//		(int) >=  0 : number of addresses inserted
//-----------------------------------------------------------------------------
int
map_seq_int_hostent (
    MAP			arg_map
    ,
    struct hostent *	arg_hostent
    )
;

#endif /* __NET_H__ */
