//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		time.h
//-----------------------------------------------------------------------------
#ifndef __TIME_H__
#define __TIME_H__

#include <limits.h>
#include <stdint.h>
#include <stdio.h>
#include <sys/time.h>
#include <time.h>

#include <libh/div.h>
#include <libh/ctools.h>

//-----------------------------------------------------------------------------
// type		JD, LJD
//
// purpose	Define a type for a Julian Date value as used in astronomy.
//		Type LJD is a greater precision version of JD.  It is used
//		in some intermediate calculations for less rounding effect.
//-----------------------------------------------------------------------------
typedef double				JD		;
typedef long double			LJD		;


//-----------------------------------------------------------------------------
// type		eday_t
//
// purpose	Define a timekeeping data type called Earth Day.
//
// resolution	1 day
//
// epoch	Monday 24 November 4714 BC Gregorian, 00:00:00
//		Monday 1 January 4713 BC Julian, 00:00:00
//-----------------------------------------------------------------------------
typedef int32_t				eday_t		;

#define EDAY_C(c)			(INT32_C(c))
#define EDAY_MIN			(INT32_MIN)
#define EDAY_MAX			(INT32_MAX)
#define EDAY_ERROR			(INT32_MAX)

//-----------------------------------------------------------------------------
// Define macros to add/subtract intervals of time to/from type etime_t.
//-----------------------------------------------------------------------------
#define eday_add_days(t,d)		((t)+(eday_t)(d))

#define eday_sub_days(t,d)		((t)-(eday_t)(d))

//-----------------------------------------------------------------------------
// Define macros to get the difference between two etime_t values and return
// the result in a specific relative scale.
//-----------------------------------------------------------------------------
#define eday_diff_days(a,b)		((b)-(a))

//-----------------------------------------------------------------------------
// Define special reference dates as EDAY values.
//-----------------------------------------------------------------------------
// The dates refer to the Gregorian proleptic calendar except those that end
// with "_JULIAN" which instead refer to the Julian proleptic calendar.
//-----------------------------------------------------------------------------
#define EDAY_23_NOV_4714_BC		EDAY_C(-1)
#define EDAY_31_DEC_4714_BC_JULIAN	EDAY_C(-1)
#define EDAY_24_NOV_4714_BC		EDAY_C(0)
#define EDAY_1_JAN_4713_BC_JULIAN	EDAY_C(0)
#define EDAY_25_NOV_4714_BC		EDAY_C(1)
#define EDAY_2_JAN_4713_BC_JULIAN	EDAY_C(1)

#define EDAY_30_DEC_2_BC		EDAY_C(1721058)
#define EDAY_1_JAN_1_BC_JULIAN		EDAY_C(1721058)
#define EDAY_1_JAN_0_JULIAN		EDAY_C(1721058)
#define EDAY_1_JAN_1_BC			EDAY_C(1721060)
#define EDAY_3_JAN_1_BC_JULIAN		EDAY_C(1721060)
#define EDAY_1_JAN_0			EDAY_C(1721060)
#define EDAY_3_JAN_0_JULIAN		EDAY_C(1721060)
#define EDAY_30_DEC_1_BC		EDAY_C(1721424)
#define EDAY_1_JAN_1_JULIAN		EDAY_C(1721424)
#define EDAY_1_JAN_1			EDAY_C(1721426)
#define EDAY_3_JAN_1_JULIAN		EDAY_C(1721426)

#define EDAY_14_SEP_763_JULIAN		EDAY_C(2000000)
#define EDAY_18_SEP_763			EDAY_C(2000000)
#define EDAY_27_JUN_1037_JULIAN		EDAY_C(2100000)
#define EDAY_3_JUL_1037			EDAY_C(2100000)
#define EDAY_4_OCT_1582_JULIAN		EDAY_C(2299160)
#define EDAY_14_OCT_1582		EDAY_C(2299160)
#define EDAY_30_DEC_1615_JULIAN		EDAY_C(2311300)
#define EDAY_9_JAN_1616			EDAY_C(2311300)

#define EDAY_16_NOV_1858		EDAY_C(2400000)
#define EDAY_1_JAN_1970			EDAY_C(2440588)
#define EDAY_1_JAN_1980			EDAY_C(2444240)
#define EDAY_1_JAN_2000			EDAY_C(2451545)
#define EDAY_1_JAN_2001			EDAY_C(2451911)
#define EDAY_1_JAN_2100			EDAY_C(2488070)
#define EDAY_1_JAN_2101			EDAY_C(2488435)
#define EDAY_31_AUG_2132		EDAY_C(2500000)
#define EDAY_1_JAN_2400			EDAY_C(2597642)
#define EDAY_1_JAN_2401			EDAY_C(2598008)
#define EDAY_14_AUG_3501		EDAY_C(2999999)
#define EDAY_15_AUG_3501		EDAY_C(3000000)
#define EDAY_10_SEP_4320		EDAY_C(3299160)

#define EDAY_1_JAN_10000		EDAY_C(5373485)
#define EDAY_20_DEC_22666		EDAY_C(10000000)
#define EDAY_20_AUG_26976		EDAY_C(11574074)
#define EDAY_7_AUG_269078		EDAY_C(100000000)
#define EDAY_3_DEC_287564		EDAY_C(106751991)

#define EDAY_UNIX_EPOCH			EDAY_1_JAN_1970
#define EDAY_DOS_EPOCH			EDAY_1_JAN_1980
#define EDAY_SYSTEM_EPOCH		EDAY_UNIX_EPOCH

//-----------------------------------------------------------------------------
// Express the minimum and maximum times for time_t in terms of eday_t.
//-----------------------------------------------------------------------------
#define EDAY_SYSTEM_MIN			(EDAY_SYSTEM_EPOCH+((eday_t)TIME_MIN/EDAY_C(86400)))
#define EDAY_SYSTEM_MAX			(EDAY_SYSTEM_EPOCH+((eday_t)TIME_MAX/EDAY_C(86400)))


//-----------------------------------------------------------------------------
// type		etime_t
//
// purpose	Define a timekeeping data type called Earth Time.
//
// resolution	1 microsecond
//
// epoch	00:00:00.000000 Monday 24 November 4714 BC
//
// earliest	00:00:00.000000 Monday 24 November 4714 BC
// latest	04:00:54.775807 Thursday 3 December 287564
//-----------------------------------------------------------------------------
typedef uint64_t			etime_t		;

#define etime_quo(x,y)			(((etime_t)(x))/((etime_t)(y)))
#define etime_mod(x,y)			(((etime_t)(x))%((etime_t)(y)))

#define ETIME_C(c)			(UINT64_C(c))
#define ETIME_MIN			(UINT64_C(0))
#define ETIME_MAX			(INT64_MAX)
#define ETIME_ERROR			(UINT64_MAX)

#define EDAY_ETIME_MIN			(etime_to_eday(ETIME_MIN))
#define EDAY_ETIME_MAX			(etime_to_eday(ETIME_MAX))

#define eday_too_lo_for_etime(e)	((e)<EDAY_ETIME_MIN)
#define eday_too_hi_for_etime(e)	((e)>EDAY_ETIME_MAX)

//-----------------------------------------------------------------------------
// Define special reference dates as ETIME values.
//-----------------------------------------------------------------------------
// The dates refer to the Gregorian proleptic calendar except those that end
// with "_JULIAN" which instead refer to the Julian proleptic calendar.
//-----------------------------------------------------------------------------
#define ETIME_23_NOV_4714_BC		ETIME_C(-86400000000)
#define ETIME_31_DEC_4714_BC_JULIAN	ETIME_C(-86400000000)
#define ETIME_24_NOV_4714_BC		ETIME_C(0)
#define ETIME_1_JAN_4713_BC_JULIAN	ETIME_C(0)
#define ETIME_25_NOV_4714_BC		ETIME_C(86400000000)
#define ETIME_2_JAN_4713_BC_JULIAN	ETIME_C(86400000000)

#define ETIME_30_DEC_2_BC		ETIME_C(148699411200000000)
#define ETIME_1_JAN_1_BC_JULIAN		ETIME_C(148699411200000000)
#define ETIME_1_JAN_0_JULIAN		ETIME_C(148699411200000000)
#define ETIME_1_JAN_1_BC		ETIME_C(148699584000000000)
#define ETIME_3_JAN_1_BC_JULIAN		ETIME_C(148699584000000000)
#define ETIME_1_JAN_0			ETIME_C(148699584000000000)
#define ETIME_3_JAN_0_JULIAN		ETIME_C(148699584000000000)
#define ETIME_30_DEC_1_BC		ETIME_C(148731033600000000)
#define ETIME_1_JAN_1_JULIAN		ETIME_C(148731033600000000)
#define ETIME_1_JAN_1			ETIME_C(148731206400000000)
#define ETIME_3_JAN_1_JULIAN		ETIME_C(148731206400000000)

#define ETIME_14_SEP_763_JULIAN		ETIME_C(172800000000000000)
#define ETIME_18_SEP_763		ETIME_C(172800000000000000)
#define ETIME_27_JUN_1037_JULIAN	ETIME_C(181440000000000000)
#define ETIME_3_JUL_1037		ETIME_C(181440000000000000)
#define ETIME_4_OCT_1582_JULIAN		ETIME_C(198647424000000000)
#define ETIME_14_OCT_1582		ETIME_C(198647424000000000)
#define ETIME_30_DEC_1615_JULIAN	ETIME_C(199696320000000000)
#define ETIME_9_JAN_1616		ETIME_C(199696320000000000)

#define ETIME_16_NOV_1858		ETIME_C(207360000000000000)
#define ETIME_1_JAN_1970		ETIME_C(210866803200000000)
#define ETIME_1_JAN_1980		ETIME_C(211182336000000000)
#define ETIME_1_JAN_2000		ETIME_C(211813488000000000)
#define ETIME_1_JAN_2001		ETIME_C(211845110400000000)
#define ETIME_1_JAN_2100		ETIME_C(214969248000000000)
#define ETIME_1_JAN_2101		ETIME_C(215000784000000000)
#define ETIME_31_AUG_2132		ETIME_C(216000000000000000)
#define ETIME_1_JAN_2400		ETIME_C(224436268800000000)
#define ETIME_1_JAN_2401		ETIME_C(224467891200000000)
#define ETIME_14_AUG_3501		ETIME_C(259199913600000000)
#define ETIME_15_AUG_3501		ETIME_C(259200000000000000)
#define ETIME_10_SEP_4320		ETIME_C(285047424000000000)

#define ETIME_1_JAN_10000		ETIME_C(464269104000000000)
#define ETIME_20_DEC_22666		ETIME_C(864000000000000000)
#define ETIME_20_AUG_26976		ETIME_C(1000000000000000000)
#define ETIME_7_AUG_269078		ETIME_C(8640000000000000000)
#define ETIME_3_DEC_287564		ETIME_C(9223372022400000000)

#define ETIME_UNIX_EPOCH		ETIME_1_JAN_1970
#define ETIME_DOS_EPOCH			ETIME_1_JAN_1980
#define ETIME_SYSTEM_EPOCH		ETIME_UNIX_EPOCH

//-----------------------------------------------------------------------------
// Express the minimum and maximum times for time_t in terms of etime_t.
//-----------------------------------------------------------------------------
#define ETIME_SYSTEM_MIN		(ETIME_SYSTEM_EPOCH+((etime_t)TIME_MIN*ETIME_C(1000000)))
#define ETIME_SYSTEM_MAX		(ETIME_SYSTEM_EPOCH+((etime_t)TIME_MAX*ETIME_C(1000000))+(ETIME_C(999999)))

//-----------------------------------------------------------------------------
// Define macros to extract time parts from type etime_t.
//-----------------------------------------------------------------------------
#define etime_get_hour_of_day(t)	(etime_mod((t),ETIME_C(86400000000))/ETIME_C(3600000000))
#define etime_get_min_of_day(t)		(etime_mod((t),ETIME_C(86400000000))/ETIME_C(60000000))
#define etime_get_sec_of_day(t)		(etime_mod((t),ETIME_C(86400000000))/ETIME_C(1000000))
#define etime_get_msec_of_day(t)	(etime_mod((t),ETIME_C(86400000000))/ETIME_C(1000))
#define etime_get_usec_of_day(t)	(etime_mod((t),ETIME_C(86400000000)))
#define etime_get_nsec_of_day(t)	(etime_mod((t),ETIME_C(86400000000))*ETIME_C(1000))

#define etime_get_min_of_hour(t)	(etime_mod((t),ETIME_C(3600000000))/ETIME_C(60000000))
#define etime_get_sec_of_hour(t)	(etime_mod((t),ETIME_C(3600000000))/ETIME_C(1000000))
#define etime_get_msec_of_hour(t)	(etime_mod((t),ETIME_C(3600000000))/ETIME_C(1000))
#define etime_get_usec_of_hour(t)	(etime_mod((t),ETIME_C(3600000000)))
#define etime_get_nsec_of_hour(t)	(etime_mod((t),ETIME_C(3600000000))*ETIME_C(1000))

#define etime_get_sec_of_min(t)		(etime_mod((t),ETIME_C(60000000))/ETIME_C(1000000))
#define etime_get_msec_of_min(t)	(etime_mod((t),ETIME_C(60000000))/ETIME_C(1000))
#define etime_get_usec_of_min(t)	(etime_mod((t),ETIME_C(60000000)))
#define etime_get_nsec_of_min(t)	(etime_mod((t),ETIME_C(60000000))*ETIME_C(1000))

#define etime_get_msec_of_sec(t)	(etime_mod((t),ETIME_C(1000000))/ETIME_C(1000))
#define etime_get_usec_of_sec(t)	(etime_mod((t),ETIME_C(1000000)))
#define etime_get_nsec_of_sec(t)	(etime_mod((t),ETIME_C(1000000))*ETIME_C(1000))

#define etime_get_usec_of_msec(t)	(etime_mod((t),ETIME_C(1000)))
#define etime_get_nsec_of_msec(t)	(etime_mod((t),ETIME_C(1000))*ETIME_C(1000))

#define etime_get_nsec_of_usec(t)	(0)

//-----------------------------------------------------------------------------
// Define macros to add/subtract intervals of time to/from type etime_t.
//-----------------------------------------------------------------------------
#define etime_add_days(t,d)		((t)+(long long)(d)*86400000000LL)
#define etime_add_hours(t,h)		((t)+(long long)(h)*3600000000LL)
#define etime_add_mins(t,m)		((t)+(long long)(m)*60000000LL)
#define etime_add_secs(t,s)		((t)+(long long)(s)*1000000LL)
#define etime_add_msecs(t,m)		((t)+(long long)(m)*1000LL)
#define etime_add_usecs(t,u)		((t)+(long long)(u))
#define etime_add_nsecs(t,n)		((t)+(long long)(n)/1000LL)

#define etime_sub_days(t,d)		((t)-(long long)(d)*86400000000LL)
#define etime_sub_hours(t,h)		((t)-(long long)(h)*3600000000LL)
#define etime_sub_mins(t,m)		((t)-(long long)(m)*60000000LL)
#define etime_sub_secs(t,s)		((t)-(long long)(s)*1000000LL)
#define etime_sub_msecs(t,m)		((t)-(long long)(m)*1000LL)
#define etime_sub_usecs(t,u)		((t)-(long long)(u))
#define etime_sub_nsecs(t,n)		((t)-(long long)(n)/1000LL)

//-----------------------------------------------------------------------------
// Define macros to get the difference between two etime_t values and return
// the result in a specific relative scale.
//-----------------------------------------------------------------------------
#define etime_diff_days(a,b)		(etime_quo((b),86400000000LL)-etime_quo((a),86400000000LL))
#define etime_diff_hours(a,b)		(etime_quo((b),3600000000LL)-etime_quo((a),3600000000LL))
#define etime_diff_mins(a,b)		(etime_quo((b),60000000LL)-etime_quo((a),60000000LL))
#define etime_diff_secs(a,b)		(etime_quo((b),1000000LL)-etime_quo((a),1000000LL))
#define etime_diff_msecs(a,b)		(etime_quo((b),1000L-etime_quo((a),1000L)))
#define etime_diff_usecs(a,b)		((b)-(a))
#define etime_diff_nsecs(a,b)		(((b)-(a))*1000LL)


//-----------------------------------------------------------------------------
// struct	ecal_t
//
// purpose	Combine calendar and time elements into one struct.
//
// note		This struct is similar to (struct tm) but includes
//		1.  a calendar type flag (defined by ECAL_TYPE_*)
//		2.  nanoseconds part of a second
//		3.  attoseconds part of a nanosecond
//-----------------------------------------------------------------------------
struct ecal_t {
    int		type		;
    int		isleap		;
    long int	year		;
    int		month		;
    int		yday		;
    int		mday		;
    int		wday		;
    int		week		;
    int		hour		;
    int		min		;
    int		sec		;
    long int	nsec		;
    long int	asec		;
};
typedef struct ecal_t		ecal_t		;

#define ECAL_ISLEAP_NOTSPEC	-1
#define ECAL_ISLEAP_NO		0
#define ECAL_ISLEAP_YES		1

#define ECAL_TYPE_NOTSPEC	0
#define ECAL_TYPE_JULIAN	1
#define ECAL_TYPE_GREGORIAN	2


//-----------------------------------------------------------------------------
// type		extime_t
//
// purpose	Define a timekeeping data type for extreme purposes.
//
// resolution	1 attosecond
//
// range	over 584 billion years
//
// epoch	00:00:00.000000 Monday 24 November 4714 BC
//-----------------------------------------------------------------------------
struct extime_t {
    int64_t	sec		;	// seconds since epoch
    int64_t	asec		;	// plus attoseconds
};
typedef struct extime_t		extime_t		;
typedef struct extime_t *	extime_p		;


//-----------------------------------------------------------------------------
// type		lday_t
//
// purpose	This is the type for a Lilian Day value.  The Lilian Day is
//		the number of days since the day before the Gregorian calendar
//		was first put into use (15 October 1582 Gregorian, which is
//		Lilian day 1).  It was created by Aloisius Lilius, advisor to
//		Pope Gregory XIII, and later proposed by IBM as an alternative
//		date representation for Y2K issues.
//
//		The LDAY format does not provide for distinction between dates
//		in this format when mixed with dates in common formats such as
//		DDMMYY as used in Europe.  The date 15 December 2001 will be
//		the value 151201 in DDMMYY format, but this same number in LDAY
//		format represents 4 October 1996.  So LDAY dates and DDMMYY or
//		MMDDYY cannot be intermixed.  For that capability, see the XDAY
//		format below.
//
//		At least 32 bits is needed.
//-----------------------------------------------------------------------------
typedef int32_t				lday_t		;

#define LDAY_C(c)			(INT32_C(c))
#define LDAY_ERROR			(INT32_MIN)
#define LDAY_MIN			(INT32_MIN+LDAY_C(1))
#define LDAY_MAX			(INT32_MAX)

//-----------------------------------------------------------------------------
// Define special reference dates as LDAY values.
//-----------------------------------------------------------------------------
// The dates refer to the Gregorian proleptic calendar except those that end
// with "_JULIAN" which instead refer to the Julian proleptic calendar.
//-----------------------------------------------------------------------------
#define LDAY_23_NOV_4714_BC		LDAY_C(-2299161)
#define LDAY_31_DEC_4714_BC_JULIAN	LDAY_C(-2299161)
#define LDAY_24_NOV_4714_BC		LDAY_C(-2299160)
#define LDAY_1_JAN_4713_BC_JULIAN	LDAY_C(-2299160)
#define LDAY_25_NOV_4714_BC		LDAY_C(-2299159)
#define LDAY_2_JAN_4713_BC_JULIAN	LDAY_C(-2299159)

#define LDAY_30_DEC_2_BC		LDAY_C(-578102)
#define LDAY_1_JAN_1_BC_JULIAN		LDAY_C(-578102)
#define LDAY_1_JAN_0_JULIAN		LDAY_C(-578102)
#define LDAY_1_JAN_1_BC			LDAY_C(-578100)
#define LDAY_3_JAN_1_BC_JULIAN		LDAY_C(-578100)
#define LDAY_1_JAN_0			LDAY_C(-578100)
#define LDAY_3_JAN_0_JULIAN		LDAY_C(-578100)
#define LDAY_30_DEC_1_BC		LDAY_C(-577736)
#define LDAY_1_JAN_1_JULIAN		LDAY_C(-577736)
#define LDAY_1_JAN_1			LDAY_C(-577734)
#define LDAY_3_JAN_1_JULIAN		LDAY_C(-577734)

#define LDAY_14_SEP_763_JULIAN		LDAY_C(-299160)
#define LDAY_18_SEP_763			LDAY_C(-299160)
#define LDAY_27_JUN_1037_JULIAN		LDAY_C(-199160)
#define LDAY_3_JUL_1037			LDAY_C(-199160)
#define LDAY_4_OCT_1582_JULIAN		LDAY_C(0)
#define LDAY_14_OCT_1582		LDAY_C(0)
#define LDAY_30_DEC_1615_JULIAN		LDAY_C(12140)
#define LDAY_9_JAN_1616			LDAY_C(12140)

#define LDAY_16_NOV_1858		LDAY_C(100840)
#define LDAY_1_JAN_1970			LDAY_C(141428)
#define LDAY_1_JAN_1980			LDAY_C(145080)
#define LDAY_1_JAN_2000			LDAY_C(152385)
#define LDAY_1_JAN_2001			LDAY_C(152751)
#define LDAY_1_JAN_2100			LDAY_C(188910)
#define LDAY_1_JAN_2101			LDAY_C(189275)
#define LDAY_31_AUG_2132		LDAY_C(200840)
#define LDAY_1_JAN_2400			LDAY_C(298482)
#define LDAY_1_JAN_2401			LDAY_C(298848)
#define LDAY_14_AUG_3501		LDAY_C(700839)
#define LDAY_15_AUG_3501		LDAY_C(700840)
#define LDAY_10_SEP_4320		LDAY_C(1000000)

#define LDAY_UNIX_EPOCH			LDAY_1_JAN_1970
#define LDAY_DOS_EPOCH			LDAY_1_JAN_1980
#define LDAY_SYSTEM_EPOCH		LDAY_UNIX_EPOCH

//-----------------------------------------------------------------------------
// Express the minimum and maximum times for time_t in terms of lday_t.
//-----------------------------------------------------------------------------
#define LDAY_SYSTEM_MIN			(LDAY_SYSTEM_EPOCH+((lday_t)TIME_MIN/LDAY_C(86400)))
#define LDAY_SYSTEM_MAX			(LDAY_SYSTEM_EPOCH+((lday_t)TIME_MAX/LDAY_C(86400)))


//-----------------------------------------------------------------------------
// type		xday_t
//
// purpose	This is the type for an eXchange Day value.  The eXchange Day
//		is the Julian Date minus 1999999.5 (Earth Day minus 2000000) to
//		make 6 digits with a value distinguishable from any MMDDYY or
//		DDMMYY formatted dates, to provide a means to store dates in
//		existing 6 digit date fields while co-existing with unconverted
//		dates and detecting them appropriately.
//
//		Since the largest valid date stored in DDMMYY format would be
//		311299, the XDAY could be distinguished with a value of 311300
//		(9 Jan 1616 Gregorian) or greater.  Simpler testing of dates
//		stored as character digits can be used if XDAY is limited to
//		400000 (16 Nov 1858) which allows testing the first digit for
//		'4' or greater.  The XDAY format is useful for 6 digit date
//		fields until XDAY value 999999 (14 Aug 3501).
//
//		The eXchange Day was designed by Bob Bemer.
//
//		At least 32 bits is needed.
//-----------------------------------------------------------------------------
typedef int32_t				xday_t		;

#define XDAY_C(c)			(INT32_C(c))
#define XDAY_ERROR			(INT32_MIN)
#define XDAY_MIN			(INT32_MIN+XDAY_C(1))
#define XDAY_MAX			(INT32_MAX)

//-----------------------------------------------------------------------------
// Define special reference dates as XDAY values.
//-----------------------------------------------------------------------------
// The dates refer to the Gregorian proleptic calendar except those that end
// with "_JULIAN" which instead refer to the Julian proleptic calendar.
//-----------------------------------------------------------------------------
#define XDAY_23_NOV_4714_BC		XDAY_C(-2000001)
#define XDAY_31_DEC_4714_BC_JULIAN	XDAY_C(-2000001)
#define XDAY_24_NOV_4714_BC		XDAY_C(-2000000)
#define XDAY_1_JAN_4713_BC_JULIAN	XDAY_C(-2000000)
#define XDAY_25_NOV_4714_BC		XDAY_C(-1999999)
#define XDAY_2_JAN_4713_BC_JULIAN	XDAY_C(-1999999)

#define XDAY_30_DEC_2_BC		XDAY_C(-278942)
#define XDAY_1_JAN_1_BC_JULIAN		XDAY_C(-278942)
#define XDAY_1_JAN_0_JULIAN		XDAY_C(-278942)
#define XDAY_1_JAN_1_BC			XDAY_C(-278940)
#define XDAY_3_JAN_1_BC_JULIAN		XDAY_C(-278940)
#define XDAY_1_JAN_0			XDAY_C(-278940)
#define XDAY_3_JAN_0_JULIAN		XDAY_C(-278940)
#define XDAY_30_DEC_1_BC		XDAY_C(-278576)
#define XDAY_1_JAN_1_JULIAN		XDAY_C(-278576)
#define XDAY_1_JAN_1			XDAY_C(-278574)
#define XDAY_3_JAN_1_JULIAN		XDAY_C(-278574)

#define XDAY_14_SEP_763_JULIAN		XDAY_C(0)
#define XDAY_18_SEP_763			XDAY_C(0)
#define XDAY_27_JUN_1037_JULIAN		XDAY_C(100000)
#define XDAY_3_JUL_1037			XDAY_C(100000)
#define XDAY_4_OCT_1582_JULIAN		XDAY_C(299160)
#define XDAY_14_OCT_1582		XDAY_C(299160)
#define XDAY_30_DEC_1615_JULIAN		XDAY_C(311300)
#define XDAY_9_JAN_1616			XDAY_C(311300)

#define XDAY_16_NOV_1858		XDAY_C(400000)
#define XDAY_1_JAN_1970			XDAY_C(440588)
#define XDAY_1_JAN_1980			XDAY_C(444240)
#define XDAY_1_JAN_2000			XDAY_C(451545)
#define XDAY_1_JAN_2001			XDAY_C(451911)
#define XDAY_1_JAN_2100			XDAY_C(488070)
#define XDAY_1_JAN_2101			XDAY_C(488435)
#define XDAY_31_AUG_2132		XDAY_C(500000)
#define XDAY_1_JAN_2400			XDAY_C(597642)
#define XDAY_1_JAN_2401			XDAY_C(598008)
#define XDAY_14_AUG_3501		XDAY_C(999999)
#define XDAY_15_AUG_3501		XDAY_C(1000000)

#define XDAY_UNIX_EPOCH			XDAY_1_JAN_1970
#define XDAY_DOS_EPOCH			XDAY_1_JAN_1980
#define XDAY_SYSTEM_EPOCH		XDAY_UNIX_EPOCH

//-----------------------------------------------------------------------------
// Express the minimum and maximum times for time_t in terms of xday_t.
//-----------------------------------------------------------------------------
#define XDAY_SYSTEM_MIN			(XDAY_SYSTEM_EPOCH+((xday_t)TIME_MIN/XDAY_C(86400)))
#define XDAY_SYSTEM_MAX			(XDAY_SYSTEM_EPOCH+((xday_t)TIME_MAX/XDAY_C(86400)))


//-----------------------------------------------------------------------------
// typedef	time_t (not actually defined here)
//
// purpose	Number of seconds since host epoch, generally in 32 bits,
//		with negative values representing time before the epoch.
//
// note		UNIX uses an epoch of 1970-01-01.00:00:00
//		There is currently no support for other host epochs.
//-----------------------------------------------------------------------------
#if 0
#define TIME_MIN			(((time_t)1)<<(((sizeof(time_t))*(CHAR_BIT))-1))
#define TIME_MAX			(~(((time_t)1)<<(((sizeof(time_t))*(CHAR_BIT))-1)))
#else
#define TIME_MIN			int_min(time_t)
#define TIME_MAX			int_max(time_t)
#endif

//-----------------------------------------------------------------------------
// Define macros to extract time parts from type time_t.
//-----------------------------------------------------------------------------
#define time_get_hour_of_day(t)		(mod_ni((t),TIME_C(86400))/TIME_C(3600))
#define time_get_min_of_day(t)		(mod_ni((t),TIME_C(86400))/TIME_C(60))
#define time_get_sec_of_day(t)		(mod_ni((t),TIME_C(86400)))

#define time_get_min_of_hour(t)		(mod_ni((t),TIME_C(3600))/TIME_C(60))
#define time_get_sec_of_hour(t)		(mod_ni((t),TIME_C(3600)))

#define time_get_sec_of_min(t)		(mod_ni((t),TIME_C(60)))

//-----------------------------------------------------------------------------
// Define macros to add/subtract intervals of time to/from type time_t.
//-----------------------------------------------------------------------------
#define time_add_days(t,d)		((t)+(long long)(d)*86400LL)
#define time_add_hours(t,h)		((t)+(long long)(h)*3600LL)
#define time_add_mins(t,m)		((t)+(long long)(m)*60LL)
#define time_add_secs(t,s)		((t)+(long long)(s))
#define time_add_msecs(t,m)		((t)+(long long)(m)/1000LL)
#define time_add_usecs(t,u)		((t)+(long long)(u)/1000000LL)
#define time_add_nsecs(t,n)		((t)+(long long)(n)/1000000000LL)

#define time_sub_days(t,d)		((t)-(long long)(d)*86400LL)
#define time_sub_hours(t,h)		((t)-(long long)(h)*3600LL)
#define time_sub_mins(t,m)		((t)-(long long)(m)*60LL)
#define time_sub_secs(t,s)		((t)-(long long)(s))
#define time_sub_msecs(t,m)		((t)-(long long)(m)/1000LL)
#define time_sub_usecs(t,u)		((t)-(long long)(u)/1000000LL)
#define time_sub_nsecs(t,n)		((t)-(long long)(n)/1000000000LL)

//-----------------------------------------------------------------------------
// Define macros to get the difference between two time_t values and return
// the result in a specific relative scale.
//-----------------------------------------------------------------------------
#define time_diff_days(a,b)		(quo_ni((b),86400LL)-quo_ni((a),86400LL))
#define time_diff_hours(a,b)		(quo_ni((b),3600LL)-quo_ni((a),3600LL))
#define time_diff_mins(a,b)		(quo_ni((b),60LL)-quo_ni((a),60LL))
#define time_diff_secs(a,b)		((b)-(a))
#define time_diff_msecs(a,b)		(((b)-(a))*1000LL)
#define time_diff_usecs(a,b)		(((b)-(a))*1000000LL)
#define time_diff_nsecs(a,b)		(((b)-(a))*1000000000LL)


//-----------------------------------------------------------------------------
// typedef	time64_t
//
// purpose	Number of seconds since host epoch, in at least 64 bits,
//		with negative values representing time before the epoch.
//
// note		UNIX uses an epoch of 1970-01-01.00:00:00
//		There is currently no support for other host epochs.
//
// note		This type is provided as a means for programs that need to use
//		64 bits for a time measured in seconds to do so regardless of
//		the size the system defines for time_t.
//-----------------------------------------------------------------------------
typedef int64_t				time64_t	;

#define TIME64_C(c)			(INT64_C(c))
#define TIME64_ERROR			(INT64_MIN)
#define TIME64_MIN			(INT64_MIN+TIME64_C(1))
#define	TIME64_MAX			(INT64_MAX)

//-----------------------------------------------------------------------------
// Define macros to extract time parts from type time64_t.
//-----------------------------------------------------------------------------
#define time64_get_hour_of_day(t)	(mod_ni((t),TIME64_C(86400))/TIME64_C(3600))
#define time64_get_min_of_day(t)	(mod_ni((t),TIME64_C(86400))/TIME64_C(60))
#define time64_get_sec_of_day(t)	(mod_ni((t),TIME64_C(86400)))
#define time64_get_msec_of_day(t)	(mod_ni((t),TIME64_C(86400))*TIME64_C(1000))
#define time64_get_usec_of_day(t)	(mod_ni((t),TIME64_C(86400))*TIME64_C(1000000))
#define time64_get_nsec_of_day(t)	(mod_ni((t),TIME64_C(86400))*TIME64_C(1000000000))

#define time64_get_min_of_hour(t)	(mod_ni((t),TIME64_C(3600))/TIME64_C(60))
#define time64_get_sec_of_hour(t)	(mod_ni((t),TIME64_C(3600)))
#define time64_get_msec_of_hour(t)	(mod_ni((t),TIME64_C(3600))*TIME64_C(1000))
#define time64_get_usec_of_hour(t)	(mod_ni((t),TIME64_C(3600))*TIME64_C(1000000))
#define time64_get_nsec_of_hour(t)	(mod_ni((t),TIME64_C(3600))*TIME64_C(1000000000))

#define time64_get_sec_of_min(t)	(mod_ni((t),TIME64_C(60)))
#define time64_get_msec_of_min(t)	(mod_ni((t),TIME64_C(60))*TIME64_C(1000))
#define time64_get_usec_of_min(t)	(mod_ni((t),TIME64_C(60))*TIME64_C(1000000))
#define time64_get_nsec_of_min(t)	(mod_ni((t),TIME64_C(60))*TIME64_C(1000000000))

#define time64_get_msec_of_sec(t)	(0)
#define time64_get_usec_of_sec(t)	(0)
#define time64_get_nsec_of_sec(t)	(0)

#define time64_get_usec_of_msec(t)	(0)
#define time64_get_nsec_of_msec(t)	(0)

#define time64_get_nsec_of_usec(t)	(0)

//-----------------------------------------------------------------------------
// Define macros to add/subtract intervals of time to/from type time64_t.
//-----------------------------------------------------------------------------
#define time64_add_days(t,d)		((t)+(long long)(d)*86400LL)
#define time64_add_hours(t,h)		((t)+(long long)(h)*3600LL)
#define time64_add_mins(t,m)		((t)+(long long)(m)*60LL)
#define time64_add_secs(t,s)		((t)+(long long)(s))
#define time64_add_msecs(t,m)		((t)+(long long)(m)/1000LL)
#define time64_add_usecs(t,u)		((t)+(long long)(u)/1000000LL)
#define time64_add_nsecs(t,n)		((t)+(long long)(n)/1000000000LL)

#define time64_sub_days(t,d)		((t)-(long long)(d)*86400LL)
#define time64_sub_hours(t,h)		((t)-(long long)(h)*3600LL)
#define time64_sub_mins(t,m)		((t)-(long long)(m)*60LL)
#define time64_sub_secs(t,s)		((t)-(long long)(s))
#define time64_sub_msecs(t,m)		((t)-(long long)(m)/1000LL)
#define time64_sub_usecs(t,u)		((t)-(long long)(u)/1000000LL)
#define time64_sub_nsecs(t,n)		((t)-(long long)(n)/1000000000LL)

//-----------------------------------------------------------------------------
// Define macros to get the difference between two time64_t values and return
// the result in a specific relative scale.
//-----------------------------------------------------------------------------
#define time64_diff_days(a,b)		(quo_ni((b),86400LL)-quo_ni((a),86400LL))
#define time64_diff_hours(a,b)		(quo_ni((b),3600LL)-quo_ni((a),3600LL))
#define time64_diff_mins(a,b)		(quo_ni((b),60LL)-quo_ni((a),60LL))
#define time64_diff_secs(a,b)		((b)-(a))
#define time64_diff_msecs(a,b)		(((b)-(a))*1000LL)
#define time64_diff_usecs(a,b)		(((b)-(a))*1000000LL)
#define time64_diff_nsecs(a,b)		(((b)-(a))*1000000000LL)


//-----------------------------------------------------------------------------
// typedef	timems_t
//
// purpose	Number of milliseconds since host epoch, in at least 64 bits,
//		with negative values representing time before the epoch.
//
// note		UNIX uses an epoch of 1970-01-01.00:00:00
//		There is currently no support for other host epochs.
//-----------------------------------------------------------------------------
typedef int64_t				timems_t	;

#define TIMEMS_C(c)			(INT64_C(c))
#define TIMEMS_ERROR			(INT64_MIN)
#define TIMEMS_MIN			(INT64_MIN+TIMEMS_C(1))
#define	TIMEMS_MAX			(INT64_MAX)

//-----------------------------------------------------------------------------
// Define macros to extract time parts from type timems_t.
//-----------------------------------------------------------------------------
#define timems_get_hour_of_day(t)	(mod_ni((t),TIMEMS_C(86400000))/TIMEMS_C(3600000))
#define timems_get_min_of_day(t)	(mod_ni((t),TIMEMS_C(86400000))/TIMEMS_C(60000))
#define timems_get_sec_of_day(t)	(mod_ni((t),TIMEMS_C(86400000))/TIMEMS_C(1000))
#define timems_get_msec_of_day(t)	(mod_ni((t),TIMEMS_C(86400000)))
#define timems_get_usec_of_day(t)	(mod_ni((t),TIMEMS_C(86400000))*TIMEMS_C(1000))
#define timems_get_nsec_of_day(t)	(mod_ni((t),TIMEMS_C(86400000))*TIMEMS_C(1000000))

#define timems_get_min_of_hour(t)	(mod_ni((t),TIMEMS_C(3600000))/TIMEMS_C(60000))
#define timems_get_sec_of_hour(t)	(mod_ni((t),TIMEMS_C(3600000))/TIMEMS_C(1000))
#define timems_get_msec_of_hour(t)	(mod_ni((t),TIMEMS_C(3600000)))
#define timems_get_usec_of_hour(t)	(mod_ni((t),TIMEMS_C(3600000))*TIMEMS_C(1000))
#define timems_get_nsec_of_hour(t)	(mod_ni((t),TIMEMS_C(3600000))*TIMEMS_C(1000000))

#define timems_get_sec_of_min(t)	(mod_ni((t),TIMEMS_C(60000))/TIMEMS_C(1000))
#define timems_get_msec_of_min(t)	(mod_ni((t),TIMEMS_C(60000)))
#define timems_get_usec_of_min(t)	(mod_ni((t),TIMEMS_C(60000))*TIMEMS_C(1000))
#define timems_get_nsec_of_min(t)	(mod_ni((t),TIMEMS_C(60000))*TIMEMS_C(1000000))

#define timems_get_msec_of_sec(t)	(mod_ni((t),TIMEMS_C(1000)))
#define timems_get_usec_of_sec(t)	(mod_ni((t),TIMEMS_C(1000))*TIMEMS_C(1000))
#define timems_get_nsec_of_sec(t)	(mod_ni((t),TIMEMS_C(1000))*TIMEMS_C(1000000))

#define timems_get_usec_of_msec(t)	(0)
#define timems_get_nsec_of_msec(t)	(0)

#define timems_get_nsec_of_usec(t)	(0)

//-----------------------------------------------------------------------------
// Define macros to add/subtract intervals of time to/from type timems_t.
//-----------------------------------------------------------------------------
#define timems_add_days(t,d)		((t)+(long long)(d)*86400000LL)
#define timems_add_hours(t,h)		((t)+(long long)(h)*3600000LL)
#define timems_add_mins(t,m)		((t)+(long long)(m)*60000LL)
#define timems_add_secs(t,s)		((t)+(long long)(s)*1000LL)
#define timems_add_msecs(t,m)		((t)+(long long)(m))
#define timems_add_usecs(t,u)		((t)+(long long)(u)/1000LL)
#define timems_add_nsecs(t,n)		((t)+(long long)(n)/1000000LL)

#define timems_sub_days(t,d)		((t)-(long long)(d)*86400000LL)
#define timems_sub_hours(t,h)		((t)-(long long)(h)*3600000LL)
#define timems_sub_mins(t,m)		((t)-(long long)(m)*60000LL)
#define timems_sub_secs(t,s)		((t)-(long long)(s)*1000LL)
#define timems_sub_msecs(t,m)		((t)-(long long)(m))
#define timems_sub_usecs(t,u)		((t)-(long long)(u)/1000LL)
#define timems_sub_nsecs(t,n)		((t)-(long long)(n)/1000000LL)

//-----------------------------------------------------------------------------
// Define macros to get the difference between two timems_t values and return
// the result in a specific relative scale.
//-----------------------------------------------------------------------------
#define timems_diff_days(a,b)		(quo_ni((b),86400000LL)-quo_ni((a),86400000LL))
#define timems_diff_hours(a,b)		(quo_ni((b),3600000LL)-quo_ni((a),3600000LL))
#define timems_diff_mins(a,b)		(quo_ni((b),60000LL)-quo_ni((a),60000LL))
#define timems_diff_secs(a,b)		(quo_ni((b),1000LL)-quo_ni((a),1000LL))
#define timems_diff_msecs(a,b)		((b)-(a))
#define timems_diff_usecs(a,b)		(((b)-(a))*1000LL)
#define timems_diff_nsecs(a,b)		(((b)-(a))*1000000LL)


//-----------------------------------------------------------------------------
// typedef	timeus_t
//
// purpose	Number of microseconds since host epoch, in at least 64 bits,
//		with negative values representing time before the epoch.
//
// note		UNIX uses an epoch of 1970-01-01.00:00:00
//		There is currently no support for other host epochs.
//-----------------------------------------------------------------------------
typedef int64_t				timeus_t	;

#define TIMEUS_C(c)			(INT64_C(c))
#define TIMEUS_MIN			(INT64_MIN+TIMEUS_C(1))
#define	TIMEUS_MAX			(INT64_MAX)

//-----------------------------------------------------------------------------
// Define macros to extract time parts from type timeus_t.
//-----------------------------------------------------------------------------
#define timeus_get_hour_of_day(t)	(mod_ni((t),TIMEUS_C(86400000000))/TIMEUS_C(3600000000))
#define timeus_get_min_of_day(t)	(mod_ni((t),TIMEUS_C(86400000000))/TIMEUS_C(60000000))
#define timeus_get_sec_of_day(t)	(mod_ni((t),TIMEUS_C(86400000000))/TIMEUS_C(1000000))
#define timeus_get_msec_of_day(t)	(mod_ni((t),TIMEUS_C(86400000000))/TIMEUS_C(1000))
#define timeus_get_usec_of_day(t)	(mod_ni((t),TIMEUS_C(86400000000)))
#define timeus_get_nsec_of_day(t)	(mod_ni((t),TIMEUS_C(86400000000))*TIMEUS_C(1000))

#define timeus_get_min_of_hour(t)	(mod_ni((t),TIMEUS_C(3600000000))/TIMEUS_C(60000000))
#define timeus_get_sec_of_hour(t)	(mod_ni((t),TIMEUS_C(3600000000))/TIMEUS_C(1000000))
#define timeus_get_msec_of_hour(t)	(mod_ni((t),TIMEUS_C(3600000000))/TIMEUS_C(1000))
#define timeus_get_usec_of_hour(t)	(mod_ni((t),TIMEUS_C(3600000000)))
#define timeus_get_nsec_of_hour(t)	(mod_ni((t),TIMEUS_C(3600000000))*TIMEUS_C(1000))

#define timeus_get_sec_of_min(t)	(mod_ni((t),TIMEUS_C(60000000))/TIMEUS_C(1000000))
#define timeus_get_msec_of_min(t)	(mod_ni((t),TIMEUS_C(60000000))/TIMEUS_C(1000))
#define timeus_get_usec_of_min(t)	(mod_ni((t),TIMEUS_C(60000000)))
#define timeus_get_nsec_of_min(t)	(mod_ni((t),TIMEUS_C(60000000))*TIMEUS_C(1000))

#define timeus_get_msec_of_sec(t)	(mod_ni((t),TIMEUS_C(1000000))/TIMEUS_C(1000))
#define timeus_get_usec_of_sec(t)	(mod_ni((t),TIMEUS_C(1000000)))
#define timeus_get_nsec_of_sec(t)	(mod_ni((t),TIMEUS_C(1000000))*TIMEUS_C(1000))

#define timeus_get_usec_of_msec(t)	(mod_ni((t),TIMEUS_C(1000)))
#define timeus_get_nsec_of_msec(t)	(mod_ni((t),TIMEUS_C(1000))*TIMEUS_C(1000))

#define timeus_get_nsec_of_usec(t)	(0)

//-----------------------------------------------------------------------------
// Define macros to add/subtract intervals of time to/from type timeus_t.
//-----------------------------------------------------------------------------
#define timeus_add_days(t,d)		((t)+(long long)(d)*86400000000LL)
#define timeus_add_hours(t,h)		((t)+(long long)(h)*3600000000LL)
#define timeus_add_mins(t,m)		((t)+(long long)(m)*60000000LL)
#define timeus_add_secs(t,s)		((t)+(long long)(s)*1000000LL)
#define timeus_add_msecs(t,m)		((t)+(long long)(m)*1000LL)
#define timeus_add_usecs(t,u)		((t)+(long long)(u))
#define timeus_add_nsecs(t,n)		((t)+(long long)(n)/1000LL)

#define timeus_sub_days(t,d)		((t)-(long long)(d)*86400000000LL)
#define timeus_sub_hours(t,h)		((t)-(long long)(h)*3600000000LL)
#define timeus_sub_mins(t,m)		((t)-(long long)(m)*60000000LL)
#define timeus_sub_secs(t,s)		((t)-(long long)(s)*1000000LL)
#define timeus_sub_msecs(t,m)		((t)-(long long)(m)*1000LL)
#define timeus_sub_usecs(t,u)		((t)-(long long)(u))
#define timeus_sub_nsecs(t,n)		((t)-(long long)(n)/1000LL)

//-----------------------------------------------------------------------------
// Define macros to get the difference between two timeus_t values and return
// the result in a specific relative scale.
//-----------------------------------------------------------------------------
#define timeus_diff_days(a,b)		(quo_ni((b),86400000000LL)-quo_ni((a),86400000000LL))
#define timeus_diff_hours(a,b)		(quo_ni((b),3600000000LL)-quo_ni((a),3600000000LL))
#define timeus_diff_mins(a,b)		(quo_ni((b),60000000LL)-quo_ni((a),60000000LL))
#define timeus_diff_secs(a,b)		(quo_ni((b),1000000LL)-quo_ni((a),1000000LL))
#define timeus_diff_msecs(a,b)		(quo_ni((b),1000LL)-quo_ni((a),1000LL))
#define timeus_diff_usecs(a,b)		((b)-(a))
#define timeus_diff_nsecs(a,b)		(((b)-(a))*1000LL)


//-----------------------------------------------------------------------------
// typedef	timens_t
//
// purpose	Number of nanoseconds since host epoch, in at least 64 bits,
//		with negative values representing time before the epoch.
//
// note		UNIX uses an epoch of 1970-01-01.00:00:00
//		There is currently no support for other host epochs.
//-----------------------------------------------------------------------------
typedef int64_t				timens_t	;

#define TIMENS_C(c)			(INT64_C(c))
#define TIMENS_ERROR			(INT64_MIN)
#define TIMENS_MIN			(INT64_MIN+TIMENS_C(1))
#define	TIMENS_MAX			(INT64_MAX)

//-----------------------------------------------------------------------------
// Define macros to extract time parts from type timens_t.
//-----------------------------------------------------------------------------
#define timens_get_hour_of_day(t)	(mod_ni((t),TIMENS_C(86400000000000))/TIMENS_C(3600000000000))
#define timens_get_min_of_day(t)	(mod_ni((t),TIMENS_C(86400000000000))/TIMENS_C(60000000000))
#define timens_get_sec_of_day(t)	(mod_ni((t),TIMENS_C(86400000000000))/TIMENS_C(1000000000))
#define timens_get_msec_of_day(t)	(mod_ni((t),TIMENS_C(86400000000000))/TIMENS_C(1000000))
#define timens_get_usec_of_day(t)	(mod_ni((t),TIMENS_C(86400000000000))/TIMENS_C(1000))
#define timens_get_nsec_of_day(t)	(mod_ni((t),TIMENS_C(86400000000000)))

#define timens_get_min_of_hour(t)	(mod_ni((t),TIMENS_C(3600000000000))/TIMENS_C(60000000000))
#define timens_get_sec_of_hour(t)	(mod_ni((t),TIMENS_C(3600000000000))/TIMENS_C(1000000000))
#define timens_get_msec_of_hour(t)	(mod_ni((t),TIMENS_C(3600000000000))/TIMENS_C(1000000))
#define timens_get_usec_of_hour(t)	(mod_ni((t),TIMENS_C(3600000000000))/TIMENS_C(1000))
#define timens_get_nsec_of_hour(t)	(mod_ni((t),TIMENS_C(3600000000000)))

#define timens_get_sec_of_min(t)	(mod_ni((t),TIMENS_C(60000000000))/TIMENS_C(1000000000))
#define timens_get_msec_of_min(t)	(mod_ni((t),TIMENS_C(60000000000))/TIMENS_C(1000000))
#define timens_get_usec_of_min(t)	(mod_ni((t),TIMENS_C(60000000000))/TIMENS_C(1000))
#define timens_get_nsec_of_min(t)	(mod_ni((t),TIMENS_C(60000000000)))

#define timens_get_msec_of_sec(t)	(mod_ni((t),TIMENS_C(1000000000))/TIMENS_C(1000000))
#define timens_get_usec_of_sec(t)	(mod_ni((t),TIMENS_C(1000000000))/TIMENS_C(1000))
#define timens_get_nsec_of_sec(t)	(mod_ni((t),TIMENS_C(1000000000)))

#define timens_get_usec_of_msec(t)	(mod_ni((t),TIMENS_C(1000000))/TIMENS_C(1000))
#define timens_get_nsec_of_msec(t)	(mod_ni((t),TIMENS_C(1000000)))

#define timens_get_nsec_of_usec(t)	(mod_ni((t),TIMENS_C(1000)))

//-----------------------------------------------------------------------------
// Define macros to add/subtract intervals of time to/from type timens_t.
//-----------------------------------------------------------------------------
#define timens_add_days(t,d)		((t)+(long long)(d)*86400000000000LL)
#define timens_add_hours(t,h)		((t)+(long long)(h)*3600000000000LL)
#define timens_add_mins(t,m)		((t)+(long long)(m)*60000000000LL)
#define timens_add_secs(t,s)		((t)+(long long)(s)*1000000000LL)
#define timens_add_msecs(t,m)		((t)+(long long)(m)*1000000LL)
#define timens_add_usecs(t,u)		((t)+(long long)(u)*1000LL)
#define timens_add_nsecs(t,n)		((t)+(long long)(n))

#define timens_sub_days(t,d)		((t)-(long long)(d)*86400000000000LL)
#define timens_sub_hours(t,h)		((t)-(long long)(h)*3600000000000LL)
#define timens_sub_mins(t,m)		((t)-(long long)(m)*60000000000LL)
#define timens_sub_secs(t,s)		((t)-(long long)(s)*1000000000LL)
#define timens_sub_msecs(t,m)		((t)-(long long)(m)*1000000LL)
#define timens_sub_usecs(t,u)		((t)-(long long)(u)*1000LL)
#define timens_sub_nsecs(t,n)		((t)-(long long)(n))

//-----------------------------------------------------------------------------
// Define macros to get the difference between two timens_t values and return
// the result in a specific relative scale.
//-----------------------------------------------------------------------------
#define timens_diff_days(a,b)		(quo_ni((b),86400000000000LL)-quo_ni((a),86400000000000LL))
#define timens_diff_hours(a,b)		(quo_ni((b),3600000000000LL)-quo_ni((a),3600000000000LL))
#define timens_diff_mins(a,b)		(quo_ni((b),60000000000LL)-quo_ni((a),60000000000LL))
#define timens_diff_secs(a,b)		(quo_ni((b),1000000000LL)-quo_ni((a),1000000000LL))
#define timens_diff_msecs(a,b)		(quo_ni((b),1000000LL)-quo_ni((a),1000000LL))
#define timens_diff_usecs(a,b)		(quo_ni((b),1000LL)-quo_ni((a),1000LL)))
#define timens_diff_nsecs(a,b)		((b)-(a))



//-----------------------------------------------------------------------------
// Define macros to convert absolute time from one scale/epoch to another.
//-----------------------------------------------------------------------------
#define etime_to_eday(t)		((eday_t)quo_ni((t),ETIME_C(86400000000)))
#define etime_to_lday(t)		(eday_to_lday((etime_to_eday(t))))
#define etime_to_xday(t)		(eday_to_xday((etime_to_eday(t))))
#define etime_to_time(t)		((time_t)quo_ni(((etime_t)(t)-ETIME_UNIX_EPOCH),1000000U))
#define etime_to_timems(t)		((timems_t)quo_ni(((etime_t)(t)-ETIME_UNIX_EPOCH),1000U))
#define etime_to_timeus(t)		((timeus_t)((time_t)(((etime_t)(t)-ETIME_UNIX_EPOCH))))
#define etime_to_timens(t)		((timens_t)(((etime_t)(t)-ETIME_UNIX_EPOCH)*1000U))

#define eday_to_etime(t)		(((etime_t)(t))*ETIME_C(86400000000))
#define eday_to_lday(t)			((lday_t)(t)-EDAY_14_OCT_1582)
#define eday_to_xday(t)			((xday_t)(t)-2000000)
#define eday_to_time(t)			(((time_t)((eday_t)(t)-EDAY_UNIX_EPOCH))*86400)
#define eday_to_time64(t)		(((time64_t)((eday_t)(t)-EDAY_UNIX_EPOCH))*TIME64_C(86400))
#define eday_to_timems(t)		(((int64_t)((eday_t)(t)-EDAY_UNIX_EPOCH))*TIMEMC_C(86400000))
#define eday_to_timeus(t)		(((int64_t)((eday_t)(t)-EDAY_UNIX_EPOCH))*TIMEUS_C(86400000000))
#define eday_to_timens(t)		(((int64_t)((eday_t)(t)-EDAY_UNIX_EPOCH))*TIMENS_C(86400000000000))

#define lday_to_etime(t)		(eday_to_etime((lday_to_eday(t))))
#define lday_to_eday(t)			((eday_t)(t)+EDAY_14_OCT_1582)
#define lday_to_xday(t)			((xday_t)(t)+XDAY_14_OCT_1582)
#define lday_to_time(t)			(((time_t)((lday_t)(t)-LDAY_UNIX_EPOCH))*86400)
#define lday_to_time64(t)		(((time64_t)((lday_t)(t)-LDAY_UNIX_EPOCH))*TIME64_C(86400))
#define lday_to_timems(t)		(((int64_t)((lday_t)(t)-LDAY_UNIX_EPOCH))*TIMEMS_C(86400000))
#define lday_to_timeus(t)		(((int64_t)((lday_t)(t)-LDAY_UNIX_EPOCH))*TIMEUS_C(86400000000))
#define lday_to_timens(t)		(((int64_t)((lday_t)(t)-LDAY_UNIX_EPOCH))*TIMENS_C(86400000000000))

#define xday_to_etime(t)		(eday_to_etime((xday_to_eday(t))))
#define xday_to_eday(t)			((eday_t)(t)+2000000)
#define xday_to_lday(t)			((lday_t)(t)-XDAY_14_OCT_1582)
#define xday_to_time(t)			(((time_t)((xday_t)(t)-XDAY_UNIX_EPOCH))*86400)
#define xday_to_time64(t)		(((time64_t)((xday_t)(t)-XDAY_UNIX_EPOCH))*TIME64_C(86400))
#define xday_to_timems(t)		(((int64_t)((xday_t)(t)-XDAY_UNIX_EPOCH))*TIMEMS_C(86400000))
#define xday_to_timeus(t)		(((int64_t)((xday_t)(t)-XDAY_UNIX_EPOCH))*TIMEUS_C(86400000000))
#define xday_to_timens(t)		(((int64_t)((xday_t)(t)-XDAY_UNIX_EPOCH))*TIMENS_C(86400000000000))

#define time_to_etime(t)		((((etime_t)(t))*ETIME_C(1000000))+ETIME_UNIX_EPOCH)
#define time_to_eday(t)			((eday_t)(quo_ni((t),86400U)+EDAY_UNIX_EPOCH))
#define time_to_lday(t)			((lday_t)(quo_ni((t),86400U)+LDAY_UNIX_EPOCH))
#define time_to_xday(t)			((xday_t)(quo_ni((t),86400U)+XDAY_UNIX_EPOCH))
#define time_to_timems(t)		(((timems_t)(t))*TIMEMS_C(1000))
#define time_to_timeus(t)		(((timeus_t)(t))*TIMEUS_C(1000000))
#define time_to_timens(t)		(((timens_t)(t))*TIMENS_C(1000000000))

#define time64_to_etime(t)		((((etime_t)(t))*ETIME_C(1000000))+ETIME_UNIX_EPOCH)
#define time64_to_eday(t)		((eday_t)(quo_ni((t),TIME64_C(86400))+EDAY_UNIX_EPOCH))
#define time64_to_lday(t)		((lday_t)(quo_ni((t),TIME64_C(86400))+LDAY_UNIX_EPOCH))
#define time64_to_xday(t)		((xday_t)(quo_ni((t),TIME64_C(86400))+XDAY_UNIX_EPOCH))
#define time64_to_timems(t)		(((timeus_t)(t))*TIMEMS_C(1000))
#define time64_to_timeus(t)		(((timens_t)(t))*TIMEUS_C(1000000))
#define time64_to_timens(t)		(((timeus_t)(t))*TIMENS_C(1000000000))

#define timems_to_etime(t)		((((etime_t)(t))*ETIME_C(1000))+ETIME_UNIX_EPOCH)
#define timems_to_eday(t)		((eday_t)(quo_ni((t),TIMEMS_C(86400000))+EDAY_UNIX_EPOCH))
#define timems_to_lday(t)		((lday_t)(quo_ni((t),TIMEMS_C(86400000))+LDAY_UNIX_EPOCH))
#define timems_to_xday(t)		((xday_t)(quo_ni((t),TIMEMS_C(86400000))+XDAY_UNIX_EPOCH))
#define timems_to_time(t)		((time_t)quo_ni((t),TIMEMS_C(1000)))
#define timems_to_timeus(t)		(((timeus_t)(t))*TIMEUS_C(1000))
#define timems_to_timens(t)		(((timens_t)(t))*TIMENS_C(1000000))

#define timeus_to_etime(t)		(((etime_t)(t))+ETIME_UNIX_EPOCH)
#define timeus_to_eday(t)		((eday_t)(quo_ni((t),TIMEUS_C(86400000000))+EDAY_UNIX_EPOCH))
#define timeus_to_lday(t)		((lday_t)(quo_ni((t),TIMEUS_C(86400000000))+LDAY_UNIX_EPOCH))
#define timeus_to_xday(t)		((xday_t)(quo_ni((t),TIMEUS_C(86400000000))+XDAY_UNIX_EPOCH))
#define timeus_to_time(t)		((time_t)quo_ni((t),TIMEUS_C(1000000)))
#define timeus_to_timems(t)		((timems_t)quo_ni((t),TIMEUS_C(1000)))
#define timeus_to_timens(t)		(((timens_t)(t))*TIMENS_C(1000))

#define timens_to_etime(t)		(((etime_t)quo_ni((t),ETIME_C(1000)))+ETIME_UNIX_EPOCH)
#define timens_to_eday(t)		((eday_t)(quo_ni((t),TIMENS_C(86400000000000))+EDAY_UNIX_EPOCH))
#define timens_to_lday(t)		((lday_t)(quo_ni((t),TIMENS_C(86400000000000))+LDAY_UNIX_EPOCH))
#define timens_to_xday(t)		((xday_t)(quo_ni((t),TIMENS_C(86400000000000))+XDAY_UNIX_EPOCH))
#define timens_to_time(t)		((time_t)quo_ni((t),TIMENS_C(1000000000)))
#define timens_to_timems(t)		((timems_t)quo_ni((t),TIMENS_C(1000000)))
#define timens_to_timeus(t)		((timeus_t)quo_ni((t),TIMENS_C(1000)))

#define etime_to_dsec(t)		mod_ni(quo_ni((t),ETIME_C(1000000)),ETIME_C(86400))
#define etime_to_dmsec(t)		mod_ni(quo_ni((t),ETIME_C(1000)),TIMEMS_C(86400000))
#define etime_to_dusec(t)		mod_ni((t),TIMEUS_C(86400000000))
#define etime_to_dnsec(t)		mod_ni((t)*ETIME_C(1000),TIMENS_C(86400000000000))

#define ecal_to_time(c)			(etime_to_time(ecal_to_etime((c))))
#define ymd_gregorian_to_time(y,m,d)		(etime_to_time(ymd_gregorian_to_etime((y),(m),(d))))
#define ymd_julian_to_time(y,m,d)		(etime_to_time(ymd_julian_to_etime((y),(m),(d))))


//-----------------------------------------------------------------------------
// other macros
//-----------------------------------------------------------------------------
#define time_to_str_app(d,s,f,t)	etime_to_str_app((d),(s),(f),time_to_etime((t)))
#define time_to_str_cpy(d,s,f,t)	etime_to_str_app((d),(s),(f),time_to_etime((t)))
#define time_to_str_dup(f,t)		etime_to_str_app((f),time_to_etime((t)))

#define time_to_eprintf(f,t)		etime_to_eprintf((f),time_to_etime((t)))
#define time_to_fprintf(o,f,t)		etime_to_fprintf((o),(f),time_to_etime((t)))
#define time_to_printf(f,t)		etime_to_printf((f),time_to_etime((t)))

#define timems_to_eprintf(f,t)		etime_to_eprintf((f),timems_to_etime((t)))
#define timems_to_fprintf(o,f,t)	etime_to_fprintf((o),(f),timems_to_etime((t)))
#define timems_to_printf(f,t)		etime_to_printf((f),timems_to_etime((t)))

#define timeus_to_eprintf(f,t)		etime_to_eprintf((f),timeus_to_etime((t)))
#define timeus_to_fprintf(o,f,t)	etime_to_fprintf((o),(f),timeus_to_etime((t)))
#define timeus_to_printf(f,t)		etime_to_printf((f),timeus_to_etime((t)))

#define timens_to_eprintf(f,t)		etime_to_eprintf((f),timens_to_etime((t)))
#define timens_to_fprintf(o,f,t)	etime_to_fprintf((o),(f),timens_to_etime((t)))
#define timens_to_printf(f,t)		etime_to_printf((f),timens_to_etime((t)))

//-----------------------------------------------------------------------------
// macro	current_eday
//
// purpose	Get the current day in eday form.
//
// arguments	-none-
//
// returns	(eday) current eday
//-----------------------------------------------------------------------------
#define current_eday() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? (eday) 0x80000000						\
		: time_to_eday( libh__timeval.tv_sec );				\
	})

//-----------------------------------------------------------------------------
// macro	current_etime
//
// purpose	Get the current time in etime form.
//
// arguments	-none-
//
// returns	(etime) current etime
//-----------------------------------------------------------------------------
#define current_etime() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: ({								\
			long long libh__time;					\
			libh__time = libh__timeval.tv_sec;			\
			libh__time *= 1000000LL;				\
			libh__time += libh__timeval.tv_usec;			\
			timeus_to_etime( libh__time );				\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	current_lday
//
// purpose	Get the current day in lday form.
//
// arguments	-none-
//
// returns	(lday) current lday
//-----------------------------------------------------------------------------
#define current_lday() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? (lday) 0x80000000						\
		: time_to_lday( libh__timeval.tv_sec );				\
	})

//-----------------------------------------------------------------------------
// macro	current_strftime_gmt
//
// purpose	Format the current gmt time via strftime formatting.
//
// arguments	1 (char *) where to put the result formatted string
//		2 (size_t) size of space to put the result string
//		3 (const char *) format string
//
// returns	(size_t) number of characters placed into result string
//
// note		This macro uses the reentrant and thread safe gmtime_r().
//-----------------------------------------------------------------------------
#define current_strftime_gmt(m,s,f) ({						\
	struct tm	libh__tm;						\
	time_t		libh__time;						\
	libh__time = time( NULL );						\
	gmtime_r( (time_t *) & libh__time, & libh__tm );			\
	strftime( (m), (s), (f), & libh__tm );					\
})

//-----------------------------------------------------------------------------
// macro	current_strftime_local
//
// purpose	Format the current local time via strftime formatting.
//
// arguments	1 (char *) where to put the result formatted string
//		2 (size_t) size of space to put the result string
//		3 (const char *) format string
//
// returns	(size_t) number of characters placed into result string
//
// note		This macro uses the reentrant and thread safe localtime_r().
//-----------------------------------------------------------------------------
#define current_strftime_local(m,s,f) ({					\
	struct tm	libh__tm;						\
	time_t		libh__time;						\
	libh__time = time( NULL );						\
	localtime_r( (time_t *) & libh__time, & libh__tm );			\
	strftime( (m), (s), (f), & libh__tm );					\
})

//-----------------------------------------------------------------------------
// macro	current_strftime_usec_gmt
//
// purpose	Format the current gmt time via strftime formatting.
//
// arguments	1 (char *) where to put the result formatted string
//		2 (size_t) size of space to put the result string
//		3 (const char *) format string
//		4 (const char *) format for microseconds
//
// returns	(size_t) number of characters placed into result string
//
// note		This macro uses the reentrant and thread safe gmtime_r().
//-----------------------------------------------------------------------------
#define current_strftime_usec_gmt(m,s,f,u) ({					\
	struct timeval	libh__tv;						\
	struct tm	libh__tm;						\
	char *		libh__ptr;						\
	size_t		libh__size;						\
	size_t		libh__len;						\
	libh__ptr = (m);							\
	libh__size = (s);							\
	gettimeofday( & libh__tv, NULL );					\
	gmtime_r( (time_t *) & libh__tv.tv_sec, & libh__tm );			\
	libh__len = strftime( libh__ptr, libh__size, (f), & libh__tm );		\
	if ( libh__len > 0 ) {							\
		libh__ptr += libh__len;							\
		libh__size -= libh__len;						\
		libh__len += snprintf( libh__ptr, libh__size, (u), libh__tv.tv_usec );	\
	}									\
	libh__len;								\
})

//-----------------------------------------------------------------------------
// macro	current_strftime_usec_local
//
// purpose	Format the current local time via strftime formatting.
//
// arguments	1 (char *) where to put the result formatted string
//		2 (size_t) size of space to put the result string
//		3 (const char *) format string
//		4 (const char *) format for microseconds
//
// returns	(size_t) number of characters placed into result string
//
// note		This macro uses the reentrant and thread safe localtime_r().
//-----------------------------------------------------------------------------
#define current_strftime_usec_local(m,s,f,u) ({					\
	struct timeval	libh__tv;						\
	struct tm	libh__tm;						\
	char *		libh__ptr;						\
	size_t		libh__size;						\
	size_t		libh__len;						\
	libh__ptr = (m);							\
	libh__size = (s);							\
	gettimeofday( & libh__tv, NULL );					\
	localtime_r( (time_t *) & libh__tv.tv_sec, & libh__tm );		\
	libh__len = strftime( libh__ptr, libh__size, (f), & libh__tm );		\
	if ( libh__len > 0 ) {							\
		libh__ptr += libh__len;							\
		libh__size -= libh__len;						\
		libh__len += snprintf( libh__ptr, libh__size, (u), libh__tv.tv_usec );	\
	}									\
	libh__len;								\
})

//-----------------------------------------------------------------------------
// macro	current_time
//
// purpose	Get the current time in seconds as a 32 bit integer.
//
// arguments	-none-
//
// returns	(time_t) current time in seconds
//-----------------------------------------------------------------------------
#define current_time() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: libh__timeval.tv_sec;						\
	})

//-----------------------------------------------------------------------------
// macro	current_timems
//
// purpose	Get the current time in microseconds as a 64 bit integer.
//
// arguments	-none-
//
// returns	(timems_t) current time in microseconds
//-----------------------------------------------------------------------------
#define current_timems() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: ({								\
			long long libh__time;					\
			libh__time = libh__timeval.tv_sec;			\
			libh__time *= 1000000LL;				\
			libh__time += libh__timeval.tv_usec;			\
			libh__time /= 1000LL;					\
			libh__time;						\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	current_timens
//
// purpose	Get the current time in nanoseconds as a 64 bit integer.
//
// arguments	-none-
//
// returns	(timens_t) current time in nanoseconds
//-----------------------------------------------------------------------------
#define current_timens() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: ({								\
			long long libh__time;					\
			libh__time = libh__timeval.tv_sec;			\
			libh__time *= 1000000LL;				\
			libh__time += libh__timeval.tv_usec;			\
			libh__time * 1000LL;					\
			libh__time;						\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	current_timespec
//
// purpose	Get current time in struct timespec.
//
// argument	-none-
//
// returns	(struct timespec)
//-----------------------------------------------------------------------------
#define current_timespec() ({							\
	struct timespec libh__timespec;						\
	gettimeofday((structtimeval*)&libh__timespec,(structtimezone*)NULL);	\
	libh__timespec->tv_nsec *= 1000L;					\
	libh__timespec;								\
})

//-----------------------------------------------------------------------------
// macro	current_timeus
//
// purpose	Get the current time in microseconds as a 64 bit integer.
//
// arguments	-none-
//
// returns	(timeus_t) current time in microseconds
//-----------------------------------------------------------------------------
#define current_timeus() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? -1LL								\
		: ({								\
			long long libh__time;					\
			libh__time = libh__timeval.tv_sec;			\
			libh__time *= 1000000LL;				\
			libh__time += libh__timeval.tv_usec;			\
			libh__time;						\
		});								\
	})

//-----------------------------------------------------------------------------
// macro	current_timeval
//
// purpose	Get current time in struct timeval.
//
// argument	-none-
//
// returns	(struct timeval)
//-----------------------------------------------------------------------------
#define current_timeval() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, (struct timezone *) ( NULL ) );		\
	libh__timeval;								\
})

//-----------------------------------------------------------------------------
// macro	current_xday
//
// purpose	Get the current day in xday form.
//
// arguments	-none-
//
// returns	(xday) current xday
//-----------------------------------------------------------------------------
#define current_xday() ({							\
	struct timeval libh__timeval;						\
	gettimeofday( & libh__timeval, NULL ) < 0				\
		? (xday) 0x80000000						\
		: time_to_xday( libh__timeval.tv_sec );				\
	})

//-----------------------------------------------------------------------------
// macro	dhms_to_msec
//
// purpose	Convert days, hours, minutes, seconds to just milliseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define dhms_to_msec(d,h,m,s) (dhms_to_sec((d),(h),(m),(s))*TIME64_C(1000))

//-----------------------------------------------------------------------------
// macro	dhms_to_nsec
//
// purpose	Convert days, hours, minutes, seconds to just nanoseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define dhms_to_nsec(d,h,m,s) (dhms_to_sec((d),(h),(m),(s))*TIME64_C(1000000000))

//-----------------------------------------------------------------------------
// macro	dhms_to_sec
//
// purpose	Convert days, hours, minutes, seconds to just seconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//
// returns	(time64_t) seconds
//-----------------------------------------------------------------------------
#define dhms_to_sec(d,h,m,s) ((time64_t)(d)*TIME64_C(24)+hms_to_sec((h),(m),(s)))

//-----------------------------------------------------------------------------
// macro	dhms_to_usec
//
// purpose	Convert days, hours, minutes, seconds to just microseconds
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define dhms_to_usec(d,h,m,s) (dhms_to_sec((d),(h),(m),(s))*TIME64_C(1000000))

//-----------------------------------------------------------------------------
// macro	dhmsm_to_msec
//
// purpose	Convert days, hours, minutes, seconds, milliseconds to just
//		milliseconds
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) milliseconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define dhmsm_to_msec(d,h,m,s,ms) (dhms_to_msec((d),(h),(m),(s))+(ms))

//-----------------------------------------------------------------------------
// macro	dhmsm_to_nsec
//
// purpose	Convert days, hours, minutes, seconds, milliseconds to just
//		nanoseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) milliseconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define dhmsm_to_nsec(d,h,m,s,ns) (dhms_to_nsec((d),(h),(m),(s))+(time64_t)((ms)*1000000))

//-----------------------------------------------------------------------------
// macro	dhmsm_to_usec
//
// purpose	Convert days, hours, minutes, seconds, milliseconds to just
//		microseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) milliseconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define dhmsm_to_usec(d,h,m,s,us) (dhms_to_usec((d),(h),(m),(s))+(time64_t)((ms)*1000))

//-----------------------------------------------------------------------------
// macro	dhmsn_to_msec
//
// purpose	Convert days, hours, minutes, seconds, nanoseconds to just
//		milliseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) nanoseconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define dhmsn_to_msec(d,h,m,s,ms) (dhms_to_msec((d),(h),(m),(s))+(time64_t)((ns)/1000000))

//-----------------------------------------------------------------------------
// macro	dhmsn_to_nsec
//
// purpose	Convert days, hours, minutes, seconds, nanoseconds to just
//		nanoseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) nanoseconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define dhmsn_to_nsec(d,h,m,s,ns) (dhms_to_nsec((d),(h),(m),(s))+(time64_t)(ns))

//-----------------------------------------------------------------------------
// macro	dhmsn_to_usec
//
// purpose	Convert days, hours, minutes, seconds, nanoseconds to just
//		microseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) nanoseconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define dhmsn_to_usec(d,h,m,s,us) (dhms_to_usec((d),(h),(m),(s))+(time64_t)((ns)/1000))

//-----------------------------------------------------------------------------
// macro	dhmsu_to_msec
//
// purpose	Convert days, hours, minutes, seconds, microseconds to just
//		milliseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) microseconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define dhmsu_to_msec(d,h,m,s,ms) (dhms_to_msec((d),(h),(m),(s))+(time64_t)((us)/1000))

//-----------------------------------------------------------------------------
// macro	dhmsu_to_nsec
//
// purpose	Convert days, hours, minutes, seconds, microseconds to just
//		nanoseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) microseconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define dhmsu_to_nsec(d,h,m,s,ns) (dhms_to_nsec((d),(h),(m),(s))+(time64_t)((us)*1000))

//-----------------------------------------------------------------------------
// macro	dhmsu_to_usec
//
// purpose	Convert days, hours, minutes, seconds, microseconds, to just
//		microseconds.
//
// arguments	1 (time64_t) days
//		2 (time64_t) hours
//		3 (time64_t) minutes
//		4 (time64_t) seconds
//		5 (time64_t) microseconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define dhmsu_to_usec(d,h,m,s,us) (dhms_to_usec((d),(h),(m),(s))+(us))

//-----------------------------------------------------------------------------
// macro	ecal_is_leap_year
//
// purpose	Yield true if the ecal struc specifies a leap year.
//
// arguments	1 (ecal_t) ecal value to check for leap year
//
// returns	(int) true or false
//-----------------------------------------------------------------------------
#define ecal_is_leap_year(c) ((c).isleap==(ECAL_ISLEAP_YES))

//-----------------------------------------------------------------------------
// macro	ecal_is_not_leap_year
//
// purpose	Yield true if the ecal struc does not specify a leap year.
//
// arguments	1 (ecal_t) ecal value to check for leap year
//
// returns	(int) true or false
//-----------------------------------------------------------------------------
#define ecal_is_not_leap_year(c) ((c).isleap==(ECAL_ISLEAP_NO))

//-----------------------------------------------------------------------------
// macro	ecal_to_time
//
// purpose	Convert ecal to system time.
//
// arguments	1 (ecal_t) ecal value to convert
//
// returns	(time_t) system time
//-----------------------------------------------------------------------------
#define ecal_to_time(c) (etime_to_time(ecal_to_etime((c))))

//-----------------------------------------------------------------------------
// macro	eday_get_day_of_month_gregorian
// alias	eday_get_day_of_month
//
// purpose	Extract day of month in the Gregorian calendar from eday_t.
//
// argument	1 (eday_t) eday_t value
//
// returns	(int) Gregorian calendar day of month (1 .. 31)
//-----------------------------------------------------------------------------
#define eday_get_day_of_month_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( ( e ), & libh__ecal );				\
	libh__ecal.mday;							\
})

#define eday_get_day_of_month(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( ( e ), & libh__ecal );					\
	libh__ecal.mday;							\
})

//-----------------------------------------------------------------------------
// macro	eday_get_day_of_month_julian
//
// purpose	Extract day of month in the Julian calendar from eday_t.
//
// argument	1 (eday_t) eday_t value
//
// returns	(int) Julian calendar day of month (1 .. 31)
//-----------------------------------------------------------------------------
#define eday_get_day_of_month_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( ( e ), & libh__ecal );					\
	libh__ecal.mday;							\
})

//-----------------------------------------------------------------------------
// macro	eday_get_day_of_week_gregorian
// alias	eday_get_day_of_week
//
// purpose	Extract day of week in the Gregorian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Gregorian calendar day of week (1 .. 7)
//
// note		The result is always the same for both Gregorian and Julian.
//-----------------------------------------------------------------------------
#define eday_get_day_of_week_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( ( e ), & libh__ecal );				\
	libh__ecal.wday;							\
})

#define eday_get_day_of_week(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( ( e ), & libh__ecal );					\
	libh__ecal.wday;							\
})

//-----------------------------------------------------------------------------
// function	eday_get_day_of_week_julian
//
// purpose	Extract day of week in the Julian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Julian calendar day of week (1 .. 7)
//
// note		The result is always the same for both Gregorian and Julian.
//-----------------------------------------------------------------------------
#define eday_get_day_of_week_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( ( e ), & libh__ecal );					\
	libh__ecal.wday;							\
})

//-----------------------------------------------------------------------------
// macro	eday_get_day_of_year_gregorian
// alias	eday_get_day_of_year
//
// purpose	Extract day of year in the Gregorian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Gregorian calendar day of year (1 .. 366)
//-----------------------------------------------------------------------------
#define eday_get_day_of_year_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( ( e ), & libh__ecal );				\
	libh__ecal.yday;							\
})

#define eday_get_day_of_year(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( ( e ), & libh__ecal );					\
	libh__ecal.yday;							\
})

//-----------------------------------------------------------------------------
// function	eday_get_day_of_year_julian
//
// purpose	Extract day of year in the Julian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Julian calendar day of year (1 .. 366)
//-----------------------------------------------------------------------------
#define eday_get_day_of_year_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( ( e ), & libh__ecal );					\
	libh__ecal.yday;							\
})

//-----------------------------------------------------------------------------
// macro	eday_get_month_of_year_gregorian
// alias	eday_get_month_of_year
//
// purpose	Extract month of year in the Gregorian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Gregorian calendar month (1 .. 12)
//-----------------------------------------------------------------------------
#define eday_get_month_of_year_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( ( e ), & libh__ecal );				\
	libh__ecal.month;							\
})

#define eday_get_month_of_year(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( ( e ), & libh__ecal );					\
	libh__ecal.month;							\
})

//-----------------------------------------------------------------------------
// function	eday_get_month_of_year_julian
//
// purpose	Extract month of year in the Julian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Julian calendar month (1 .. 12)
//-----------------------------------------------------------------------------
#define eday_get_month_of_year_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( ( e ), & libh__ecal );					\
	libh__ecal.month;							\
})

//-----------------------------------------------------------------------------
// macro	eday_get_year_gregorian
// alias	eday_get_year
//
// purpose	Extract year in the Gregorian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(long) Gregorian calendar year
//-----------------------------------------------------------------------------
#define eday_get_year_gregorian(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( ( e ), & libh__ecal );				\
	libh__ecal.year;							\
})

#define eday_get_year(e) ({							\
	ecal_t libh__ecal;							\
	eday_to_ecal( ( e ), & libh__ecal );					\
	libh__ecal.year;							\
})

//-----------------------------------------------------------------------------
// function	eday_get_year_julian
//
// purpose	Extract year in the Julian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(long) Julian calendar year
//-----------------------------------------------------------------------------
#define eday_get_year_julian(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( ( e ), & libh__ecal );					\
	libh__ecal.year;							\
})

//-----------------------------------------------------------------------------
// macro	eday_to_eprintf
//
// purpose	Convert an Earth Day value to a formatted string and output.
//
// arguments	1 (const char *) format string
//		2 (eday_t) Earth Day value
//
// returns	(int) return value from fputs()
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although time format specifiers are valid when formatting
//		an Earth Day value, they are always converted from 0 which
//		represents the beginning of the given day.
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
#define eday_to_eprintf(f,d) eday_to_fprintf(stderr,(f),(d))

//-----------------------------------------------------------------------------
// macro	eday_to_jd
//
// purpose	Convert an Earth Day value to a Julian Date as used in
//		astronomy.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(JD) Julian Date
//-----------------------------------------------------------------------------
#define eday_to_jd(d) (((JD)(d))-(JD)0.5)

//-----------------------------------------------------------------------------
// macro	eday_to_printf
//
// purpose	Convert an Earth Day value to a formatted string and output.
//
// arguments	1 (const char *) format string
//		2 (eday_t) Earth Day value
//
// returns	(int) return value from fputs()
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although time format specifiers are valid when formatting
//		an Earth Day value, they are always converted from 0 which
//		represents the beginning of the given day.
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
#define eday_to_printf(f,d) eday_to_fprintf(stdout,(f),(d))

//-----------------------------------------------------------------------------
// macro	eday_to_str_cpy
//
// purpose	Copy to the first string (given with maximum available space)
//		an Earth Day value in the format specified.
//
// arguments	1 (char *) target string to copy to
//		2 (size_t) size limit for target string
//		3 (const char *) format string
//		4 (eday_t) Earth Day value
//
// returns	(size_t) new length of target string or ~0 if copy fails
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although time format specifiers are valid when formatting
//		an Earth Day value, they are always converted from 0 which
//		represents the beginning of the given day.
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
#define eday_to_str_cpy(t,l,f,e) ((*(t)=0),eday_to_str_app((t),(l),(f),(e)))

//-----------------------------------------------------------------------------
// macro	eday_to_tm
//
// purpose	Convert an Earth Day value to Gregorian calendar elements in
//		a Standard C struct tm.
//
// arguments	1 (eday_t) Earth Day value
//		2 (struct tm *) pointer to struct tm to store into
//
// returns	(void)
//
// warning	Since Standard C defines struct tm generally in terms of it
//		being converted from, or to, type time_t, programs might be
//		expecting the values in struct tm to be restricted to the same
//		range of dates.  Implementations of conversion from struct tm
//		to strings (e.g. strftime()) may be limited in that way due to
//		use of type time_t to do necessary calculations, and lack of a
//		standard requirement for a greater range.
//
//		The ecal_to_tm() function will apply the full scale of years
//		to struct tm, which may not be handled properly by programs
//		and functions not supporting that range.  The range of years
//		supported by Earth Day is over 11 million years.
//-----------------------------------------------------------------------------
#define eday_to_tm(e,t) ({							\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( (e), & libh__ecal );				\
	ecal_to_tm( & libh__ecal, (t) );					\
})

//-----------------------------------------------------------------------------
// macro	eday_tod_to_jd
//
// purpose	Convert an Earth Day value to a Julian Date as used in
//		astronomy.
//
// argument	1 (eday_t) Earth Day value
//		2 (etime_t) time of day to include
//
// returns	(JD) Julian Date
//-----------------------------------------------------------------------------
#define eday_tod_to_jd(d,t) (((((JD)(t))-(JD)43200000000.0)/(JD)86400000000.0)+((JD)(d)))

//-----------------------------------------------------------------------------
// macro	etime_get_day_of_month_gregorian
// alias	etime_get_day_of_month
//
// purpose	Extract day of month in the Gregorian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Gregorian calendar day of month (1 .. 31)
//-----------------------------------------------------------------------------
#define etime_get_day_of_month_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( etime_to_eday( ( e ) ), & libh__ecal );		\
	libh__ecal.mday;							\
})

#define etime_get_day_of_month(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.mday;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_day_of_month_julian
//
// purpose	Extract day of month in the Julian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Julian calendar day of month (1 .. 31)
//-----------------------------------------------------------------------------
#define etime_get_day_of_month_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.mday;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_day_of_week_gregorian
// alias	etime_get_day_of_week
//
// purpose	Extract day of week in the Gregorian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Gregorian calendar day of week (1 .. 7)
//
// note		The result is always the same for both Gregorian and Julian.
//-----------------------------------------------------------------------------
#define etime_get_day_of_week_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( etime_to_eday( ( e ) ), & libh__ecal );		\
	libh__ecal.wday;							\
})

#define etime_get_day_of_week(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.wday;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_day_of_week_julian
//
// purpose	Extract day of month in the Julian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Julian calendar day of week (1 .. 7)
//
// note		The result is always the same for both Gregorian and Julian.
//-----------------------------------------------------------------------------
#define etime_get_day_of_week_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.wday;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_day_of_year_gregorian
// alias	etime_get_day_of_year
//
// purpose	Extract day of year in the Gregorian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Gregorian calendar day of year (1 .. 366)
//-----------------------------------------------------------------------------
#define etime_get_day_of_year_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( etime_to_eday( ( e ) ), & libh__ecal );		\
	libh__ecal.yday;							\
})

#define etime_get_day_of_year(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.yday;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_day_of_year_julian
//
// purpose	Extract day of year in the Julian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Julian calendar day of year (1 .. 366)
//-----------------------------------------------------------------------------
#define etime_get_day_of_year_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.yday;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_month_of_year_gregorian
// alias	etime_get_month_of_year
//
// purpose	Extract month of year in the Gregorian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Gregorian calendar month (1 .. 12)
//-----------------------------------------------------------------------------
#define etime_get_month_of_year_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( etime_to_eday( ( e ) ), & libh__ecal );		\
	libh__ecal.month;							\
})

#define etime_get_month_of_year(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.month;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_month_of_year_julian
//
// purpose	Extract month of year in the Julian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(int) Julian calendar month (1 .. 12)
//-----------------------------------------------------------------------------
#define etime_get_month_of_year_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.month;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_year_gregorian
// alias	etime_get_year
//
// purpose	Extract year in the Gregorian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(long) Gregorian calendar year
//-----------------------------------------------------------------------------
#define etime_get_year_gregorian(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( etime_to_eday( ( e ) ), & libh__ecal );		\
	libh__ecal.year;							\
})

#define etime_get_year(e) ({							\
	ecal_t libh__ecal;							\
	eday_to_ecal( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.year;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_year_julian
//
// purpose	Extract year in the Julian calendar from etime_t.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(long) Julian calendar year
//-----------------------------------------------------------------------------
#define etime_get_year_julian(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( etime_to_eday( ( e ) ), & libh__ecal );			\
	libh__ecal.year;							\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymd_gregorian
// alias	etime_get_ymd
//
// purpose	Extract calendar elements in the Gregorian calendar from etime_t.
//
// arguments	1 (etime_t) Earth Time value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymd_gregorian(t,y,m,d) (eday_get_ymd_gregorian(etime_to_eday((t)),(y),(m),(d)))
#define etime_get_ymd(t,y,m,d) (eday_get_ymd(etime_to_eday((t)),(y),(m),(d)))

//-----------------------------------------------------------------------------
// macro	etime_get_ymd_julian
//
// purpose	Extract calendar elements in the Julian calendar from etime_t.
//
// arguments	1 (etime_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymd_julian(t,y,m,d) (eday_get_ymd_julian(etime_to_eday((t)),(y),(m),(d)))

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhms_gregorian
// alias	etime_get_ymdhms
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhms_gregorian(t,yr,mo,da,hr,mn,se) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_gregorian(libh__etime,(yr),(mo),(da));			\
	etime_get_hms(libh__etime,(hr),(mn),(se));				\
})

#define etime_get_ymdhms(t,yr,mo,da,hr,mn,se) ({				\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd(libh__etime,(yr),(mo),(da));				\
	etime_get_hms(libh__etime,(hr),(mn),(se));				\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhms_julian
//
// purpose	Extract calendar and time of day elements in the Julian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhms_julian(t,yr,mo,da,hr,mn,se) ({				\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd(libh__etime,(yr),(mo),(da));				\
	etime_get_hms(libh__etime,(hr),(mn),(se));				\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsm_gregorian
// alias	etime_get_ymdhmsm
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store milliseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsm_gregorian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_gregorian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsm(libh__etime,(hr),(mn),(se),(ms));			\
})

#define etime_get_ymdhmsm(t,yr,mo,da,hr,mn,se,ms) ({				\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd(libh__etime,(yr),(mo),(da));				\
	etime_get_hmsm(libh__etime,(hr),(mn),(se),(ms));			\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsm_julian
//
// purpose	Extract calendar and time of day elements in the Julian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store milliseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsm_julian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_julian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsm(libh__etime,(hr),(mn),(se),(ms));			\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsn_gregorian
// alias	etime_get_ymdhmsn
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store nanoseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsn_gregorian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_gregorian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsn(libh__etime,(hr),(mn),(se),(ms));			\
})

#define etime_get_ymdhmsn(t,yr,mo,da,hr,mn,se,ms) ({				\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd(libh__etime,(yr),(mo),(da));				\
	etime_get_hmsn(libh__etime,(hr),(mn),(se),(ms));			\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsn_julian
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store nanoseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsn_julian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_julian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsn(libh__etime,(hr),(mn),(se),(ms));			\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsu_gregorian
// alias	etime_get_ymdhmsu
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store macroseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsu_gregorian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_gregorian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsu(libh__etime,(hr),(mn),(se),(ms));			\
})

#define etime_get_ymdhmsu(t,yr,mo,da,hr,mn,se,ms) ({				\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd(libh__etime,(yr),(mo),(da));				\
	etime_get_hmsu(libh__etime,(hr),(mn),(se),(ms));			\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsu_julian
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store macroseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsu_julian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_julian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsu(libh__etime,(hr),(mn),(se),(ms));			\
})

//-----------------------------------------------------------------------------
// macro	etime_to_eprintf
//
// purpose	Convert an Earth Time value to a formatted string and output.
//
// arguments	1 (const char *) format string
//		2 (etime_t) Earth Time value
//
// returns	(int) return value from fputs()
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%%	Literal "%" character
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
#define etime_to_eprintf(f,t) etime_to_fprintf(stderr,(f),(t))

//-----------------------------------------------------------------------------
// macro	etime_to_jd
//
// purpose	Convert an Earth Time value to a Julian Date as used in
//		astronomy.
//
// argument	1 (etime_t) Earth Time value
//
// returns	(JD) Julian Date
//-----------------------------------------------------------------------------
#define etime_to_jd(t) ((((JD)(t))-(JD)43200000000.0)/(JD)86400000000.0)

//-----------------------------------------------------------------------------
// macro	etime_to_printf
//
// purpose	Convert an Earth Time value to a formatted string and output.
//
// arguments	1 (const char *) format string
//		2 (etime_t) Earth Time value
//
// returns	(int) return value from fputs()
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%%	Literal "%" character
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
#define etime_to_printf(f,t) etime_to_fprintf(stdout,(f),(t))

//-----------------------------------------------------------------------------
// macro	etime_to_str_cpy
//
// purpose	Copy to the first string (given with maximum available space)
//		an Earth Time value in the format specified.
//
// arguments	1 (char *) target string to copy to
//		2 (size_t) size limit for target string
//		3 (const char *) format string
//		4 (etime_t) Earth Time value
//
// returns	(size_t) new length of target string or ~0 if copy fails
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
#define etime_to_str_cpy(t,l,f,e) ((*(t)=0),etime_to_str_app((t),(l),(f),(e)))

//-----------------------------------------------------------------------------
// macro	etime_to_tm
//
// purpose	Convert an Earth Time value to Gregorian calendar elements in
//		a Standard C struct tm.
//
// arguments	1 (etime_t) Earth Time value
//		2 (struct tm *) pointer to struct tm to store into
//
// returns	(void)
//
// warning	Since Standard C defines struct tm generally in terms of it
//		being converted from, or to, type time_t, programs might be
//		expecting the values in struct tm to be restricted to the same
//		range of dates.  Implementations of conversion from struct tm
//		to strings (e.g. strftime()) may be limited in that way due to
//		use of type time_t to do necessary calculations, and lack of a
//		standard requirement for a greater range.
//
//		The ecal_to_tm() function will apply the full scale of years
//		to struct tm, which may not be handled properly by programs
//		and functions not supporting that range.  The range of years
//		supported by Earth Day is over 11 million years.
//-----------------------------------------------------------------------------
#define etime_to_tm(e,t) ({							\
	ecal_t libh__ecal;							\
	etime_to_ecal(e), & libh__ecal );					\
	ecal_to_tm( & libh__ecal, (t) );					\
})

//-----------------------------------------------------------------------------
// macro	hms_to_msec
//
// purpose	Convert hours, minutes, seconds to just milliseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define hms_to_msec(h,m,s) (hms_to_sec((h),(m),(s))*TIME64_C(1000))

//-----------------------------------------------------------------------------
// macro	hms_to_nsec
//
// purpose	Convert hours, minutes, seconds to just nanoseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define hms_to_nsec(h,m,s) (hms_to_sec((h),(m),(s))*TIME64_C(1000000000))

//-----------------------------------------------------------------------------
// macro	hms_to_sec
//
// purpose	Convert hours, minutes, seconds to just seconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//
// returns	(time64_t) seconds
//-----------------------------------------------------------------------------
#define hms_to_sec(h,m,s) ((h)*TIME64_C(3600)+(m)*TIME64_C(60)+(s))

//-----------------------------------------------------------------------------
// macro	hms_to_usec
//
// purpose	Convert hours, minutes, seconds to just microseconds
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define hms_to_usec(h,m,s) (hms_to_sec((h),(m),(s))*TIME64_C(1000000))

//-----------------------------------------------------------------------------
// macro	hmsm_to_msec
//
// purpose	Convert hours, minutes, seconds, milliseconds to just
//		milliseconds
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) milliseconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define hmsm_to_msec(h,m,s,ms) (hms_to_msec((h),(m),(s))+(ms))

//-----------------------------------------------------------------------------
// macro	hmsm_to_nsec
//
// purpose	Convert hours, minutes, seconds, milliseconds to just
//		nanoseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) milliseconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define hmsm_to_nsec(h,m,s,ns) (hms_to_nsec((h),(m),(s))+(time64_t)((ms)*1000000))

//-----------------------------------------------------------------------------
// macro	hmsm_to_usec
//
// purpose	Convert hours, minutes, seconds, milliseconds to just
//		microseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) milliseconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define hmsm_to_usec(h,m,s,us) (hms_to_usec((h),(m),(s))+(time64_t)((ms)*1000))

//-----------------------------------------------------------------------------
// macro	hmsn_to_msec
//
// purpose	Convert hours, minutes, seconds, nanoseconds to just
//		milliseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) nanoseconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define hmsn_to_msec(h,m,s,ms) (hms_to_msec((h),(m),(s))+(time64_t)((ns)/1000000))

//-----------------------------------------------------------------------------
// macro	hmsn_to_nsec
//
// purpose	Convert hours, minutes, seconds, nanoseconds to just
//		nanoseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) nanoseconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define hmsn_to_nsec(h,m,s,ns) (hms_to_nsec((h),(m),(s))+(time64_t)(ns))

//-----------------------------------------------------------------------------
// macro	hmsn_to_usec
//
// purpose	Convert hours, minutes, seconds, nanoseconds to just
//		microseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) nanoseconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define hmsn_to_usec(h,m,s,us) (hms_to_usec((h),(m),(s))+(time64_t)((ns)/1000))

//-----------------------------------------------------------------------------
// macro	hmsu_to_msec
//
// purpose	Convert hours, minutes, seconds, microseconds to just
//		milliseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) microseconds
//
// returns	(time64_t) milliseconds
//-----------------------------------------------------------------------------
#define hmsu_to_msec(h,m,s,ms) (hms_to_msec((h),(m),(s))+(time64_t)((us)/1000))

//-----------------------------------------------------------------------------
// macro	hmsu_to_nsec
//
// purpose	Convert hours, minutes, seconds, microseconds to just
//		nanoseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) microseconds
//
// returns	(time64_t) nanoseconds
//-----------------------------------------------------------------------------
#define hmsu_to_nsec(h,m,s,ns) (hms_to_nsec((h),(m),(s))+(time64_t)((us)*1000))

//-----------------------------------------------------------------------------
// macro	hmsu_to_usec
//
// purpose	Convert hours, minutes, seconds, microseconds, to just
//		microseconds.
//
// arguments	1 (time64_t) hours
//		2 (time64_t) minutes
//		3 (time64_t) seconds
//		4 (time64_t) microseconds
//
// returns	(time64_t) microseconds
//-----------------------------------------------------------------------------
#define hmsu_to_usec(h,m,s,us) (hms_to_usec((h),(m),(s))+(us))

//-----------------------------------------------------------------------------
// macro	str_gregorian_to_ecal
// macro	str_to_ecal
//
// purpose	Convert date and time in a string to type ecal_t for the
//		Gregorian calendar.
//
// arguments	1 (const char *) date and time string
//		2 (ecal_t *) where to store calendar and time data.
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
#define str_gregorian_to_ecal(s,c) ({						\
	ecal_t * libh__cal;							\
	int libh__rc;								\
	libh__cal = (c);							\
	libh__rc = str_cal_to_ecal((s),libh__cal);				\
	if ( libh__rc == 0 ) libh__cal->type = ECAL_TYPE_GREGORIAN;		\
	libh__rc;								\
})

#define str_to_ecal(s,c) ({							\
	ecal_t * libh__cal;							\
	int libh__rc;								\
	libh__cal = (c);							\
	libh__rc = str_cal_to_ecal((s),libh__cal);				\
	if ( libh__rc == 0 ) libh__cal->type = ECAL_TYPE_GREGORIAN;		\
	libh__rc;								\
})

//-----------------------------------------------------------------------------
// macro	str_gregorian_to_eday
// macro	str_to_eday
//
// purpose	Convert date in a string to type eday for Gregorian calendar.
//
// arguments	1 (const char *) date and time string
//
// returns	(eday_t) == EDAY_ERROR: error
//		(eday_t) != EDAY_ERROR: date converted
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
#define str_gregorian_to_eday(s) ({						\
	long int	libh__year	;					\
	int		libh__month	;					\
	int		libh__mday	;					\
	const char *	libh__str	;					\
	libh__str = (s);							\
	( libh__str )								\
	? ({									\
		str_cal_to_ymd( libh__str, & libh__year, & libh__month, & libh__mday );	\
		ymd_gregorian_to_eday( libh__year, libh__month, libh__mday );		\
	})									\
	: EDAY_ERROR;								\
})

#define str_to_eday(s) ({							\
	long int	libh__year	;					\
	int		libh__month	;					\
	int		libh__mday	;					\
	const char *	libh__str	;					\
	libh__str = (s);							\
	( libh__str )								\
	? ({									\
		str_cal_to_ymd( libh__str, & libh__year, & libh__month, & libh__mday );	\
		ymd_gregorian_to_eday( libh__year, libh__month, libh__mday );		\
	})									\
	: EDAY_ERROR;								\
})

//-----------------------------------------------------------------------------
// macro	str_gregorian_to_etime
// macro	str_to_etime
//
// purpose	Convert date in a string to type etime for Gregorian calendar.
//
// arguments	1 (const char *) date and time string
//
// returns	(etime_t) == ETIME_ERROR: error
//		(etime_t) != ETIME_ERROR: date converted
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
#define str_gregorian_to_etime(s) ((str_cal_to_etime)((s),(ymd_gregorian_to_eday)))
#define str_to_etime(s) ((str_cal_to_etime)((s),(ymd_gregorian_to_eday)))

//-----------------------------------------------------------------------------
// macro	str_julian_to_ecal
//
// purpose	Convert date and time in a string to type ecal_t for the
//		Julian calendar.
//
// arguments	1 (const char *) date and time string
//		2 (ecal_t *) where to store calendar and time data.
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
#define str_julian_to_ecal(s,c) ({						\
	ecal_t * libh__cal;							\
	int libh__rc;								\
	libh__cal = (c);							\
	libh__rc = str_cal_to_ecal((s),libh__cal);				\
	if ( libh__rc == 0 ) libh__cal->type = ECAL_TYPE_JULIAN;		\
	libh__rc;								\
})

//-----------------------------------------------------------------------------
// macro	str_julian_to_eday
//
// purpose	Convert date in a string to type eday for Julian calendar.
//
// arguments	1 (const char *) date and time string
//
// returns	(eday_t) == EDAY_ERROR: error
//		(eday_t) != EDAY_ERROR: date converted
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
#define str_julian_to_eday(s) ({						\
	long int	libh__year	;					\
	int		libh__month	;					\
	int		libh__mday	;					\
	const char *	libh__str	;					\
	libh__str = (s);							\
	( libh__str )								\
	? ({									\
		str_cal_to_ymd( libh__str, & libh__year, & libh__month, & libh__mday );	\
		ymd_julian_to_eday( libh__year, libh__month, libh__mday );		\
	})									\
	: EDAY_ERROR;								\
})

//-----------------------------------------------------------------------------
// macro	str_julian_to_etime
//
// purpose	Convert date in a string to type etime for Julian calendar.
//
// arguments	1 (const char *) date and time string
//
// returns	(etime_t) == ETIME_ERROR: error
//		(etime_t) != ETIME_ERROR: date converted
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
#define str_julian_to_etime(s) ((str_cal_to_etime)((s),(ymd_julian_to_eday)))

//-----------------------------------------------------------------------------
// macro	time_to_jd
//
// purpose	Convert a system time value to a Julian Date as used in
//		astronomy.
//
// argument	1 (time_t) system time value
//
// returns	(JD) Julian Date
//-----------------------------------------------------------------------------
#define time_to_jd(t) ((JD)((long double)(t)/86400.0L+2440587.5L))

//-----------------------------------------------------------------------------
// macro	timems_to_jd
//
// purpose	Convert a system time value in milliseconds to a Julian Date
//		as used in astronomy.
//
// argument	1 (timems_t) system time value
//
// returns	(JD) Julian Date
//-----------------------------------------------------------------------------
#define timems_to_jd(t) ((JD)((long double)(t)/86400000.0L+2440587.5L))

//-----------------------------------------------------------------------------
// macro	timens_to_jd
//
// purpose	Convert a system time value in nanoseconds to a Julian Date
//		as used in astronomy.
//
// argument	1 (timens_t) system time value
//
// returns	(JD) Julian Date
//-----------------------------------------------------------------------------
#define timens_to_jd(t) ((JD)((long double)(t)/86400000000000.0L+2440587.5L))

//-----------------------------------------------------------------------------
// macro	timer_cancel
//
// purpose	Cancel the timer.
//
// arguments	-none-
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define timer_cancel() ({							\
	struct itimerval libh__itimerval;					\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = 0;					\
	libh__itimerval.it_value.tv_usec = 0;					\
	setitimer( ( ITIMER_REAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_prof_cancel
//
// purpose	Cancel the timer for profile timing.
//
// arguments	-none-
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define timer_prof_cancel() ({							\
	struct itimerval libh__itimerval;					\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = 0;					\
	libh__itimerval.it_value.tv_usec = 0;					\
	setitimer( ( ITIMER_PROF ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_prof_set_msec
//
// purpose	Set the profile interval timer to signal the alarm at
//		expiration based on a time interval specified with a 64 bit
//		integer value.
//
// arguments	1 (number) time until alarm in milliseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_prof_set_msec(t) ({						\
	struct itimerval libh__itimerval;					\
	timems_t libh__timems;							\
	libh__timems = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timems / 1000L;			\
	libh__itimerval.it_value.tv_usec = libh__timems % 1000L * 1000L;	\
	setitimer( ( ITIMER_PROF ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_prof_set_nsec
//
// purpose	Set the profile interval timer to signal the alarm at
//		expiration based on a time interval specified with a 64 bit
//		integer value.
//
// arguments	1 (number) time until alarm in nanoseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_prof_set_nsec(t) ({						\
	struct itimerval libh__itimerval;					\
	timens_t libh__timens;							\
	libh__timens = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timens / 1000000000L;		\
	libh__itimerval.it_value.tv_usec = libh__timens / 1000L % 1000000L;	\
	setitimer( ( ITIMER_PROF ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_prof_set_sec
//
// purpose	Set the profile interval timer to signal the alarm at
//		expiration based on a time interval specified with a 32 bit
//		integer value.
//
// arguments	1 (number) time until alarm in seconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_prof_set_sec(t) ({						\
	struct itimerval libh__itimerval;					\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = ( t );				\
	libh__itimerval.it_value.tv_usec = 0;					\
	setitimer( ( ITIMER_PROF ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_prof_set_usec
//
// purpose	Set the profile interval timer to signal the alarm at
//		expiration based on a time interval specified with a 64 bit
//		integer value.
//
// arguments	1 (number) time until alarm in microseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_prof_set_usec(t) ({						\
	struct itimerval libh__itimerval;					\
	timeus_t libh__timeus;							\
	libh__timeus = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timeus / 100000L;		\
	libh__itimerval.it_value.tv_usec = libh__timeus % 1000000L;		\
	setitimer( ( ITIMER_PROF ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_set_msec
//
// purpose	Set the real interval timer to signal the alarm at expiration
//		based on a time interval specified with a 64 bit integer value.
//
// arguments	1 (number) time until alarm in milliseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_set_msec(t) ({							\
	struct itimerval libh__itimerval;					\
	timems_t libh__timems;							\
	libh__timems = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timems / 1000L;			\
	libh__itimerval.it_value.tv_usec = libh__timems % 1000L * 1000L;	\
	setitimer( ( ITIMER_REAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_set_nsec
//
// purpose	Set the real interval timer to signal the alarm at expiration
//		based on a time interval specified with a 64 bit integer value.
//
// arguments	1 (number) time until alarm in nanoseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_set_nsec(t) ({							\
	struct itimerval libh__itimerval;					\
	timens_t libh__timens;							\
	libh__timens = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timens / 1000000000L;		\
	libh__itimerval.it_value.tv_usec = libh__timens / 1000L % 1000000L;	\
	setitimer( ( ITIMER_REAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_set_sec
//
// purpose	Set the real interval timer to signal the alarm at expiration
//		based on a time interval specified with a 32 bit integer value.
//
// arguments	1 (number) time until alarm in seconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_set_sec(t) ({							\
	struct itimerval libh__itimerval;					\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = ( t );				\
	libh__itimerval.it_value.tv_usec = 0;					\
	setitimer( ( ITIMER_REAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_set_usec
//
// purpose	Set the real interval timer to signal the alarm at expiration
//		based on a time interval specified with a 64 bit integer value.
//
// arguments	1 (number) time until alarm in microseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_set_usec(t) ({							\
	struct itimerval libh__itimerval;					\
	timeus_t libh__timeus;							\
	libh__timeus = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timeus / 100000L;		\
	libh__itimerval.it_value.tv_usec = libh__timeus % 1000000L;		\
	setitimer( ( ITIMER_REAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// function	timer_virt_cancel
//
// purpose	Cancel the timer for virtual timing.
//
// arguments	-none-
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define timer_virt_cancel() ({							\
	struct itimerval libh__itimerval;					\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = 0;					\
	libh__itimerval.it_value.tv_usec = 0;					\
	setitimer( ( ITIMER_VIRTUAL ), & timeit, ( NULL ) );			\
})

//-----------------------------------------------------------------------------
// macro	timer_virt_set_msec
//
// purpose	Set the virtual interval timer to signal the alarm at
//		expiration based on a time interval specified with a 64 bit
//		integer value.
//
// arguments	1 (number) time until alarm in milliseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_virt_set_msec(t) ({						\
	struct itimerval libh__itimerval;					\
	timems_t libh__timems;							\
	libh__timems = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timems / 1000L;			\
	libh__itimerval.it_value.tv_usec = libh__timems % 1000L * 1000L;	\
	setitimer( ( ITIMER_VIRTUAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_virt_set_nsec
//
// purpose	Set the virtual interval timer to signal the alarm at
//		expiration based on a time interval specified with a 64 bit
//		integer value.
//
// arguments	1 (number) time until alarm in nanoseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_virt_set_nsec(t) ({						\
	struct itimerval libh__itimerval;					\
	timens_t libh__timens;							\
	libh__timens = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timens / 1000000000L;		\
	libh__itimerval.it_value.tv_usec = libh__timens / 1000L % 1000000L;	\
	setitimer( ( ITIMER_VIRTUAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_virt_set_sec
//
// purpose	Set the virtual interval timer to signal the alarm at
//		expiration based on a time interval specified with a 32 bit
//		integer value.
//
// arguments	1 (number) time until alarm in seconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_virt_set_sec(t) ({						\
	struct itimerval libh__itimerval;					\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = ( t );				\
	libh__itimerval.it_value.tv_usec = 0;					\
	setitimer( ( ITIMER_VIRTUAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timer_virt_set_usec
//
// purpose	Set the virtual interval timer to signal the alarm at
//		expiration based on a time interval specified with a 64 bit
//		integer value.
//
// arguments	1 (number) time until alarm in microseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_virt_set_usec(t) ({						\
	struct itimerval libh__itimerval;					\
	timeus_t libh__timeus;							\
	libh__timeus = ( t );							\
	libh__itimerval.it_interval.tv_sec = 0;					\
	libh__itimerval.it_interval.tv_usec = 0;				\
	libh__itimerval.it_value.tv_sec = libh__timeus / 100000L;		\
	libh__itimerval.it_value.tv_usec = libh__timeus % 1000000L;		\
	setitimer( ( ITIMER_VIRTUAL ), & libh__itimerval, ( NULL ) );		\
})

//-----------------------------------------------------------------------------
// macro	timespec_to_eday
//
// purpose	Convert struct timespec to earth day
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(eday_t) earth day
//-----------------------------------------------------------------------------
#define timespec_to_eday(t) ((eday_t)(quo_ni(timespec_to_sec(t),86400UL)+(EDAY_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	timespec_to_etime
//
// purpose	Convert struct timespec to earth time
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(etime_t) earth time
//-----------------------------------------------------------------------------
#define timespec_to_etime(t) ((etime_t)(quo_ni(timespec_to_timens(t),1000UL)+(ETIME_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	timespec_to_lday
//
// purpose	Convert struct timespec to earth day
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(lday_t) earth day
//-----------------------------------------------------------------------------
#define timespec_to_lday(t) ((lday_t)(quo_ni(timespec_to_sec(t),86400UL)+(LDAY_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	timespec_to_timens
//
// purpose	Convert struct timespec to system time in nanoseconds.
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(timens_t) system time in microseconds
//-----------------------------------------------------------------------------
#define timespec_to_timens(t) ((((timens_t)((t)->tv_sec))*1000000000L)+((timens_t)((t)->tv_nsec)))

//-----------------------------------------------------------------------------
// macro	timespec_to_timems
//
// purpose	Convert struct timespec to system time in microseconds.
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(timems_t) system time in milliseconds
//-----------------------------------------------------------------------------
#define timespec_to_timems(t) quo_ni(timespec_to_timens(t),1000000UL)

//-----------------------------------------------------------------------------
// macro	timespec_to_timens
//
// purpose	Convert struct timespec to system time in nanoseconds.
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(timens_t) system time in microseconds
//-----------------------------------------------------------------------------
#define timespec_to_timens(t) ((((timens_t)((t)->tv_sec))*1000000000L)+((timens_t)((t)->tv_nsec)))

//-----------------------------------------------------------------------------
// macro	timespec_to_timeus
//
// purpose	Convert struct timespec to system time in microseconds.
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(timeus_t) system time in nanoseconds
//-----------------------------------------------------------------------------
#define timespec_to_timeus(t) quo_ni(timespec_to_timens(t),1000UL)

//-----------------------------------------------------------------------------
// macro	timespec_to_xday
//
// purpose	Convert struct timespec to earth day
//
// argument	1 (struct timespec *) pointer to struct timespec to convert
//
// returns	(xday_t) earth day
//-----------------------------------------------------------------------------
#define timespec_to_xday(t) ((xday_t)(quo_ni(timespec_to_sec(t),86400UL)+(XDAY_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	timeus_to_jd
//
// purpose	Convert a system time value in microseconds to a Julian Date
//		as used in astronomy.
//
// argument	1 (timeus_t) system time value
//
// returns	(JD) Julian Date
//-----------------------------------------------------------------------------
#define timeus_to_jd(t) ((JD)((long double)(t)/86400000000.0L+2440587.5L))

//-----------------------------------------------------------------------------
// macro	timeval_to_eday
//
// purpose	Convert struct timeval to earth day
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(eday_t) earth day
//-----------------------------------------------------------------------------
#define timeval_to_eday(t) ((eday_t)(quo_ni(timeval_to_sec(t),86400UL)+(EDAY_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	timeval_to_etime
//
// purpose	Convert struct timeval to earth time
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(etime_t) earth time
//-----------------------------------------------------------------------------
#define timeval_to_etime(t) ((etime_t)(timeval_to_timeus(t)+(ETIME_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	timeval_to_lday
//
// purpose	Convert struct timeval to earth day
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(lday_t) earth day
//-----------------------------------------------------------------------------
#define timeval_to_lday(t) ((lday_t)(quo_ni(timeval_to_sec(t),86400UL)+(LDAY_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	timeval_to_time
// alias	timeval_to_sec
//
// purpose	Convert struct timeval to system time in time.
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(time_t) system time in seconds
//-----------------------------------------------------------------------------
#define timeval_to_time(t) ((time_t)((t)->tv_sec))
#define timeval_to_sec(t) ((time_t)((t)->tv_sec))

//-----------------------------------------------------------------------------
// macro	timeval_to_timems
//
// purpose	Convert struct timeval to system time in microseconds.
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(timems_t) system time in milliseconds
//-----------------------------------------------------------------------------
#define timeval_to_timems(t) quo_ni(timeval_to_timeus(t),1000U)

//-----------------------------------------------------------------------------
// macro	timeval_to_timens
//
// purpose	Convert struct timeval to system time in nanoseconds.
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(timens_t) system time in nanoseconds
//-----------------------------------------------------------------------------
#define timeval_to_timens(t) (timeval_to_timeus(t)*1000L)

//-----------------------------------------------------------------------------
// macro	timeval_to_timeus
//
// purpose	Convert struct timeval to system time in microseconds.
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(timeus_t) system time in microseconds
//-----------------------------------------------------------------------------
#define timeval_to_timeus(t) ((((timeus_t)((t)->tv_sec))*1000000UL)+((timeus_t)((t)->tv_usec)))

//-----------------------------------------------------------------------------
// macro	timeval_to_xday
//
// purpose	Convert struct timeval to earth day
//
// argument	1 (struct timeval *) pointer to struct timeval to convert
//
// returns	(xday_t) earth day
//-----------------------------------------------------------------------------
#define timeval_to_xday(t) ((xday_t)(quo_ni(timeval_to_sec(t),86400UL)+(XDAY_SYSTEM_EPOCH)))

//-----------------------------------------------------------------------------
// macro	ymd_gregorian_to_etime
//
// purpose	Convert a Gregorian calendar date to an Earth Time value.
//
// arguments	1 (long) Gregorian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Gregorian calendar month (1-12)
//		3 (int) Gregorian calendar date (1-31)
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Gregorian calendar.
//		There is no accomodation for selection between Gregorian
//		and Julian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymd_gregorian_to_etime(y,m,d) (eday_to_etime(ymd_gregorian_to_eday((y),(m),(d))))

//-----------------------------------------------------------------------------
// macro	ymd_julian_to_etime
//
// purpose	Convert a Julian calendar date to an Earth Time value.
//
// arguments	1 (long) Julian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Julian calendar month (1-12)
//		3 (int) Julian calendar date (1-31)
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Julian calendar.
//		There is no accomodation for selection between Julian and
//		Gregorian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymd_julian_to_etime(y,m,d) (eday_to_etime(ymd_julian_to_eday((y),(m),(d))))

//-----------------------------------------------------------------------------
// macro	ymdhms_gregorian_to_etime
//
// purpose	Convert a Gregorian calendar date, plus time in hours, minutes,
//		and seconds, to an Earth Time value.
//
// arguments	1 (long) Gregorian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Gregorian calendar month (1-12)
//		3 (int) Gregorian calendar date (1-31)
//		4 (int) Time of day in hours
//		5 (int) Time of day in minutes
//		6 (int) Time of day in seconds
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Gregorian calendar.
//		There is no accomodation for selection between Gregorian
//		and Julian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhms_gregorian_to_etime(yr,mo,da,hr,mi,se) (ymd_gregorian_to_etime((yr),(mo),(da))+hms_to_usec((hr),(mi),(se)))

//-----------------------------------------------------------------------------
// macro	ymdhms_julian_to_etime
//
// purpose	Convert a Julian calendar date, plus time in hours, minutes,
//		and seconds, to an Earth Time value.
//
// arguments	1 (long) Julian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Julian calendar month (1-12)
//		3 (int) Julian calendar date (1-31)
//		4 (int) Time of day in hours
//		5 (int) Time of day in minutes
//		6 (int) Time of day in seconds
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Julian calendar.
//		There is no accomodation for selection between Julian and
//		Gregorian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhms_julian_to_etime(yr,mo,da,hr,mi,se) (ymd_julian_to_etime((yr),(mo),(da))+hms_to_usec((hr),(mi),(se)))

//-----------------------------------------------------------------------------
// macro	ymdhmsm_gregorian_to_etime
//
// purpose	Convert a Gregorian calendar date, plus time in hours, minutes,
//		seconds, and milliseconds, to an Earth Time value.
//
// arguments	1 (long) Gregorian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Gregorian calendar month (1-12)
//		3 (int) Gregorian calendar date (1-31)
//		4 (int) Hours part
//		5 (int) Minutes part
//		6 (int) Seconds part
//		7 (long) Milliseconds part
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Gregorian calendar.
//		There is no accomodation for selection between Gregorian
//		and Julian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhmsm_gregorian_to_etime(yr,mo,da,hr,mi,se,ms) (ymd_gregorian_to_etime((yr),(mo),(da))+hmsm_to_usec((hr),(mi),(se),(ms)))

//-----------------------------------------------------------------------------
// macro	ymdhmsm_julian_to_etime
//
// purpose	Convert a Julian calendar date, plus time in hours, minutes,
//		seconds, and milliseconds, to an Earth Time value.
//
// arguments	1 (long) Julian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Julian calendar month (1-12)
//		3 (int) Julian calendar date (1-31)
//		4 (int) Hours part
//		5 (int) Minutes part
//		6 (int) Seconds part
//		7 (long) Milliseconds part
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Julian calendar.
//		There is no accomodation for selection between Julian and
//		Gregorian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhmsm_julian_to_etime(yr,mo,da,hr,mi,se,ms) (ymd_julian_to_etime((yr),(mo),(da))+hmsm_to_usec((hr),(mi),(se),(ms)))

//-----------------------------------------------------------------------------
// macro	ymdhmsn_gregorian_to_etime
//
// purpose	Convert a Gregorian calendar date, plus time in hours, minutes,
//		seconds, and nanoseconds, to an Earth Time value.
//
// arguments	1 (long) Gregorian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Gregorian calendar month (1-12)
//		3 (int) Gregorian calendar date (1-31)
//		4 (int) Hours part
//		5 (int) Minutes part
//		6 (int) Seconds part
//		7 (long) Nanoseconds part
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Gregorian calendar.
//		There is no accomodation for selection between Gregorian
//		and Julian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhmsn_gregorian_to_etime(yr,mo,da,hr,mi,se,ms) (ymd_gregorian_to_etime((yr),(mo),(da))+hmsn_to_usec((hr),(mi),(se),(ms)))

//-----------------------------------------------------------------------------
// macro	ymdhmsn_julian_to_etime
//
// purpose	Convert a Julian calendar date, plus time in hours, minutes,
//		seconds, and nanoseconds, to an Earth Time value.
//
// arguments	1 (long) Julian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Julian calendar month (1-12)
//		3 (int) Julian calendar date (1-31)
//		4 (int) Hours part
//		5 (int) Minutes part
//		6 (int) Seconds part
//		7 (long) Nanoseconds part
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Julian calendar.
//		There is no accomodation for selection between Julian and
//		Gregorian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhmsn_julian_to_etime(yr,mo,da,hr,mi,se,ns) (ymd_julian_to_etime((yr),(mo),(da))+hmsn_to_usec((hr),(mi),(se),(ns)))

//-----------------------------------------------------------------------------
// macro	ymdhmsu_gregorian_to_etime
//
// purpose	Convert a Gregorian calendar date, plus time in hours, minutes,
//		seconds, and microseconds, to an Earth Time value.
//
// arguments	1 (long) Gregorian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Gregorian calendar month (1-12)
//		3 (int) Gregorian calendar date (1-31)
//		4 (int) Hours part
//		5 (int) Minutes part
//		6 (int) Seconds part
//		7 (long) Microseconds part
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Gregorian calendar.
//		There is no accomodation for selection between Gregorian
//		and Julian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhmsu_gregorian_to_etime(yr,mo,da,hr,mi,se,us) (ymd_gregorian_to_etime((yr),(mo),(da))+hmsu_to_usec((hr),(mi),(se),(us)))

//-----------------------------------------------------------------------------
// macro	ymdhmsu_julian_to_etime
//
// purpose	Convert a Julian calendar date, plus time in hours, minutes,
//		seconds, and microseconds, to an Earth Time value.
//
// arguments	1 (long) Julian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Julian calendar month (1-12)
//		3 (int) Julian calendar date (1-31)
//		4 (int) Hours part
//		5 (int) Minutes part
//		6 (int) Seconds part
//		7 (long) Microseconds part
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Julian calendar.
//		There is no accomodation for selection between Julian and
//		Gregorian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhmsu_julian_to_etime(yr,mo,da,hr,mi,se,us) (ymd_julian_to_etime((yr),(mo),(da))+hmsu_to_usec((hr),(mi),(se),(us)))

//-----------------------------------------------------------------------------
// function	day_abbrev
//
// purpose	Given a day of week number in the range 0 to 6, return the
//		abbreviation of the day obtained from nl_langinfo() which
//		will be based on the current locale setting.
//
// arguments	1 (int) day of week number in the range 0 to 6
//
// returns	(const char *) pointer to string with day abbreviation.
//-----------------------------------------------------------------------------
const char *
day_abbrev (
    int		arg_day_num
    )
;

//-----------------------------------------------------------------------------
// function	day_name
//
// purpose	Given a day of week number in the range 0 to 6, return the
//		name of the day obtained from nl_langinfo() which will be
//		based on the current locale setting.
//
// arguments	1 (int) day of week number in the range 0 to 6
//
// returns	(const char *) pointer to string with day name.
//-----------------------------------------------------------------------------
const char *
day_name (
    int		arg_day_num
    )
;

//-----------------------------------------------------------------------------
// function	doy_to_dom
//
// purpose	Convert a day of year to a day of month for both Gregorian and
//		Julian calendars.
//
// arguments	1 (int) day of year (0..364 or 0..365)
//		2 (int) 0: non-leap year, 1: leap-year
//
// returns	(int) 1..31: day of month
//		(int) -1: error
//-----------------------------------------------------------------------------
int
doy_to_dom (
    int		arg_doy
    ,
    int		arg_leap
    )
;

//-----------------------------------------------------------------------------
// function	doy_to_mon
//
// purpose	Convert a day of year to a month for both Gregorian and	Julian
//		calendars.
//
// arguments	1 (int) day of year (0..364 or 0..365)
//		2 (int) 0: non-leap year, 1: leap-year
//
// returns	(int) 1..12: month
//		(int) -1: error
//-----------------------------------------------------------------------------
int
doy_to_mon (
    int		arg_doy
    ,
    int		arg_leap
    )
;

//-----------------------------------------------------------------------------
// function	ecal_to_eday
//
// purpose	Convert calendar elements to an Earth Day value.
//
// arguments	1 (ecal_t *) calendar struct to be converted
//
// returns	(eday_t) resultant Earth Day value
//-----------------------------------------------------------------------------
eday_t
ecal_to_eday (
    ecal_t *	arg_cal_p
    )
;

//-----------------------------------------------------------------------------
// function	ecal_to_etime
//
// purpose	Convert calendar elements to an Earth Time value.
//
// arguments	1 (ecal_t *) calendar struct to be converted
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//-----------------------------------------------------------------------------
etime_t
ecal_to_etime (
    ecal_t *	arg_cal_p
    )
;

//-----------------------------------------------------------------------------
// function	ecal_to_tm
//
// purpose	Convert an struct ecal to struct tm.
//
// arguments	1 (ecal_t *) Ecal struct pointer
//		2 (struct tm *) pointer to struct tm to store into
//
// returns	(void)
//
// warning	Since Standard C defines struct tm generally in terms of it
//		being converted from, or to, type time_t, programs might be
//		expecting the values in struct tm to be restricted to the same
//		range of dates.  Implementations of conversion from struct tm
//		to strings (e.g. strftime()) may be limited in that way due to
//		use of type time_t to do necessary calculations, and lack of a
//		standard requirement for a greater range.
//
//		The ecal_to_tm() function will apply the full scale of years
//		to struct tm, which may not be handled properly by programs
//		and functions not supporting that range.  The range of years
//		supported by Earth Day is over 11 million years.
//-----------------------------------------------------------------------------
void
ecal_to_tm (
    ecal_t *		arg_ecal_p
    ,
    struct tm *		arg_tm_p
    )
;

//-----------------------------------------------------------------------------
// function	eday_get_ymd_gregorian
// alias	eday_get_ymd
//
// purpose	Extract calendar elements in the Gregorian calendar from eday_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//
// returns	(void)
//-----------------------------------------------------------------------------
#define eday_get_ymd eday_get_ymd_gregorian
void
eday_get_ymd_gregorian (
    eday_t	arg_eday
    ,
    long *	arg_year_p
    ,
    int *	arg_month_p
    ,
    int *	arg_mday_p
    )
;

//-----------------------------------------------------------------------------
// function	eday_get_ymd_julian
//
// purpose	Extract calendar elements in the Julian calendar from eday_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//
// returns	(void)
//-----------------------------------------------------------------------------
void
eday_get_ymd_julian (
    eday_t	arg_eday
    ,
    long *	arg_year_p
    ,
    int *	arg_month_p
    ,
    int *	arg_mday_p
    )
;

//-----------------------------------------------------------------------------
// function	eday_to_ecal_gregorian
// alias	eday_to_ecal
//
// purpose	Convert an Earth Day value to Gregorian calendar elements.
//
// arguments	1 (eday_t) Earth Day value
//		2 (ecal_t *) where to store calendar elements.
//
// returns	(void)
//-----------------------------------------------------------------------------
#define eday_to_ecal eday_to_ecal_gregorian
void
eday_to_ecal_gregorian (
    eday_t	arg_eday
    ,
    ecal_t *	arg_caltime
    )
;

//-----------------------------------------------------------------------------
// function	eday_to_ecal_julian
//
// purpose	Convert an Earth Day value to Julian calendar elements.
//
// arguments	1 (eday_t) Earth Day value
//		2 (ecal_t *) where to store calendar elements.
//
// returns	(void)
//-----------------------------------------------------------------------------
void
eday_to_ecal_julian (
    eday_t	arg_eday
    ,
    ecal_t *	arg_caltime
    )
;

//-----------------------------------------------------------------------------
// function	eday_to_fprintf
//
// purpose	Convert an Earth Day value to a formatted string and output.
//
// arguments	1 (FILE *) which file to write to
//		2 (const char *) format string
//		3 (eday_t) Earth Day value
//
// returns	(int) return value from fputs()
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although time format specifiers are valid when formatting
//		an Earth Day value, they are always converted from 0 which
//		represents the beginning of the given day.
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
int
eday_to_fprintf (
    FILE *		arg_file
    ,
    const char *	arg_format
    ,
    eday_t		arg_eday
    )
;

//-----------------------------------------------------------------------------
// function	eday_to_str_app
//
// purpose	Append to the first string (given with maximum available space)
//		an Earth Day value in the format specified.
//
// arguments	1 (char *) target string to append to
//		2 (size_t) size limit for target string
//		3 (const char *) format string
//		4 (eday_t) Earth Day value
//
// returns	(size_t) new length of target string or ~0 if append fails
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although time format specifiers are valid when formatting
//		an Earth Day value, they are always converted from 0 which
//		represents the beginning of the given day.
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
size_t
eday_to_str_app (
    char *		arg_target
    ,
    size_t		arg_space
    ,
    const char *	arg_format
    ,
    eday_t		arg_eday
    )
;

//-----------------------------------------------------------------------------
// function	eday_to_str_dup
//
// purpose	Convert an Earth Day value to a formatted string.
//
// arguments	1 (const char *) format string
//		2 (eday_t) Earth Day value
//
// returns	(char *) pointer to allocated string.
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although time format specifiers are valid when formatting
//		an Earth Day value, they are always converted from 0 which
//		represents the beginning of the given day.
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
char *
eday_to_str_dup (
    const char *	arg_format
    ,
    eday_t		arg_eday
    )
;

//-----------------------------------------------------------------------------
// function	eday_to_timespec
//
// purpose	Convert earth day to struct timespec.  The converted time is
//		the beginning of the day.
//
// arguments	1 (struct timespec *) pointer to destination struct timespec
//		2 (eday_t) earth day
//
// returns	(void)
//-----------------------------------------------------------------------------
void
eday_to_timespec (
    struct timespec *	arg_timets_p
    ,
    eday_t		arg_eday
    )
;

//-----------------------------------------------------------------------------
// function	eday_to_timeval
//
// purpose	Convert earth day to struct timeval.  The converted time is
//		the beginning of the day.
//
// arguments	1 (struct timeval *) pointer to destination struct timeval
//		2 (eday_t) earth day
//
// returns	(void)
//-----------------------------------------------------------------------------
void
eday_to_timeval (
    struct timeval *	arg_timetv_p
    ,
    eday_t		arg_eday
    )
;

//-----------------------------------------------------------------------------
// function	etime_to_ecal_gregorian
// alias	etime_to_ecal
//
// purpose	Convert an Earth Time value to Gregorian calendar elements.
//
// arguments	1 (etime_t) Earth Time value
//		2 (ecal_t *) where to store calendar elements.
//
// returns	(int) 0 : OK
//-----------------------------------------------------------------------------
#define etime_to_ecal(t,c) etime_to_ecal_gregorian((t),(c))
int
etime_to_ecal_gregorian (
    etime_t	arg_etime
    ,
    ecal_t *	arg_ecal_p
    )
;

//-----------------------------------------------------------------------------
// function	etime_to_ecal_julian
//
// purpose	Convert an Earth Time value to Julian calendar elements.
//
// arguments	1 (etime_t) Earth Time value
//		2 (ecal_t *) where to store calendar elements.
//
// returns	(int) 0 : OK
//-----------------------------------------------------------------------------
int
etime_to_ecal_julian (
    etime_t	arg_etime
    ,
    ecal_t *	arg_ecal_p
    )
;

//-----------------------------------------------------------------------------
// function	etime_to_fprintf
//
// purpose	Convert an Earth Time value to a formatted string and output.
//
// arguments	1 (FILE *) which file to write to
//		2 (const char *) format string
//		3 (etime_t) Earth Time value
//
// returns	(int) return value from fputs()
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
int
etime_to_fprintf (
    FILE *		arg_file
    ,
    const char *	arg_format
    ,
    etime_t		arg_etime
    )
;

//-----------------------------------------------------------------------------
// function	etime_to_str_app
//
// purpose	Append to the first string (given with maximum available space)
//		an Earth Time value in the format specified.
//
// arguments	1 (char *) target string to append to
//		2 (size_t) size limit for target string
//		3 (const char *) format string
//		4 (etime_t) Earth Time value
//
// returns	(size_t) new length of target string or ~0 if append fails
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
size_t
etime_to_str_app (
    char *		arg_target
    ,
    size_t		arg_space
    ,
    const char *	arg_format
    ,
    etime_t		arg_etime
    )
;

//-----------------------------------------------------------------------------
// function	etime_to_str_dup
//
// purpose	Convert an Earth Time value to a formatted string.
//
// arguments	1 (const char *) format string
//		2 (etime_t) Earth Time value
//
// returns	(char *) pointer to allocated string.
//
// format	The format string is copied with format specifiers replaced
//		with character expressions of the specified date or time
//		component.  The available specifiers are:
//		%g	Specify Gregorian calendar (no output)
//		%j	Specify Julian calendar (no output)
//		%l	Specify lower case conversion (no output)
//		%u	Specify upper case conversion (no output)
//		%n	Specify no case conversion (no output)
//		%E	(1) Earth Time as a decimal number
//		%e	(1) Earth Day as a decimal number
//		%t	(1) System time as a decimal number (e.g. time())
//		%J	(1,0) Julian Date (floating point format)
//		%Y	(04) Year number in the specified calendar
//		%Z	(04) Year number with BC if needed
//		%m	(02) Month number in the specified calendar
//		%d	(02) Day of month in the specified calendar
//		%D	(03) Day of year in the specified calendar
//		%w	(1) Day of week number (0=Sunday, 6=Saturay)
//		%W	(02) Week number in the year (0..53)
//		%A	Day of week name full
//		%a	Day of week name abbreviated
//		%B	Month of year name full
//		%b	Month of year name abbreviated
//		%H	(02) Hours (00..23) of day
//		%M	(02) Minutes (00..59) of hour
//		%S	(02) Seconds (00..59) of minute
//		%F	(03) Fractions of a second (default milliseconds)
//		%U	(06) Fractions of a second (default microseconds)
//		%N	(09) Fractions of a second (default nanoseconds)
//		%s	(05) Seconds (00000..86401) of day
//		%L	(06) Lilian day (LDAY, IBM Y2K hack)
//		%X	(06) eXchange day (XDAY, Bob Bemer Y2K hack)
//		%I	(02) Hour (12,01..11) of meridian
//		%p	Meridian hour designator, AM or PM
//		%T	short form for "%H:%M:%S"
//		%r	short form for "%I:%M:%S %p"
//		%%	Literal "%" character
//
// note		Although many specifiers are the same as those defined for
//		use in strftime(), many others are different.
//
// modifiers	A sequence of decimal digits may modify the minimum size
//		and leading character for fields.  Some default to a normal
//		size and this may be overridden.  For example, the day of
//		month defaults to a size specifier of "02" if none is given.
//		To force a date to have 1 digit when less than 10, use "1"
//		as the modifier like "%1d".
//-----------------------------------------------------------------------------
char *
etime_to_str_dup (
    const char *	arg_format
    ,
    etime_t		arg_etime
    )
;

//-----------------------------------------------------------------------------
// function	etime_to_timespec
//
// purpose	Convert earth time to struct timespec.
//
// arguments	1 (struct timespec *) pointer to destination struct timespec
//		2 (etime_t) earth time
//
// returns	(void)
//-----------------------------------------------------------------------------
void
etime_to_timespec (
    struct timespec *	arg_timets_p
    ,
    etime_t		arg_etime
    )
;

//-----------------------------------------------------------------------------
// function	etime_to_timeval
//
// purpose	Convert earth time to struct timeval.
//
// arguments	1 (struct timeval *) pointer to destination struct timeval
//		2 (etime_t) earth time
//
// returns	(void)
//-----------------------------------------------------------------------------
void
etime_to_timeval (
    struct timeval *	arg_timetv_p
    ,
    etime_t		arg_etime
    )
;

//-----------------------------------------------------------------------------
// function	jd_to_eday
//
// purpose	Convert a Julian Date value as used in astronomy to an
//		Earth Day value.
//
// arguments	1 (JD) Julian Date value
//
// returns	(eday_t) Earth Day value
//-----------------------------------------------------------------------------
eday_t
jd_to_eday (
    JD		arg_jd
    )
;

//-----------------------------------------------------------------------------
// function	jd_to_etime
//
// purpose	Convert a Julian Date value as used in astronomy to an
//		Earth Time value.
//
// arguments	1 (JD) Julian Date value
//
// returns	(etime_t) Earth Time value
//-----------------------------------------------------------------------------
etime_t
jd_to_etime (
    JD		arg_jd
    )
;

//-----------------------------------------------------------------------------
// function	jd_to_time
//
// purpose	Convert a Julian Date value as used in astronomy to a system
//		time_t value.
//
// arguments	1 (JD) Julian Date value
//
// returns	(time_t) system time value
//-----------------------------------------------------------------------------
time_t
jd_to_time (
    JD		arg_jd
    )
;

//-----------------------------------------------------------------------------
// function	jd_to_timems
//
// purpose	Convert a Julian Date value as used in astronomy to a system
//		time value in milliseconds.
//
// arguments	1 (JD) Julian Date value
//
// returns	(timems_t) system time value in milliseconds
//-----------------------------------------------------------------------------
timems_t
jd_to_timems (
    JD		arg_jd
    )
;

//-----------------------------------------------------------------------------
// function	jd_to_timens
//
// purpose	Convert a Julian Date value as used in astronomy to a system
//		time value in nanoseconds.
//
// arguments	1 (JD) Julian Date value
//
// returns	(timens_t) system time value in nanoseconds
//-----------------------------------------------------------------------------
timens_t
jd_to_timens (
    JD		arg_jd
    )
;

//-----------------------------------------------------------------------------
// function	jd_to_timeus
//
// purpose	Convert a Julian Date value as used in astronomy to a system
//		time value in microseconds.
//
// arguments	1 (JD) Julian Date value
//
// returns	(timeus_t) system time value in microseconds
//-----------------------------------------------------------------------------
timeus_t
jd_to_timeus (
    JD		arg_jd
    )
;

//-----------------------------------------------------------------------------
// function	month_abbrev
//
// purpose	Given a month of year number in the range 0 to 11, return the
//		abbreviation of the month obtained from nl_langinfo() which
//		will be based on the current locale setting.
//
// arguments	1 (int) month of year number in the range 0 to 11
//
// returns	(const char *) pointer to string with month abbreviation.
//-----------------------------------------------------------------------------
const char *
month_abbrev (
    int		arg_month_num
    )
;

//-----------------------------------------------------------------------------
// function	month_name
//
// purpose	Given a month of year number in the range 0 to 11, return the
//		name of the month obtained from nl_langinfo() which will be
//		based on the current locale setting.
//
// arguments	1 (int) month of year number in the range 0 to 11
//
// returns	(const char *) pointer to string with month name.
//-----------------------------------------------------------------------------
const char *
month_name (
    int		arg_month_num
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ecal
//
// purpose	Convert date and time in a string to type ecal_t for either
//		the Gregorian calendar or the Julian calendar.
//
// arguments	1 (const char *) date and time string
//		2 (ecal_t *) where to store calendar and time data.
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
int
str_cal_to_ecal (
    const char *	arg_str
    ,
    ecal_t *		arg_ecal
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_eday
//
// purpose	Convert date in a string to type eday for either Gregorian or
//		Julian calendar.
//
// arguments	1 (const char *) date and time string
//		2 (eday_t (*)(long,int,int)) function to do conversion
//
// returns	(eday_t) == EDAY_ERROR: error
//		(eday_t) != EDAY_ERROR: date converted
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
eday_t
str_cal_to_eday (
    const char *	arg_str
    ,
    eday_t ( *		arg_fun		)( long, int, int )
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_etime
//
// purpose	Convert date and time in a string to type etime.
//
// arguments	1 (const char *) date and time string
//		2 (eday_t (*)(long,int,int)) function to do conversion
//
// returns	(etime_t) == ETIME_MIN: error
//		(etime_t) == ETIME_MAX: error
//		(etime_t) date and time converted
//
// format	yyyy/mm/dd_hh:mm:ss.nnnnnn[nnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//-----------------------------------------------------------------------------
etime_t
str_cal_to_etime (
    const char *	arg_str
    ,
    eday_t ( *		arg_fun		)( long, int, int )
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymd
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
int
str_cal_to_ymd (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymdhms
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour of day
//		6 (int *) where to store minute of hour
//		7 (int *) where to store second of minute
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd_hh:mm:ss[.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Fractional time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
int
str_cal_to_ymdhms (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    ,
    int *		arg_hour
    ,
    int *		arg_min
    ,
    int *		arg_sec
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymdhmsa
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour of day
//		6 (int *) where to store minute of hour
//		7 (int *) where to store second of minute
//		8 (long long *) where to store attosecond of second
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd_hh:mm:ss.nnnnnnnnnnnnnnnnnn
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//-----------------------------------------------------------------------------
int
str_cal_to_ymdhmsa (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    ,
    int *		arg_hour
    ,
    int *		arg_min
    ,
    int *		arg_sec
    ,
    long long *		arg_asec
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymdhmsf
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour of day
//		6 (int *) where to store minute of hour
//		7 (int *) where to store second of minute
//		8 (long long *) where to store femtosecond of second
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd_hh:mm:ss.nnnnnnnnnnnnnnn[nnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//-----------------------------------------------------------------------------
int
str_cal_to_ymdhmsf (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    ,
    int *		arg_hour
    ,
    int *		arg_min
    ,
    int *		arg_sec
    ,
    long long *		arg_fsec
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymdhmsm
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour of day
//		6 (int *) where to store minute of hour
//		7 (int *) where to store second of minute
//		8 (int *) where to store millisecond of second
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd_hh:mm:ss.nnn[nnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//-----------------------------------------------------------------------------
int
str_cal_to_ymdhmsm (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    ,
    int *		arg_hour
    ,
    int *		arg_min
    ,
    int *		arg_sec
    ,
    int *		arg_msec
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymdhmsn
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour of day
//		6 (int *) where to store minute of hour
//		7 (int *) where to store second of minute
//		8 (long *) where to store nanosecond of second
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd_hh:mm:ss.nnnnnnnnn[nnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//-----------------------------------------------------------------------------
int
str_cal_to_ymdhmsn (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    ,
    int *		arg_hour
    ,
    int *		arg_min
    ,
    int *		arg_sec
    ,
    long *		arg_nsec
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymdhmsp
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour of day
//		6 (int *) where to store minute of hour
//		7 (int *) where to store second of minute
//		8 (long long *) where to store picosecond of second
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd_hh:mm:ss.nnnnnnnnnnnn[nnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//-----------------------------------------------------------------------------
int
str_cal_to_ymdhmsp (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    ,
    int *		arg_hour
    ,
    int *		arg_min
    ,
    int *		arg_sec
    ,
    long long *		arg_psec
    )
;

//-----------------------------------------------------------------------------
// function	str_cal_to_ymdhmsu
//
// purpose	Convert date and time in a string to calendar elements via
//		separate pointer arguments.
//
// arguments	1 (const char *) date and time string
//		2 (long int *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour of day
//		6 (int *) where to store minute of hour
//		7 (int *) where to store second of minute
//		8 (long *) where to store microsecond of second
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd_hh:mm:ss.nnnnnn[nnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//-----------------------------------------------------------------------------
int
str_cal_to_ymdhmsu (
    const char *	arg_str
    ,
    long int *		arg_year
    ,
    int *		arg_month
    ,
    int *		arg_mday
    ,
    int *		arg_hour
    ,
    int *		arg_min
    ,
    int *		arg_sec
    ,
    long *		arg_usec
    )
;

//-----------------------------------------------------------------------------
// function	str_to_msecs
//
// purpose	Convert a time period expressed in a string with suffixes and
//		decimal fractions to int_fast64_t milliseconds.
//
// arguments	1 (const char *) time string
//
// returns	(int_fast64_t) >= 0 : resultant time in milliseconds
//		(int_fast64_t)  < 0 : error
//
// format	[#[.#]w|W][#[.#]d|D][#[.#]h|H][#[.#]m|M][#[.#][s|S]]
//-----------------------------------------------------------------------------
int_fast64_t
str_to_msecs (
    const char *	arg_time_str
    )
;

//-----------------------------------------------------------------------------
// function	str_to_nsecs
//
// purpose	Convert a time period expressed in a string with suffixes and
//		decimal fractions to int_fast64_t nanoseconds.
//
// arguments	1 (const char *) time string
//
// returns	(int_fast64_t) >= 0 : resultant time in nanoseconds
//		(int_fast64_t)  < 0 : error
//
// format	[#[.#]w|W][#[.#]d|D][#[.#]h|H][#[.#]m|M][#[.#][s|S]]
//-----------------------------------------------------------------------------
int_fast64_t
str_to_nsecs (
    const char *	arg_time_str
    )
;

//-----------------------------------------------------------------------------
// function	str_to_secs
//
// purpose	Convert a time period expressed in a string with suffixes to
//		to int seconds.  Unlike other related functions, decimal
//		fractions are not implemented.
//
// arguments	1 (const char *) time string
//
// returns	(int) >= 0 : resultant time in seconds
//		(int)  < 0 : error
//
// format	[#w|W][#d|D][#h|H][#m|M][#[s|S]]
//-----------------------------------------------------------------------------
int
str_to_secs (
    const char *	arg_time_str
    )
;

//-----------------------------------------------------------------------------
// function	str_to_timespec
//
// purpose	Convert a time period expressed in a string with suffixes and
//		decimal fractions to timespec, stored where specified.
//
// arguments	1 (const char *) time string
//		2 (struct timespec *) timespec to store result in
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	[#[.#][wdhms]]
//-----------------------------------------------------------------------------
int
str_to_timespec (
    const char *	arg_time_str
    ,
    struct timespec *	arg_timespec_p
    )
;

//-----------------------------------------------------------------------------
// function	str_to_timeval
//
// purpose	Convert a time period expressed in a string with suffixes and
//		decimal fractions to timeval, stored where specified.
//
// arguments	1 (const char *) time string
//		2 (struct timeval *) timeval to store result in
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	[#[.#][wdhms]]
//-----------------------------------------------------------------------------
int
str_to_timeval (
    const char *	arg_time_str
    ,
    struct timeval *	arg_timeval_p
    )
;

//-----------------------------------------------------------------------------
// function	str_to_usecs
//
// purpose	Convert a time period expressed in a string with suffixes and
//		decimal fractions to int_fast64_t microseconds.
//
// arguments	1 (const char *) time string
//
// returns	(int_fast64_t) >= 0 : resultant time in microseconds
//		(int_fast64_t)  < 0 : error
//
// format	[#[.#]w|W][#[.#]d|D][#[.#]h|H][#[.#]m|M][#[.#][s|S]]
//-----------------------------------------------------------------------------
int_fast64_t
str_to_usecs (
    const char *	arg_time_str
    )
;

//-----------------------------------------------------------------------------
// function	time_to_ecal_gregorian
// alias	time_to_ecal
//
// purpose	Convert a system time value to Gregorian calendar elements.
//
// arguments	1 (time_t) system time value
//		2 (ecal_t *) where to store calendar elements.
//
// returns	(int) 0 : OK
//-----------------------------------------------------------------------------
#define time_to_ecal(t,c) time_to_ecal_gregorian((t),(c))
int
time_to_ecal_gregorian (
    time_t	arg_time
    ,
    ecal_t *	arg_ecal_p
    )
;

//-----------------------------------------------------------------------------
// function	time_to_ecal_julian
//
// purpose	Convert a system time value to Julian calendar elements.
//
// arguments	1 (time_t) system time value
//		2 (ecal_t *) where to store calendar elements.
//
// returns	(int) 0 : OK
//-----------------------------------------------------------------------------
int
time_to_ecal_julian (
    time_t	arg_time
    ,
    ecal_t *	arg_ecal_p
    )
;

//-----------------------------------------------------------------------------
// function	timems_to_timespec
//
// purpose	Convert system time in milliseconds to struct timespec.
//
// arguments	1 (struct timespec *) pointer to destination struct timespec
//		2 (timems_t) system time in milliseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
void
timems_to_timespec (
    struct timespec *	arg_timets_p
    ,
    timens_t		arg_time_msec
    )
;

//-----------------------------------------------------------------------------
// function	timems_to_timeval
//
// purpose	Convert system time in milliseconds to struct timeval.
//
// arguments	1 (struct timeval *) pointer to destination struct timeval
//		2 (timems_t) system time in milliseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
void
timems_to_timeval (
    struct timeval *	arg_timetv_p
    ,
    timens_t		arg_time_msec
    )
;

//-----------------------------------------------------------------------------
// function	timens_to_timespec
//
// purpose	Convert system time in nanoseconds to struct timespec.
//
// arguments	1 (struct timespec *) pointer to destination struct timespec
//		2 (timens_t) system time in nanooseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
void
timens_to_timespec (
    struct timespec *	arg_timets_p
    ,
    timens_t		arg_time_nsec
    )
;

//-----------------------------------------------------------------------------
// function	timens_to_timeval
//
// purpose	Convert system time in nanoseconds to struct timeval.
//
// arguments	1 (struct timeval *) pointer to destination struct timeval
//		2 (timens_t) system time in nanoseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
void
timens_to_timeval (
    struct timeval *	arg_timetv_p
    ,
    timens_t		arg_time_nsec
    )
;

//-----------------------------------------------------------------------------
// function	timer_mod_set_usec
// macro	timer_mod_set_sec
// macro	time_mod_set_msec
// macro	time_mod_set_nsec
//
// purpose	Set the interval timer to signal the alarm at expiration based
//		on a fixed repeating modulo time interval and an offset within
//		the repeating interval, both with 64 bit integer values.  If
//		the requested offset is greater than the offset of the current
//		time within the current cycle, the expiration will be in the
//		current cycle.
//
// arguments	1 (timeus_t) repeating modulus in microseconds
//		2 (timeus_t) offset in modulus in microseconds
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define timer_mod_set_sec(m,o) timer_mod_set_usec((timeus_t)(m)*1000000UL,(timeus_t)(o)*1000000UL)
#define timer_mod_set_msec(m,o) timer_mod_set_usec((timeus_t)(m)*1000UL,(timeus_t)(p)*1000UL)
#define timer_mod_set_nsec(m,o) timer_mod_set_usec(((timeus_t)(m)+999UL)/1000UL,((timeus_t)(o)+999UL)/1000UL)
int
timer_mod_set_usec (
    timeus_t	arg_modulo
    ,
    timeus_t	arg_offset
    )
;

//-----------------------------------------------------------------------------
// function	timeus_to_timespec
//
// purpose	Convert system time in microseconds to struct timespec.
//
// arguments	1 (struct timespec *) pointer to destination struct timespec
//		2 (timeus_t) system time in microseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
void
timeus_to_timespec (
    struct timespec *	arg_timets_p
    ,
    timeus_t		arg_time_usec
    )
;

//-----------------------------------------------------------------------------
// function	timeus_to_timeval
//
// purpose	Convert system time in microseconds to struct timeval.
//
// arguments	1 (struct timeval *) pointer to destination struct timeval
//		2 (timeus_t) system time in microseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
void
timeus_to_timeval (
    struct timeval *	arg_timetv_p
    ,
    timeus_t		arg_time_usec
    )
;

//-----------------------------------------------------------------------------
// function	ymd_gregorian_to_eday
//
// purpose	Convert a Gregorian calendar date to an Earth Day value.
//
// arguments	1 (long) Gregorian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Gregorian calendar month (1-12)
//		3 (int) Gregorian calendar date (1-31)
//
// returns	(eday_t) resultant Earth Day value
//
// note		The calculation is for the proleptic Gregorian calendar.
//		There is no accomodation for selection between Gregorian
//		and Julian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
eday_t
ymd_gregorian_to_eday (
    long	arg_year
    ,
    int		arg_month
    ,
    int		arg_date
    )
;

//-----------------------------------------------------------------------------
// function	ymd_julian_to_eday
//
// purpose	Convert a Julian calendar date to an Earth Day value.
//
// arguments	1 (long) Julian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Julian calendar month (1-12)
//		3 (int) Julian calendar date (1-31)
//
// returns	(eday_t) resultant Earth Day value
//
// note		The calculation is for the proleptic Julian calendar.
//		There is no accomodation for selection between Julian and
//		Gregorian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
eday_t
ymd_julian_to_eday (
    long	arg_year
    ,
    int		arg_month
    ,
    int		arg_date
    )
;

#endif /* __TIME_H__ */
